Type: Package
Package: LikertMakeR
Title: Synthesise and Correlate Likert Scale and Rating-Scale Data
        Based on Summary Statistics
Version: 2.3.0
Date: 2026-04-28
Authors@R: 
    person("Hume", "Winzar", , "winzar@gmail.com", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-7475-2641"))
Description: Generate and correlate synthetic Likert and rating-scale
    questionnaire responses with predefined means, standard deviations, 
    Cronbach's Alpha, Factor Loading table, coefficients, and other summary 
    statistics. 
    It can be used to simulate Likert data, construct multi-item scales, 
    generate correlation matrices, and create example survey datasets for 
    teaching statistics, psychometrics, and methodological research.
    Worked examples and documentation are available in the package 
    articles, accessible via the package website,
    <https://winzarh.github.io/LikertMakeR/>. 
License: MIT + file LICENSE
URL: https://github.com/WinzarH/LikertMakeR/,
        https://winzarh.github.io/LikertMakeR/,
        https://github.com/WinzarH/LikertMakeR
BugReports: https://github.com/WinzarH/LikertMakeR/issues
Depends: R (>= 4.2.0)
Imports: dplyr, gtools, Matrix, matrixStats, Rcpp, stats, tibble,
        utils,
Suggests: effectsize, kableExtra, knitr, ggplot2, ggrepel, psych,
        polycor, psychTools, rmarkdown, testthat (>= 3.0.0), tidyr
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Config/Needs/website: dplyr, effectsize, ggplot2, kableExtra, lavaan,
        matrixcalc, matrixStats, pkgdown, psych, quarto, rmarkdown,
        semPlot, semptools, stringr, tibble, tidyr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-21 03:35:24 UTC; winza
Author: Hume Winzar [cre, aut] (ORCID: <https://orcid.org/0000-0001-7475-2641>)
Maintainer: Hume Winzar <winzar@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 04:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-25 04:01:10 UTC; unix
Archs: LikertMakeR.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(print,likert_reliability)
S3method(print,makeScalesRegression)
export(alpha)
export(alpha_sensitivity)
export(correlateScales)
export(eigenvalues)
export(lcor)
export(lexact)
export(lfast)
export(makeCorrAlpha)
export(makeCorrLoadings)
export(makeItemsScale)
export(makePaired)
export(makeRepeated)
export(makeScales)
export(makeScalesRegression)
export(ordinal_diagnostics)
export(reliability)
importFrom(Matrix,nearPD)
importFrom(Rcpp,sourceCpp)
importFrom(dplyr,bind_cols)
importFrom(graphics,abline)
importFrom(graphics,rect)
importFrom(gtools,combinations)
importFrom(matrixStats,colVars)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,optimize)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(LikertMakeR)

<!-- badges: start -->
[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![metacran downloads total](https://cranlogs.r-pkg.org/badges/grand-total/LikertMakeR)](https://cran.r-project.org/package=LikertMakeR)
[![metacran downloads last month](https://cranlogs.r-pkg.org/badges/last-month/LikertMakeR)](https://cran.r-project.org/package=LikertMakeR)
[![R-CMD-check](https://github.com/WinzarH/LikertMakeR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/WinzarH/LikertMakeR/actions/workflows/R-CMD-check.yaml)
[![pkgdown](https://github.com/WinzarH/LikertMakeR/actions/workflows/pkgdown.yml/badge.svg?branch=main)](https://winzarh.github.io/LikertMakeR/)
[![GitHub stars](https://img.shields.io/github/stars/WinzarH/LikertMakeR.svg?style=social&label=Star)](https://github.com/WinzarH/LikertMakeR)
<!-- badges: end -->

# LikertMakeR <img src="man/figures/logo.png" align="right" height="134" alt="LikertMakeR" />

**LikertMakeR** synthesises Likert-scale and related bounded rating-scale data 
with predefined *means*, *standard deviations*, *correlations*,
*Cronbach’s alpha*, and *factor-loading-based structure* for simulation 
studies, teaching, and methodological research..

Version 2.1.1 (April 2026)

## Purpose

1. *Reverse-engineer* published results when only summary statistics are reported (for re-analysis, visualisation, or teaching).
2. *Teaching & demos*: generate data with known properties without collecting real data.
3. *Methods work / simulation*: explore how reliability, items, bounds, and sample size interact.

For a full introduction and worked examples, see the package website:
<https://winzarh.github.io/LikertMakeR/>

---

## Installation

From **_CRAN_**:

```r

  install.packages("LikertMakeR")
```

The latest development version is available from the 
author's **_GitHub_** repository.
     
```r  

 library(devtools)
 
 install_github("WinzarH/LikertMakeR")
```     
     
## Quick Start

1. Make a target correlation matrix

```r
library(LikertMakeR)

R <- matrix(
  c(
    1.00, 0.60, 0.45, 0.40,
    0.60, 1.00, 0.75, 0.35,
    0.45, 0.75, 1.00, 0.50,
    0.40, 0.35, 0.50, 1.00
  ),
  nrow = 4, ncol = 4, byrow = TRUE
)
```    

2. Generate synthetic rating-scale data with predefined moments

```r
dat <- makeScales(
  n = 64,
  means = c(2.75, 3.00, 3.25, 3.50),
  sds = c(1.25, 1.50, 1.30, 1.25),
  lowerbound = rep(1, 4),
  upperbound = rep(5, 4),
  items = c(5, 5, 4, 4),
  cormatrix = R
)

head(dat)
cor(dat) |> round(2)
```    

## Key functions

  - `lfast()`: generate bounded/discrete data with target mean & SD

  - `lcor()`: rearrange columns to approximate a target correlation matrix

  - `makeCorrAlpha()`: generate an item correlation matrix with target Cronbach’s alpha

  - `makeScales()`: wrapper for lfast() + lcor() to generate a dataframe of correlated columns

  - `makeCorrLoadings()`: build an item correlation matrix from factor loadings and factor correlations

  - `makeItemsScale()`: generate items from a summated scale with target alpha

  - `makePaired()` / `makeRepeated()`: reconstruct data from paired t-test / repeated-measures summaries

  - `makeScalesRegression()`: generate data from summary of multiple-regression analysis

  - `correlateScales()`: combine multiple item sets so summated scales match a target correlation matrix

  - Helpers: `alpha()`, `eigenvalues()`, `reliability()`

  
## Rating scale properties

A Likert scale is the mean, or sum, of several ordinal rating scales. 
They are bipolar (usually “agree-disagree”) responses to propositions 
that are determined to be moderately-to-highly correlated among each other, 
and capturing various facets of a theoretical construct.

> ### NOTE   
> A single 1-5 rating scale is **_NOT_** a true Likert scale - it may be a Likert-scale item.
    
Summated rating scales are not continuous or unbounded.
For example, a 5-point Likert scale that is constructed with, say, 
five items (questions) will have a summed range of between 5 
(all rated ‘1’) and 25 (all rated ‘5’) with all integers in between, 
and the mean range will be ‘1’ to ‘5’ with intervals of 1/5=0.20.
A 7-point Likert scale constructed from eight items will have a 
summed range between 8 (all rated ‘1’) and 56 (all rated ‘7’) with 
all integers in between, and the mean range will be ‘1’ to ‘7’ with 
intervals of 1/8=0.125.


## Learn more

Package website (recommended): https://winzarh.github.io/LikertMakeR/

Vignettes cover:

  - generating scales from summary statistics,

  - correlation matrices from alpha or loadings,

  - repeated-measures and paired designs,

  - reliability estimation and diagnostics,
  
  - validation studies demonstrating function accuracy. 
  
____


### To cite _LikertMakeR_

#### APA:

     Winzar, H. (2026). LikertMakeR (version 2.0.0) [R package]. 
     The Comprehensive R Archive Network (CRAN),
    <https://CRAN.R-project.org/package=LikertMakeR>
        

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, echo=FALSE--------------------------------------------------------
library(LikertMakeR)

## ----logo, fig.align='center', echo=FALSE, out.width = '20%'------------------
knitr::include_graphics("img/LikertMakeR_3.png")

## ----skew, fig.align='center', echo=FALSE, out.width = '85%', fig.cap="Off-centre means always give skewed distribution in bounded rating scales"----
knitr::include_graphics("img/skew_chart.png")

## ----lfastExample-------------------------------------------------------------
nItems <- 4
mean <- 2.5
sd <- 0.75

x1 <- lfast(
  n = 512,
  mean = mean,
  sd = sd,
  lowerbound = 1,
  upperbound = 5,
  items = nItems
)

## ----fig1, fig.height=4, fig.width=5, fig.align='center', echo = FALSE, crop = TRUE, fig.cap="Example: 4-item, 1-5 Likert scale"----
## distribution of x

hist(x1,
  cex.axis = 0.5, cex.main = 0.75,
  breaks = seq(from = (1 - (1 / 8)), to = (5 + (1 / 8)), by = (1 / 4)),
  col = "skyblue", xlab = NULL, ylab = NULL,
  main = paste0("mu=", round(mean(x1), 2), ", sd=", round(sd(x1), 2))
)

## ----lfastx2------------------------------------------------------------------
x2 <- lfast(256, 3, 2.5, 0, 10)

## ----fig2, fig.height=4, fig.width=5, fig.align='center', echo = FALSE, crop = TRUE, fig.cap="Example: likelihood-of-purchase scale"----
## generate histogram

hist(x2,
  cex.axis = 0.5, cex.main = 0.75,
  breaks = seq(from = -0.5, to = 10.5, by = 1),
  col = "skyblue", xlab = NULL, ylab = NULL,
  main = paste0("mu=", round(mean(x2), 2), ", sd=", round(sd(x2), 2))
)

## ----lcorExample--------------------------------------------------------------
## generate uncorrelated synthetic data
n <- 128
lowerbound <- 1
upperbound <- 5
items <- 5

mydat3 <- data.frame(
  x1 = lfast(n, 2.5, 0.75, lowerbound, upperbound, items),
  x2 = lfast(n, 3.0, 1.50, lowerbound, upperbound, items),
  x3 = lfast(n, 3.5, 1.00, lowerbound, upperbound, items)
)

## ----mydat3Head, echo = FALSE-------------------------------------------------
head(mydat3, 6)

## ----mydat3Moments, echo = FALSE----------------------------------------------
moments <- data.frame(
  mean = apply(mydat3, 2, mean) |> round(3),
  sd = apply(mydat3, 2, sd) |> round(3)
) |> t()

moments

## ----mydat3Cor, echo = FALSE--------------------------------------------------
cor(mydat3) |> round(2)

## ----tgt3---------------------------------------------------------------------
## describe a target correlation matrix
tgt3 <- matrix(
  c(
    1.00, 0.85, 0.75,
    0.85, 1.00, 0.65,
    0.75, 0.65, 1.00
  ),
  nrow = 3
)

## ----new3---------------------------------------------------------------------
## apply lcor() function
new3 <- lcor(data = mydat3, target = tgt3)

## ----new3Head, echo = FALSE---------------------------------------------------
head(new3, 10)

## ----new3Cor, echo = FALSE----------------------------------------------------
cor(new3) |> round(3)

## ----cor_matrix_4-------------------------------------------------------------
## define parameters
items <- 4
alpha <- 0.85

## apply makeCorrAlpha() function
set.seed(42)

cor_matrix_4 <- makeCorrAlpha(items, alpha)

## ----cor_matrix_4Print, echo = FALSE------------------------------------------
cor_matrix_4 |> round(3)

## ----cor_matrix_4Alpha--------------------------------------------------------
## using helper function alpha()

alpha(cor_matrix_4)

## ----fig3, fig.height=4, fig.width=5, fig.align='center', echo = TRUE, crop = TRUE----
## using helper function eigenvalues()

eigenvalues(cor_matrix_4, 1)

## ----cor_matrix_d-------------------------------------------------------------
## apply makeCorrAlpha() with diagnostics
set.seed(42)

cor_matrix_5 <- makeCorrAlpha(
  items = 6,
  alpha = 0.90,
  diagnostics = TRUE
)

## ----cor_matrix_5_mat---------------------------------------------------------
## output
cor_matrix_5$R |> round(2)

cor_matrix_5$diagnostics

## ----parameters_makeCorrLoadings, echo=TRUE-----------------------------------
## Example loadings

factorLoadings <- matrix(
  c(
    0.05, 0.20, 0.70,
    0.10, 0.05, 0.80,
    0.05, 0.15, 0.85,
    0.20, 0.85, 0.15,
    0.05, 0.85, 0.10,
    0.10, 0.90, 0.05,
    0.90, 0.15, 0.05,
    0.80, 0.10, 0.10
  ),
  nrow = 8, ncol = 3, byrow = TRUE
)

## row and column names

rownames(factorLoadings) <- c("Q1", "Q2", "Q3", "Q4", "Q5", "Q6", "Q7", "Q8")
colnames(factorLoadings) <- c("Factor1", "Factor2", "Factor3")

## Factor correlation matrix**

factorCor <- matrix(
  c(
    1.0,  0.5, 0.4,
    0.5,  1.0, 0.3,
    0.4,  0.3, 1.0
  ),
  nrow = 3, byrow = TRUE
)

## ----apply_makeCorrLoadings, echo=TRUE----------------------------------------
## apply makeCorrLoadings() function
itemCorrelations <- makeCorrLoadings(factorLoadings, factorCor)

## derived correlation matrix to two decimal places
round(itemCorrelations, 2)

## ----test_makeCorrLoadings, echo=TRUE-----------------------------------------
## correlated factors mean that eigenvalues should suggest two or three factors
eigenvalues(cormatrix = itemCorrelations, scree = TRUE)

## ----orthogonal_makeCorrLoadings, echo=TRUE-----------------------------------
## orthogonal factors are assumed when factor correlation matrix is not included
orthogonalItemCors <- makeCorrLoadings(factorLoadings)

## derived correlation matrix to two decimal places
round(orthogonalItemCors, 2)

## ----test_orthogonal, echo=TRUE-----------------------------------------------
## eigenvalues should suggest exactly  three factors
eigenvalues(cormatrix = orthogonalItemCors, scree = TRUE)

## ----makeScalesExample1-------------------------------------------------------
## define parameters
n <- 128
dfMeans <- c(2.5, 3.0, 3.0, 3.5)
dfSds <- c(1.0, 1.0, 1.5, 0.75)
lowerbound <- rep(1, 4)
upperbound <- rep(5, 4)

corMat <- matrix(
  c(
    1.00, 0.25, 0.35, 0.45,
    0.25, 1.00, 0.70, 0.75,
    0.35, 0.70, 1.00, 0.85,
    0.45, 0.75, 0.85, 1.00
  ),
  nrow = 4, ncol = 4
)

var_names <- c("var1", "var2", "var3", "var4")
colnames(corMat) <- var_names
rownames(corMat) <- var_names

## apply makeScales() function
df <- makeScales(
  n = n,
  means = dfMeans,
  sds = dfSds,
  lowerbound = lowerbound,
  upperbound = upperbound,
  cormatrix = corMat
)

## ----str_makeScalesExample1---------------------------------------------------
## test the function
str(df)

## ----moments_makeScalesExample1-----------------------------------------------
### means should be correct to two decimal places
dfmoments <- data.frame(
  mean = apply(df, 2, mean) |> round(3),
  sd = apply(df, 2, sd) |> round(3)
) |> t()

dfmoments

## ----cors_makeScalesExample1--------------------------------------------------
### correlations should be correct to two decimal places
cor(df) |> round(3)

## ----makeScalesExample2-------------------------------------------------------
## define parameters
n <- 256
dfMeans <- c(3.9, 4.1, 3.6, 4.0)
dfSds <- c(0.6, 0.5, 0.8, 0.7)
lowerbound <- rep(1, 4)
upperbound <- rep(5, 4)
items <- c(4, 3, 4, 3)

corMat <- matrix(
  c(
    1.00, 0.75, 0.60, 0.70,
    0.75, 1.00, 0.65, 0.72,
    0.60, 0.65, 1.00, 0.68,
    0.70, 0.72, 0.68, 1.00
  ),
  nrow = 4, ncol = 4
)

scale_names <- c("BT", "BS", "BL", "BLY")
rownames(corMat) <- scale_names
colnames(corMat) <- scale_names

## apply makeScales() function
df <- makeScales(
  n = n,
  means = dfMeans,
  sds = dfSds,
  lowerbound = lowerbound,
  upperbound = upperbound,
  items = items,
  cormatrix = corMat
)

## test the function
head(df)
tail(df)

### means should be correct to two decimal places
dfmoments <- data.frame(
  mean = apply(df, 2, mean) |> round(3),
  sd = apply(df, 2, sd) |> round(3)
) |> t()

dfmoments

### correlations should be correct to two decimal places
cor(df) |> round(3)

## ----genCorrelation-----------------------------------------------------------
## define parameters
k <- 6
myAlpha <- 0.85

## generate correlation matrix
set.seed(42)
myCorr <- makeCorrAlpha(items = k, alpha = myAlpha)

## display correlation matrix
myCorr |> round(3)

### checking Cronbach's Alpha
alpha(cormatrix = myCorr)

## ----gendataframe-------------------------------------------------------------
## define parameters
n <- 256
myMeans <- c(2.75, 3.00, 3.00, 3.25, 3.50, 3.5)
mySds <- c(1.00, 0.75, 1.00, 1.00, 1.00, 1.5)
lowerbound <- rep(1, k)
upperbound <- rep(5, k)

## Generate Items
myItems <- makeScales(
  n = n, means = myMeans, sds = mySds,
  lowerbound = lowerbound, upperbound = upperbound,
  items = 1,
  cormatrix = myCorr
)

## resulting dataframe
head(myItems)
tail(myItems)

## means and standard deviations
myMoments <- data.frame(
  means = apply(myItems, 2, mean) |> round(3),
  sds = apply(myItems, 2, sd) |> round(3)
) |> t()
myMoments

## Cronbach's Alpha of dataframe
alpha(NULL, myItems)

## ----fig5, fig.height=5, fig.width=5, fig.align='center', echo=FALSE, warning=FALSE, crop = TRUE, fig.cap="Summary of dataframe from makeScales() function"----
# Correlation panel

panel.cor <- function(x, y) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))
  r <- round(cor(x, y), digits = 2)
  txt <- paste0(r)
  cex.cor <- 0.8 / strwidth(txt)
  text(0.5, 0.5, txt, cex = 1.25)
}

# Customize upper panel

upper.panel <- function(x, y) {
  points(x, y, pch = 19, col = "#0000ff11")
}

# diagonals

panel.hist <- function(x, ...) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(usr[1:2], 0, 1.5))
  h <- hist(x, plot = FALSE)
  breaks <- h$breaks
  nB <- length(breaks)
  y <- h$counts
  y <- y / max(y)
  rect(breaks[-nB], 0, breaks[-1], y, col = "#87ceeb66")
}

# Create the plots

pairs(myItems,
  lower.panel = panel.cor,
  upper.panel = upper.panel,
  diag.panel = panel.hist
)

## ----makeItemsScale_example_1, fig.height=4, fig.width=5, fig.align='center', crop = TRUE----
summatedScale <- c(11, 8, 14, 12, 9, 12, 10, 9, 8, 18, 16, 16, 14, 8, 15, 12)

## apply makeItemsScale() function

newItems <- makeItemsScale(
  scale = summatedScale,
  lowerbound = 1,
  upperbound = 5,
  items = 4
)

###  observations and summated scale
head(cbind(newItems, summatedScale), 16)

### correlation matrix
cor(newItems) |> round(2)

### default Cronbach's alpha = 0.80
alpha(data = newItems) |> round(4)

### calculate eigenvalues and print scree plot
eigenvalues(cor(newItems), 1) |> round(3)

## ----makeItemsScale_example_2, fig.height=4, fig.width=5, fig.align='center', crop = TRUE----
## apply makeItemsScale() function
newItems_2 <- makeItemsScale(
  scale = summatedScale,
  lowerbound = 1,
  upperbound = 5,
  items = 4,
  alpha = 0.9
)

### observations and summated scale
head(cbind(newItems_2, summatedScale), 16)

### correlation matrix
cor(newItems_2) |> round(2)

### requested Cronbach's alpha = 0.90
alpha(data = newItems_2) |> round(4)

### calculate eigenvalues and print scree plot
eigenvalues(cor(newItems_2), 1) |> round(3)

## ----independent-samples_t-test, echo = TRUE----------------------------------
## define parameters
lower <- 1
upper <- 5
items <- 6

## generate two independent samples
x1 <- lfast(
  n = 20, mean = 2.5, sd = 0.75,
  lowerbound = lower, upperbound = upper, items = items
)
x2 <- lfast(
  n = 30, mean = 3.0, sd = 0.85,
  lowerbound = lower, upperbound = upper, items = items
)

## run independent-samples t-test
t.test(x1, x2)

## ----paired-sample_t-test_data, , echo = TRUE---------------------------------
## define parameters
n <- 20
means <- c(2.5, 3.0)
sds <- c(0.75, 0.85)
lower <- 1
upper <- 5
items <- 6
t <- -2.5

## run the function
pairedDat <- makePaired(
  n = n, means = means, sds = sds,
  t_value = t,
  lowerbound = lower, upperbound = upper, items = items
)

## ----makePaired_output_properties, echo=TRUE----------------------------------
## test function output
str(pairedDat)

cor(pairedDat) |> round(2)

pairedMoments <- data.frame(
  mean = apply(pairedDat, MARGIN = 2, FUN = mean) |> round(3),
  sd = apply(pairedDat, MARGIN = 2, FUN = sd) |> round(3)
) |> t()

pairedMoments

## ----paired-sample_t-test, echo=TRUE------------------------------------------
## run a paired-sample t-test

paired_t <- t.test(x = pairedDat$X1, y = pairedDat$X2, paired = TRUE)

# paired_t <- t.test(pairedDat$X1, pairedDat$X2, paired = TRUE)


paired_t

## ----makeRepeated-test, echo=TRUE---------------------------------------------
out1 <- makeRepeated(
  n = 128,
  k = 3,
  means = c(3.1, 3.5, 3.9),
  sds = c(1.0, 1.1, 1.0),
  items = 4,
  f_stat = 4.87,
  structure = "cs",
  diagnostics = FALSE
)

head(out1$data)
out1$correlation_matrix


out2 <- makeRepeated(
  n = 32, k = 4,
  means = c(2.75, 3.5, 4.0, 4.4),
  sds = c(0.8, 1.0, 1.2, 1.0),
  f_stat = 16,
  structure = "ar1",
  items = 5,
  lowerbound = 1, upperbound = 7,
  return_corr_only = FALSE,
  diagnostics = TRUE
)

print(out2)


out3 <- makeRepeated(
  n = 32, k = 4,
  means = c(2.0, 2.5, 3.0, 2.8),
  sds = c(0.8, 0.9, 1.0, 0.9),
  items = 4,
  f_stat = 24,
  structure = "toeplitz",
  diagnostics = TRUE
)

str(out3)

## ----correlateScales_dataframes, echo = TRUE----------------------------------
n <- 128
lower <- 1
upper <- 5

### attitude #1

#### generate a correlation matrix
cor_1 <- makeCorrAlpha(items = 4, alpha = 0.80)

#### specify moments as vectors
means_1 <- c(2.5, 2.5, 3.0, 3.5)
sds_1 <- c(0.75, 0.85, 0.85, 0.75)

#### apply makeScales() function
Att_1 <- makeScales(
  n = n, means = means_1, sds = sds_1,
  lowerbound = rep(lower, 4), upperbound = rep(upper, 4),
  items = 1,
  cormatrix = cor_1
)

### attitude #2

#### generate a correlation matrix
cor_2 <- makeCorrAlpha(items = 5, alpha = 0.85)

#### specify moments as vectors
means_2 <- c(2.5, 2.5, 3.0, 3.0, 3.5)
sds_2 <- c(0.75, 0.85, 0.75, 0.85, 0.75)

#### apply makeScales() function
Att_2 <- makeScales(
  n, means_2, sds_2,
  rep(lower, 5), rep(upper, 5),
  items = 1,
  cor_2
)

### attitude #3

#### generate a correlation matrix
cor_3 <- makeCorrAlpha(items = 6, alpha = 0.90)

#### specify moments as vectors
means_3 <- c(2.5, 2.5, 3.0, 3.0, 3.5, 3.5)
sds_3 <- c(0.75, 0.85, 0.85, 1.0, 0.75, 0.85)

#### apply makeScales() function
Att_3 <- makeScales(
  n, means_3, sds_3,
  rep(lower, 6), rep(upper, 6),
  items = 1,
  cor_3
)

### behavioural intention
intent <- lfast(n, mean = 4.0, sd = 3, lowerbound = 0, upperbound = 10) |>
  data.frame()
names(intent) <- "int"

## ----dataframe_properties-----------------------------------------------------
## Attitude #1
A1_moments <- data.frame(
  means = apply(Att_1, 2, mean) |> round(2),
  sds = apply(Att_1, 2, sd) |> round(2)
) |> t()

### Attitude #1 moments
A1_moments

### Attitude #1 correlations
cor(Att_1) |> round(2)

### Attitude #1 cronbach's alpha
alpha(cor(Att_1)) |> round(3)

## Attitude #2
A2_moments <- data.frame(
  means = apply(Att_2, 2, mean) |> round(2),
  sds = apply(Att_2, 2, sd) |> round(2)
) |> t()

### Attitude #2 moments
A2_moments

### Attitude #2 correlations
cor(Att_2) |> round(2)

### Attitude #2 cronbach's alpha
alpha(cor(Att_2)) |> round(3)

## Attitude #3
A3_moments <- data.frame(
  means = apply(Att_3, 2, mean) |> round(2),
  sds = apply(Att_3, 2, sd) |> round(2)
) |> t()

### Attitude #3 moments
A3_moments

### Attitude #3 correlations
cor(Att_3) |> round(2)

### Attitude #2 cronbach's alpha
alpha(cor(Att_3)) |> round(3)


## Behavioural Intention

intent_moments <- data.frame(
  mean = apply(intent, 2, mean) |> round(3),
  sd = apply(intent, 2, sd) |> round(3)
) |> t()

### Intention moments
intent_moments

## ----correlateScales_parameters-----------------------------------------------
### target scale correlation matrix
scale_cors <- matrix(
  c(
    1.0, 0.7, 0.6, 0.5,
    0.7, 1.0, 0.4, 0.3,
    0.6, 0.4, 1.0, 0.2,
    0.5, 0.3, 0.2, 1.0
  ),
  nrow = 4
)

### bring dataframes into a list
data_frames <- list("A1" = Att_1, "A2" = Att_2, "A3" = Att_3, "Int" = intent)

## ----my_correlated_scales-----------------------------------------------------
### apply correlateScales() function
my_correlated_scales <- correlateScales(
  dataframes = data_frames,
  scalecors = scale_cors
)

## ----fig6, fig.height=9, fig.width=9, fig.align='center', echo=FALSE, warning=FALSE,  crop = TRUE----
# Correlation panel
panel.cor <- function(x, y) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))
  r <- round(cor(x, y), digits = 2)
  txt <- paste0(r)
  cex.cor <- 0.8 / strwidth(txt)
  text(0.5, 0.5, txt, cex = 1.25)
}
# Customize upper panel
upper.panel <- function(x, y) {
  points(x, y, pch = 19, col = "#0000ff11")
}
# diagonals
panel.hist <- function(x, ...) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(usr[1:2], 0, 1.5))
  h <- hist(x, plot = FALSE)
  breaks <- h$breaks
  nB <- length(breaks)
  y <- h$counts
  y <- y / max(y)
  rect(breaks[-nB], 0, breaks[-1], y, col = "#0000ff50")
}
# Create the plots
pairs(my_correlated_scales,
  lower.panel = panel.cor,
  upper.panel = upper.panel,
  diag.panel = panel.hist
)

## ----newdata_check------------------------------------------------------------
## data structure
str(my_correlated_scales)

## ----fig7, fig.height=4, fig.width=5, fig.align='center', echo = TRUE,  crop = TRUE----
## eigenvalues of dataframe correlations
Cor_Correlated_Scales <- cor(my_correlated_scales)
eigenvalues(cormatrix = Cor_Correlated_Scales, scree = TRUE) |> round(2)

## ----fig7a, fig.height=4, fig.width=5, fig.align='center', echo = TRUE,  crop = TRUE----
#### Eigenvalues of predictor variable items only
Cor_Attitude_items <- cor(my_correlated_scales[, -16])
eigenvalues(cormatrix = Cor_Attitude_items, scree = TRUE) |> round(2)

## ----alphaExample-------------------------------------------------------------
## define parameters
df <- data.frame(
  V1 = c(4, 2, 4, 3, 2, 2, 2, 1),
  V2 = c(3, 1, 3, 4, 4, 3, 2, 3),
  V3 = c(4, 1, 3, 5, 4, 1, 4, 2),
  V4 = c(4, 3, 4, 5, 3, 3, 3, 3)
)

corMat <- matrix(
  c(
    1.00, 0.35, 0.45, 0.75,
    0.35, 1.00, 0.65, 0.55,
    0.45, 0.65, 1.00, 0.65,
    0.75, 0.55, 0.65, 1.00
  ),
  nrow = 4, ncol = 4
)

## apply function examples
alpha(cormatrix = corMat)
alpha(data = df)
alpha(NULL, df)
alpha(corMat, df)

## ----eigenExample-------------------------------------------------------------
## define parameters
correlationMatrix <- matrix(
  c(
    1.00, 0.25, 0.35, 0.45,
    0.25, 1.00, 0.70, 0.75,
    0.35, 0.70, 1.00, 0.85,
    0.45, 0.75, 0.85, 1.00
  ),
  nrow = 4, ncol = 4
)

## apply function
evals <- eigenvalues(cormatrix = correlationMatrix)

print(evals)

## ----fig8, fig.height=4, fig.width=5, fig.align='center', echo = TRUE, crop = TRUE----
evals <- eigenvalues(correlationMatrix, 1)
print(evals)

## ----reliabilityExample, warning=FALSE, message=FALSE-------------------------
## create dataset
my_cor <- LikertMakeR::makeCorrAlpha(
  items = 4,
  alpha = 0.80
)

my_data <- LikertMakeR::makeScales(
  n = 64,
  means = c(2.75, 3.00, 3.25, 3.50),
  sds = c(1.25, 1.50, 1.30, 1.25),
  lowerbound = rep(1, 4),
  upperbound = rep(5, 4),
  cormatrix = my_cor
)

## run function
reliability(my_data)

reliability(
  my_data,
  include = c("lambda6", "polychoric")
)


## bootstrapped Confidence intervals can be slow!
reliability(
  my_data,
  include = "polychoric",
  ci = TRUE,
  n_boot = 64
)

## ----eval = FALSE-------------------------------------------------------------
# n <- 128
# sample(1:5, n,
#   replace = TRUE,
#   prob = c(0.1, 0.2, 0.4, 0.2, 0.1)
# )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup-packages, message=FALSE, warning=FALSE-----------------------------
# Load required packages only if installed
if (requireNamespace("rosetta", quietly = TRUE)) library(rosetta)
if (requireNamespace("dplyr", quietly = TRUE)) library(dplyr)
if (requireNamespace("psych", quietly = TRUE)) library(psych)
if (requireNamespace("psychTools", quietly = TRUE)) library(psychTools)
if (requireNamespace("knitr", quietly = TRUE)) library(knitr)
if (requireNamespace("kableExtra", quietly = TRUE)) library(kableExtra)

library(LikertMakeR)

## -----------------------------------------------------------------------------
itemQuestions <- c(
  "Expectation that a high dose results in a longer trip",
  "Expectation that a high dose results in a more intense trip",
  "Expectation that a high dose makes you more intoxicated",
  "Expectation that a high dose provides more energy",
  "Expectation that a high dose produces more euphoria",
  "Expectation that a high dose yields more insight",
  "Expectation that a high dose strengthens your connection with others",
  "Expectation that a high dose facilitates making contact with others",
  "Expectation that a high dose improves sex"
)

itemLabels <- c(
  "long",
  "intensity",
  "intoxicated",
  "energy",
  "euphoria",
  "insight",
  "connection",
  "contact",
  "sex"
)

labels <- data.frame(
  Questions = itemQuestions,
  Labels = itemLabels
)

kable(labels) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
## variable names
item_list <- c(
  "highDose_AttBeliefs_long",
  "highDose_AttBeliefs_intensity",
  "highDose_AttBeliefs_intoxicated",
  "highDose_AttBeliefs_energy",
  "highDose_AttBeliefs_euphoria",
  "highDose_AttBeliefs_insight",
  "highDose_AttBeliefs_connection",
  "highDose_AttBeliefs_contact",
  "highDose_AttBeliefs_sex"
)

## read the data/ select desired variables/ remove obs with missing values
dat <- read.csv2(file = "data/pp15.csv") |>
  select(all_of(item_list)) |>
  na.omit()

## give variables shorter names
names(dat) <- itemLabels

sampleSize <- nrow(dat)

## -----------------------------------------------------------------------------
## correlation matrix
pp15_cor <- cor(dat)

## -----------------------------------------------------------------------------
kable(pp15_cor, digits = 2) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
## factor analysis from `rosetta` package
rfaDose <- rosetta::factorAnalysis(
  data = dat,
  nfactors = 2,
  rotate = "promax"
)

factorLoadings <- rfaDose$output$loadings
factorCorrs <- rfaDose$output$correlations

## -----------------------------------------------------------------------------
kable(factorLoadings, digits = 2) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
kable(factorCorrs, digits = 2) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
## round input values to 5 decimal places
# factor loadings
fl1 <- factorLoadings[, 1:2] |>
  round(5) |>
  as.matrix()
# item uniquenesses
un1 <- factorLoadings[, 3] |> round(5)
# factor correlations
fc1 <- round(factorCorrs, 5) |> as.matrix()
# run makeCorrLoadings() function
itemCors_1 <- makeCorrLoadings(
  loadings = fl1,
  factorCor = fc1,
  uniquenesses = un1
)
## Compare the two matrices
chiSq_1 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_1,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## round input values to 2 decimal places
# factor loadings
fl2 <- factorLoadings[, 1:2] |>
  round(5) |>
  as.matrix()
# factor correlations
fc2 <- factorCorrs |>
  round(5) |>
  as.matrix()
itemCors_2 <- makeCorrLoadings(
  loadings = fl2,
  factorCor = fc2,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_2 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_2,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## round input values to 2 decimal places
# factor loadings
fl3 <- factorLoadings[, 1:2] |>
  round(2) |>
  as.matrix()
# item uniquenesses
un3 <- factorLoadings[, 3] |>
  round(2)
## factor correlations
fc3 <- factorCorrs |>
  round(2) |>
  as.matrix()
## Compare the two matrices
itemCors_3 <- makeCorrLoadings(
  loadings = fl3,
  factorCor = fc3,
  uniquenesses = un3
)
## Compare the two matrices
chiSq_3 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_3,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## round input values to 2 decimal places
# factor loadings
fl4 <- factorLoadings[, 1:2] |>
  round(2) |>
  as.matrix()
## factor correlations
fc4 <- factorCorrs |>
  round(2) |>
  as.matrix()
# apply the function
itemCors_4 <- makeCorrLoadings(
  loadings = fl4,
  factorCor = fc4,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_4 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_4,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
fl_0 <- factorLoadings[, 1:2] |>
  round(2)
fl_a <- fl_0
# convert factor loadings < '0.1' to '0'
censor_a <- 0.1
fl_a[abs(fl_a) < censor_a] <- 0
fl_a_mean <- mean(fl_a == 0, na.rm = TRUE) |> round(2)
fl_a[fl_a == 0] <- " "
fl_b <- fl_0
# convert factor loadings < '0.2' to '0'
censor_b <- 0.2
fl_b[abs(fl_b) < censor_b] <- 0
fl_b_mean <- mean(fl_b == 0, na.rm = TRUE) |> round(2)
fl_b[fl_b == 0] <- " "
fl_c <- fl_0
# convert factor loadings < '0.3' to '0'
censor_c <- 0.3
fl_c[abs(fl_c) < censor_c] <- 0
fl_c_mean <- mean(fl_c == 0, na.rm = TRUE) |> round(2)
fl_c[fl_c == 0] <- " "
#  bring factor loadings together
fl <- cbind(fl_0, fl_a, fl_b, fl_c)
colnames(fl) <- c("f1", "f2", "f1", "f2", "f1", "f2", "f1", "f2")
header_text <- c("Item" = 1, "all values" = 2, "< 0.1 out" = 2, "< 0.2 out" = 2, "<0.3 out" = 2)
# print summary factor loadings
kable(fl, digits = 2, align = rep("c", 8)) |>
  column_spec(1:9, border_left = T, border_right = T) |>
  kable_styling() |>
  add_header_above(
    header = header_text,
    align = "c"
  )

## -----------------------------------------------------------------------------
## round input values to 2 decimal places
# factor loadings
fl5a <- factorLoadings[, 1:2] |>
  round(2)
# convert factor loadings < '0.1' to '0'
fl5a[abs(fl5a) < 0.1] <- 0
fl5a <- as.matrix(fl5a)
# item uniquenesses
un5 <- factorLoadings[, 3] |>
  round(2)
# factor correlations
fc5 <- factorCorrs |>
  round(2) |>
  as.matrix()
# apply the function
itemCors_5a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5a <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_5a,
  n1 = sampleSize, n2 = sampleSize
)

# factor loadings
fl5b <- factorLoadings[, 1:2] |>
  round(2)
# convert factor loadings < '0.2' to '0'
fl5b[abs(fl5b) < 0.2] <- 0
fl5b <- as.matrix(fl5b)
# apply the function
itemCors_5b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5b <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_5b,
  n1 = sampleSize, n2 = sampleSize
)

# factor loadings
fl5c <- factorLoadings[, 1:2] |>
  round(2)
# convert factor loadings < '0.2' to '0'
fl5c[abs(fl5c) < 0.3] <- 0
fl5c <- as.matrix(fl5c)
# apply the function
itemCors_5c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5c <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_5c,
  n1 = sampleSize, n2 = sampleSize
)
# kable(itemCors_5, digits = 2)

## -----------------------------------------------------------------------------
itemCors_6a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6a <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_6a,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_6b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6b <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_6b,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_6c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6c <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_6c,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
itemCors_7a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7a <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_7a,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_7b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7b <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_7b,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_7c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7c <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_7c,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
cases <- c(
  "Full information",
  "Full information - No uniqueness",
  "Rounded loadings",
  "Rounded loadings - No uniqueness",
  "Censored loadings <0.1",
  "Censored loadings <0.2",
  "Censored loadings <0.3",
  "Censored loadings <0.1 - no uniqueness",
  "Censored loadings <0.2 - no uniqueness",
  "Censored loadings <0.3 - no uniqueness",
  "Censored loadings <0.1 - no uniqueness, factor cors",
  "Censored loadings <0.2 - no uniqueness, factor cors",
  "Censored loadings <0.3 - no uniqueness, factor cors"
)

chi2 <- c(
  chiSq_1$chi2,
  chiSq_2$chi2,
  chiSq_3$chi2,
  chiSq_4$chi2,
  chiSq_5a$chi2,
  chiSq_5b$chi2,
  chiSq_5c$chi2,
  chiSq_6a$chi2,
  chiSq_6b$chi2,
  chiSq_6c$chi2,
  chiSq_7a$chi2,
  chiSq_7b$chi2,
  chiSq_7c$chi2
) |> round(2)

p <- c(
  chiSq_1$prob,
  chiSq_2$prob,
  chiSq_3$prob,
  chiSq_4$prob,
  chiSq_5a$prob,
  chiSq_5b$prob,
  chiSq_5c$prob,
  chiSq_6a$prob,
  chiSq_6b$prob,
  chiSq_6c$prob,
  chiSq_7a$prob,
  chiSq_7b$prob,
  chiSq_7c$prob
) |> round(3)

summary_results_1 <- data.frame(
  Treatment = cases,
  chi2 = chi2,
  p = p
)

kable(summary_results_1) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
## download data
data(bfi)
## filter for highly-educated women
bfi_short <- bfi |>
  filter(education == 5 & gender == 2) |>
  na.omit()
## keep just the 25 items
bfi_short <- bfi_short[, 1:25]
sampleSize <- nrow(bfi_short)
## derive correlation matrix
bfi_cor <- cor(bfi_short)

## -----------------------------------------------------------------------------
## factor analysis from `rosetta` package is a less messy version of the `psych::fa()` function

fa_bfi <- rosetta::factorAnalysis(
  data = bfi_short,
  nfactors = 5,
  rotate = "promax"
)

bfiLoadings <- fa_bfi$output$loadings
bfiCorrs <- fa_bfi$output$correlations

## -----------------------------------------------------------------------------
kable(bfiLoadings, digits = 2) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
kable(bfiCorrs, digits = 2) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
## round input values to 5 decimal places
# factor loadings
fl1 <- bfiLoadings[, 1:5] |>
  round(5) |>
  as.matrix()
# item uniquenesses
un1 <- bfiLoadings[, 6] |> round(5)
# factor correlations
fc1 <- round(bfiCorrs, 5) |> as.matrix()
# run makeCorrLoadings() function
itemCors_1 <- makeCorrLoadings(
  loadings = fl1,
  factorCor = fc1,
  uniquenesses = un1
)
## Compare the two matrices
chiSq_1 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_1,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## round input values to 5 decimal places
# factor loadings
fl2 <- bfiLoadings[, 1:5] |>
  round(5) |>
  as.matrix()
# factor correlations
fc2 <- bfiCorrs |>
  round(5) |>
  as.matrix()
# run makeCorrLoadings() function
itemCors_2 <- makeCorrLoadings(
  loadings = fl2,
  factorCor = fc2,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_2 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_2,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## round input values to 2 decimal places
# factor loadings
fl3 <- bfiLoadings[, 1:5] |>
  round(2) |>
  as.matrix()

# item uniquenesses
un3 <- bfiLoadings[, 6] |>
  round(2)
## factor correlations
fc3 <- bfiCorrs |>
  round(2) |>
  as.matrix()
# run makeCorrLoadings() function
itemCors_3 <- makeCorrLoadings(
  loadings = fl3,
  factorCor = fc3,
  uniquenesses = un3
)

## Compare the two matrices
chiSq_3 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_3,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## round input values to 2 decimal places
# factor loadings
fl4 <- bfiLoadings[, 1:5] |>
  round(2) |>
  as.matrix()
## factor correlations
fc4 <- bfiCorrs |>
  round(2) |>
  as.matrix()
# run makeCorrLoadings() function
itemCors_4 <- makeCorrLoadings(
  loadings = fl4,
  factorCor = fc4,
  uniquenesses = NULL
)

## Compare the two matrices
chiSq_4 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_4,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## round input values to 2 decimal places
# factor loadings
fl5a <- bfiLoadings[, 1:5] |>
  round(2)
# convert factor loadings < '0.1' to '0'
fl5a[abs(fl5a) < 0.1] <- 0
fl5a <- as.matrix(fl5a)
# factor correlations
fc5 <- bfiCorrs |>
  round(2) |>
  as.matrix()
# item uniquenesses
un5 <- bfiLoadings[, 6] |>
  round(2)
# run makeCorrLoadings() function
itemCors_5a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5a <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_5a,
  n1 = sampleSize, n2 = sampleSize
)
# factor loadings
fl5b <- bfiLoadings[, 1:5] |>
  round(2)
# convert factor loadings < '0.2' to '0'
fl5b[abs(fl5b) < 0.2] <- 0
fl5b <- as.matrix(fl5b)
# run makeCorrLoadings() function
itemCors_5b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5b <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_5b,
  n1 = sampleSize, n2 = sampleSize
)
# factor loadings
fl5c <- bfiLoadings[, 1:5] |>
  round(2)
# convert factor loadings < '0.3' to '0'
fl5c[abs(fl5c) < 0.3] <- 0
fl5c <- as.matrix(fl5c)
# run makeCorrLoadings() function
itemCors_5c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5c <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_5c,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## Loadings and factor correlations are the same, so we only need to change
## parameters of the makeCorrLoadings() application.

# run makeCorrLoadings() function
itemCors_6a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6a <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_6a,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_6b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6b <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_6b,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_6c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6c <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_6c,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
## Loadings and factor correlations are the same, so we only need to change
## parameters of the makeCorrLoadings() application.

# run makeCorrLoadings() function
itemCors_7a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7a <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_7a,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_7b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7b <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_7b,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_7c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7c <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_7c,
  n1 = sampleSize, n2 = sampleSize
)

## -----------------------------------------------------------------------------
cases <- c(
  "Full information",
  "Full information - No uniquenesses",
  "Rounded loadings",
  "Rounded loadings - No uniquenesses",
  "Censored loadings <0.1",
  "Censored loadings <0.2",
  "Censored loadings <0.3",
  "Censored loadings <0.1 - no uniqueness",
  "Censored loadings <0.2 - no uniqueness",
  "Censored loadings <0.3 - no uniqueness",
  "Censored loadings <0.1 - no uniqueness, no factor cors",
  "Censored loadings <0.2 - no uniqueness, no factor cors",
  "Censored loadings <0.3 - no uniqueness, no factor cors"
)

chi2 <- c(
  chiSq_1$chi2,
  chiSq_2$chi2,
  chiSq_3$chi2,
  chiSq_4$chi2,
  chiSq_5a$chi2,
  chiSq_5b$chi2,
  chiSq_5c$chi2,
  chiSq_6a$chi2,
  chiSq_6b$chi2,
  chiSq_6c$chi2,
  chiSq_7a$chi2,
  chiSq_7b$chi2,
  chiSq_7c$chi2
) |> round(2)

p <- c(
  chiSq_1$prob,
  chiSq_2$prob,
  chiSq_3$prob,
  chiSq_4$prob,
  chiSq_5a$prob,
  chiSq_5b$prob,
  chiSq_5c$prob,
  chiSq_6a$prob,
  chiSq_6b$prob,
  chiSq_6c$prob,
  chiSq_7a$prob,
  chiSq_7b$prob,
  chiSq_7c$prob
) |> round(3)

summary_results_2 <- data.frame(
  Treatment = cases,
  chi2 = chi2,
  p = p
)

# summary_results_2
kable(summary_results_2, digits = c(0, 1, 5)) |>
  kable_classic(full_width = F)

## -----------------------------------------------------------------------------
overall_summary <- data.frame(
  treatment = cases,
  chi2.1 = summary_results_1[, 2],
  p.1 = summary_results_1[, 3],
  chi2.2 = summary_results_2[, 2],
  p.2 = summary_results_2[, 3]
)

names(overall_summary) <- c("treatment", "chi2", "p", "chi2", "p")

kable(overall_summary, digits = c(0, 1, 3, 1, 3)) |>
  column_spec(4, border_left = T) |>
  kable_classic(full_width = F) |>
  add_header_above(c(" " = 1, "Party panel" = 2, "Big 5 (bfi)" = 2))

#' calculate eigenvalues of a correlation matrix with optional scree plot
#'
#' @name eigenvalues
#' @description `eigenvalues()` calculates eigenvalues of a correlation
#'  matrix and optionally produces a scree plot.
#'
#' @param cormatrix (real, matrix) a correlation matrix
#' @param scree (logical) default = `FALSE`. If `TRUE` (or `1`), then
#'  `eigenvalues()` produces a scree plot to illustrate the eigenvalues
#'
#' @importFrom graphics abline rect
#'
#' @return a vector of eigenvalues
#' @return report on positive-definite status of cormatrix
#' @export eigenvalues
#'
#' @examples
#'
#' ## define parameters
#'
#' correlationMatrix <- matrix(
#'   c(
#'     1.00, 0.25, 0.35, 0.40,
#'     0.25, 1.00, 0.70, 0.75,
#'     0.35, 0.70, 1.00, 0.80,
#'     0.40, 0.75, 0.80, 1.00
#'   ),
#'   nrow = 4, ncol = 4
#' )
#'
#' ## apply function
#'
#' evals <- eigenvalues(cormatrix = correlationMatrix)
#' evals <- eigenvalues(correlationMatrix, 1)
#'
eigenvalues <- function(cormatrix, scree = FALSE) {
  e_vals <- eigen(cormatrix)$values
  if (scree == TRUE) {
    plot(e_vals,
      pch = 19, col = "black", cex = 1.0,
      ylab = "Eigenvalues",
      ylim = c(min(0, min(e_vals)), max(e_vals)),
      main = paste0("Scree Plot: ", deparse(substitute(cormatrix))),
      cex.main = 1, col.main = "black",
      type = "b"
    )
    abline(h = c(0, 1), col = "skyblue", lty = 2)
    rect(
      xleft = 0.8, xright = length(e_vals) * 1.1,
      ytop = 0, ybottom = min(0, min(e_vals) * 1.25),
      col = "#FFC0CB56", border = NA
    )
  }
  if (min(e_vals) >= 0) {
    cat(deparse(substitute(cormatrix)), " is positive-definite\n\n")
  } else {
    cat(deparse(substitute(cormatrix)), " is NOT positive-definite\n\n")
  }

  return(e_vals)
} ## end eigenvalues function

## Unit tests for eigenvalues() function

test_that("eigenvalues runs silently with valid inputs", {
  ## define parameters

  correlationMatrix <- matrix(
    c(
      1.00, 0.25, 0.35, 0.40,
      0.25, 1.00, 0.70, 0.75,
      0.35, 0.70, 1.00, 0.80,
      0.40, 0.75, 0.80, 1.00
    ),
    nrow = 4, ncol = 4
  )


  ## apply function examples

  expect_no_warning(eigenvalues(cormatrix = correlationMatrix))

  expect_no_warning(eigenvalues(correlationMatrix, 1))
})

#' Deprecated. Use lfast() instead
#'
#' @name lexact
#'
#' @description \code{lexact} is DEPRECATED.
#' Replaced in LikertMakeR Version 0.4.0 by new version of \code{lfast}.
#'
#' \code{lexact} remains as a legacy for earlier package users.
#' It is now just a wrapper for \code{lfast}
#'
#' Previously, \code{lexact} used a Differential Evolution (DE) algorithm to
#' find an optimum solution with desired mean and standard deviation,
#' but we found that the updated \code{lfast} function is much faster and just
#' as accurate.
#'
#' Also the package is much less bulky.
#'
#' @param n (positive, int) number of observations to generate
#' @param mean (real) target mean
#' @param sd  (real) target standard deviation
#' @param lowerbound (positive, int) lower bound
#' @param upperbound (positive, int) upper bound
#' @param items (positive, int) number of items in the rating scale.
#'
#' @return a vector of simulated data approximating user-specified conditions.
#'
#' @importFrom stats rbeta
#'
#' @export lexact
#'
#' @examples
#'
#' x <- lexact(
#'   n = 256,
#'   mean = 4.0,
#'   sd = 1.0,
#'   lowerbound = 1,
#'   upperbound = 7,
#'   items = 6
#' )
#'
#' x <- lexact(256, 2, 1.8, 0, 10)
#'
lexact <- function(n, mean, sd, lowerbound, upperbound, items = 1) {
  message("lexact() function is deprecated.
          \nUsing the more efficient lfast() function instead")
  lfast(
    n = n, mean = mean, sd = sd,
    lowerbound = lowerbound, upperbound = upperbound, items = items
  )
}

#' Synthesise rating-scale data with predefined mean and standard deviation
#'
#' @name lfast
#'
#' @description `lfast()` applies a simple Evolutionary Algorithm to
#' find a vector that best fits the desired moments.
#'
#' `lfast()` generates random discrete values from a
#' scaled Beta distribution so the data replicate an ordinal rating scale -
#' for example, a Likert scale made from multiple items (questions) or 0-10
#' likelihood-of-purchase scale. Data generated are generally consistent with
#' real data, as shown in the _lfast() validation article_.
#'
#' @seealso
#' - lfast validation article:
#' <https://winzarh.github.io/LikertMakeR/articles/lfast_validation.html>
#' - Package website:
#' <https://winzarh.github.io/LikertMakeR/>
#'
#' @param n (positive, int) number of observations to generate
#' @param mean (real) target mean, between upper and lower bounds
#' @param sd  (positive, real) target standard deviation
#' @param lowerbound (int) lower bound (e.g. '1' for a 1-5 rating scale)
#' @param upperbound (int) upper bound (e.g. '5' for a 1-5 rating scale)
#' @param items (positive, int) number of items in the rating scale.
#'   Default = 1
#' @param precision (positive, real) can relax the level of accuracy required.
#' (e.g. '1' generally generates a vector with moments correct within '0.025',
#' '2' generally within '0.05') Default = 0
#'
#' @return a vector approximating user-specified conditions.
#'
#' @importFrom stats rbeta
#'
#' @examples
#'
#' ## six-item 1-7 rating scale
#' x <- lfast(
#'   n = 256,
#'   mean = 4.0,
#'   sd = 1.25,
#'   lowerbound = 1,
#'   upperbound = 7,
#'   items = 6
#' )
#'
#' ## five-item -3 to +3 rating scale
#' x <- lfast(
#'   n = 64,
#'   mean = 0.025,
#'   sd = 1.25,
#'   lowerbound = -3,
#'   upperbound = 3,
#'   items = 5
#' )
#'
#' ## four-item 1-5 rating scale with medium variation
#' x <- lfast(
#'   n = 128,
#'   mean = 3.0,
#'   sd = 1.00,
#'   lowerbound = 1,
#'   upperbound = 5,
#'   items = 4,
#'   precision = 5
#' )
#'
#' ## eleven-point 'likelihood of purchase' scale
#' x <- lfast(256, 3, 3.0, 0, 10)
#'
#' @export
lfast <- function(n, mean, sd,
                  lowerbound, upperbound,
                  items = 1, precision = 0) {
  tolerance <- 0.0025 * 2^precision
  range <- upperbound - lowerbound
  m <- (mean - lowerbound) / range ## rescale mean
  s <- sd / range ## rescale sd
  ## idiot checks
  if (mean <= lowerbound || mean >= upperbound) {
    stop("ERROR: mean is out of range")
  }
  if (sd >= range * 0.6) {
    warning("Standard Deviation is large relative to range
            \nDerived SD may be less than specified
            \nOr the solution is not feasible, producing 'NA' values")
  }
  ## Beta distribution shape parameters
  a <- (m^2 - m^3 - m * s^2) / s^2 ## alpha shape parameter
  b <- (m - 2 * m^2 + m^3 - s^2 + m * s^2) / s^2 ## beta shape parameter

  best_value <- Inf ## set high value to start
  best_vector <- NULL

  maxiter <- max(1024, n^2) ## at least 2^10 iterations

  for (i in 1:maxiter) {
    ## generate data with range 0-1 as Beta distribution
    item_vector <- rbeta(n, a, b)
    ## rescale Beta values to desired parameters
    item_vector <- round((item_vector * range + lowerbound) * items) / items

    ## difference between target moments and actual moments
    mean_dif <- mean(item_vector) - mean
    sd_dif <- sd(item_vector) - sd
    temp_value <- abs(mean_dif) + abs(sd_dif) ## simple value to be minimised

    ## keep the best result so far
    if (temp_value < best_value) {
      best_vector <- item_vector
      best_value <- temp_value
    }

    ## tolerance ensures both mean and sd accurate to 2 decimal places
    if (best_value < tolerance) {
      break
    }
  }

  ## print iterations reached
  if (i == maxiter) {
    message(paste0("reached maximum of ", i, " iterations"))
  } else {
    message(paste0("best solution in ", i, " iterations"))
  }

  return(best_vector)
}

## Unit tests for lfast() function

test_that("lfast produces correct length and bounded output", {
  set.seed(123) # for reproducibility
  x <- lfast(
    n = 256,
    mean = 4.5,
    sd = 1.0,
    lowerbound = 1,
    upperbound = 7,
    items = 5
  )

  # Check the length of the output
  expect_length(x, 256)

  # Check if the values are within the specified bounds
  expect_true(all(x >= 1 & x <= 7))

  # Check if moments are within tolerance bounds
  expect_true(abs(mean(x) - 4.5) < 0.1)
  expect_true(abs(sd(x) - 1.0) < 0.1)
})


test_that("lfast runs silently with valid inputs", {
  set.seed(123)
  expect_no_warning(lfast(
    n = 64,
    mean = 4.5,
    sd = 1.0,
    lowerbound = 1,
    upperbound = 7,
    items = 5
  ))
})


test_that("lfast throws an error with invalid inputs", {
  expect_error(
    lfast(
      n = -1,
      mean = 4.5,
      sd = 1.0,
      lowerbound = 1,
      upperbound = 7,
      items = 5
    )
  )
})

#' Generate scale items from a summated scale, with desired Cronbach's Alpha
#'
#' @name makeItemsScale
#'
#' @description `makeItemsScale()` generates a random dataframe
#'  of scale items based on a predefined summated scale
#'  (such as created by the [lfast()] function),
#'  and a desired _Cronbach's Alpha_.
#'
#' @param scale (int) a vector or dataframe of the summated rating scale.
#' Should range from \eqn{lowerbound \times items} to
#'  \eqn{upperbound \times items}
#' @param lowerbound (int) lower bound of the scale item
#' (example: '1' in a '1' to '5' rating)
#' @param upperbound (int) upper bound of the scale item
#' (example: '5' in a '1' to '5' rating)
#' @param items (positive, int) k, or number of columns to generate
#' @param alpha (posiitve, real) desired _Cronbach's Alpha_ for the
#' new dataframe of items.
#' Default = '0.8'.
#' @param summated logical. If TRUE (default), `scale` is assumed to be a
#' summated scale. If FALSE, the input is treated as a mean scale and will
#' be multiplied by `items` internally.
#' @param progress logical. If TRUE (default is `interactive()`), a progress
#' bar is displayed during optimisation.
#' Set to FALSE for slightly faster execution.
#'
#' @details
#'
#' ## simulated annealing
#'
#' Simulated annealing is used in `makeItemsScale()` to search for item
#' configurations that satisfy several competing objectives simultaneously.
#' These include achieving a target Cronbach’s alpha, maintaining similar
#' item means, and preserving constant row sums. Because improving one
#' objective can worsen another, a simple greedy search often becomes
#' trapped in suboptimal solutions.
#'
#' Simulated annealing addresses this by allowing occasional acceptance of
#' worse solutions early in the search, promoting exploration of the solution
#' space. As the algorithm progresses, a cooling schedule reduces this
#' randomness, increasingly favouring improvements across all criteria.
#' This process enables the function to converge on a balanced solution that
#' jointly satisfies the desired scale properties, rather than optimizing any
#' single objective at the expense of others.
#'
#' @importFrom gtools combinations
#' @importFrom stats quantile runif sd var
#' @importFrom utils combn setTxtProgressBar txtProgressBar
#' @importFrom matrixStats colVars
#'
#' @return a dataframe with 'items' columns and 'length(scale)' rows
#'
#' @examples
#'
#' summatedScale <- c(16, 11, 14, 7, 10, 13, 16, 13, 12, 14, 8, 11, 8, 18, 8, 13)
#'
#' newItems <- makeItemsScale(
#'   scale = summatedScale,
#'   lowerbound = 1,
#'   upperbound = 5,
#'   items = 4,
#'   alpha = 0.8
#' )
#'
#' @export
#'
makeItemsScale <- function(
  scale,
  lowerbound,
  upperbound,
  items,
  alpha = 0.80,
  summated = TRUE,
  progress = TRUE
) {
  ###
  ##  makeCombinations produces a dataframe of all combinations of item values
  ###
  makeCombinations <- function(lowerbound, upperbound, items) {
    mycombinations <- gtools::combinations(
      v = c(lowerbound:upperbound),
      r = items,
      n = length(c(lowerbound:upperbound)),
      repeats.allowed = TRUE
    )
    return(mycombinations)
  } # END makeCombinations function

  ###
  ## makeVector extracts candidate rows for given rowSums (scale value)
  ###
  makeVector <- function(targetSum, items) {
    shortdat <- combos_by_sum[[as.character(targetSum)]]

    if (is.null(shortdat) || nrow(shortdat) == 0) {
      stop(paste0("No candidate partition found for sum = ", targetSum))
    }

    idx <- sample.int(nrow(shortdat), size = 1)

    vec <- shortdat[idx, 1:items]

    return(as.numeric(vec))
  } # END makeVector function


  ## tune weights
  ## we want means to be balanced - about the same
  w_balance <- 0.025
  expected_mean <- mean(scale) / items

  ## score_items attempts to find penalty values so that
  ## column means are similar for the new dataset
  score_items <- function(dat, target_alpha, w_balance) {
    k <- ncol(dat)

    item_vars <- matrixStats::colVars(dat)

    rs <- .rowSums(dat, nrow(dat), ncol(dat))
    mean_rs <- mean(rs)
    total_var <- sum((rs - mean_rs)^2) / (length(rs) - 1)

    if (!is.finite(total_var) || total_var <= 0) {
      alpha_val <- 0
    } else {
      alpha_val <- (k / (k - 1)) * (1 - sum(item_vars) / total_var)
    }

    alpha_err <- abs(alpha_val - target_alpha)

    col_means <- .colSums(dat, nrow(dat), ncol(dat)) / nrow(dat)
    balance_penalty <- mean((col_means - expected_mean)^2)

    score <- alpha_err^2 +
      w_balance * balance_penalty * (1 + alpha_err)

    list(
      score = score,
      alpha = alpha_val
    )
  } ## END score_items function


  ###
  ##  rearrangeRowValues attempts to move column values
  ##  in each row to achieve desired Cronbach's Alpha
  ###
  rearrangeRowValues <- function(progress = progress) {
    # Initialize variables for best alpha and corresponding dataframe

    # Target alpha value
    target_alpha <- alpha
    alpha_tolerance <- 0.00125

    current_items <- mydat
    current_eval <- score_items(current_items, target_alpha, w_balance)
    current_score <- current_eval$score

    best_items <- current_items
    best_score <- current_score
    best_alpha <- current_eval$alpha

    min_iter <- max(64, items^2)
    max_iter <- items^2 * n * 2

    iter <- 0

    if (progress) {
      pb <- txtProgressBar(min = 0, max = max_iter, style = 3)
    }

    pairs <- combn(ncol(current_items), 2, simplify = FALSE)

    while (iter < max_iter) {
      iter <- iter + 1

      if (progress) {
        setTxtProgressBar(pb, iter)
      }

      ## add temeprature cooling
      T0 <- 0.001
      T <- T0 * (1 - iter / max_iter)
      T <- max(T, 1e-6)

      # Loop through each pair of columns j and k

      pairs_subset <- sample(pairs, size = min(2 * items, length(pairs)))
      row_ids <- sample.int(nrow(current_items), size = min(5 * items, nrow(current_items)))

      for (pair in pairs_subset) {
        j <- pair[1]
        k <- pair[2]

        # Skip if j == k, (no need to swap a column with itself)
        if (j == k) next

        # Loop through each row and swap columns j and k in that row
        # for (i in 1:nrow(current_items)) {

        for (i in row_ids) {
          temp_items <- current_items

          u <- runif(1)

          if (u < 0.18) {
            ## safe ±1 redistribution within row (sum-preserving)
            row <- temp_items[i, ]

            j1 <- j
            j2 <- k

            if (runif(1) < 0.5) {
              if (row[j1] < upperbound && row[j2] > lowerbound) {
                row[j1] <- row[j1] + 1
                row[j2] <- row[j2] - 1
                temp_items[i, ] <- row
              }
            } else {
              if (row[j1] > lowerbound && row[j2] < upperbound) {
                row[j1] <- row[j1] - 1
                row[j2] <- row[j2] + 1
                temp_items[i, ] <- row
              }
            }
          } else if (u < 0.40) {
            i2 <- sample(setdiff(1:nrow(current_items), i), 1)

            row_i <- as.numeric(temp_items[i, ])
            row_i2 <- as.numeric(temp_items[i2, ])

            # check sum compatibility
            if ((row_i[j] + row_i[k]) == (row_i2[j] + row_i2[k])) {
              new_i <- row_i
              new_i2 <- row_i2

              new_i[j] <- row_i2[j]
              new_i2[j] <- row_i[j]

              new_i[k] <- row_i2[k]
              new_i2[k] <- row_i[k]

              temp_items[i, ] <- new_i
              temp_items[i2, ] <- new_i2
            }
          } else {
            # within-row swap (~60%)
            temp <- temp_items[i, j]
            temp_items[i, j] <- temp_items[i, k]
            temp_items[i, k] <- temp
          }

          ## search and evaluation is expensive, so we use C++ code

          new_eval <- score_items_cpp(temp_items, target_alpha, w_balance, expected_mean)

          delta <- new_eval$score - current_score

          ## temperature

          # enforce constraint BEFORE accepting
          valid <- (sum(temp_items[i, ]) == scale[i])

          if (valid) {
            if (delta < 0 || runif(1) < exp(-delta / T)) {
              current_items <- temp_items
              current_score <- new_eval$score
            }
          } else {
            stop(paste(
              "Invalid move detected:",
              "i =", i,
              "j =", j,
              "k =", k,
              "u =", round(u, 3)
            ))
          }

          if (current_score < best_score) {
            best_items <- current_items
            best_score <- current_score
            best_alpha <- new_eval$alpha
          }

          # Stop if alpha is within tolerance
          if (iter > min_iter && abs(best_alpha - target_alpha) <= alpha_tolerance) {
            if (progress) close(pb)

            message(paste0("iterations=", iter))

            return(best_items |> as.data.frame())
          }
        }
      }
    }

    if (progress) close(pb)

    message(paste0("reached maximum of ", iter, " iterations"))

    return(best_items |> as.data.frame())
  } ## END rearrangeRowValues function


  ###
  ## Functions done.. Now we run some calculations!
  ###

  candidates <- makeCombinations(
    lowerbound = lowerbound,
    upperbound = upperbound,
    items = items
  )

  # --- precompute combinations by sum ---

  candidates_df <- as.data.frame(candidates)

  candidates_df$sums <- rowSums(candidates_df)

  # compute sd
  candidates_df$sds <- apply(candidates_df[, 1:items], 1, sd)

  # split by sum
  combos_by_sum <- split(candidates_df, candidates_df$sums)

  if (!summated) {
    scale <- scale * items
  }

  scale <- as.numeric(scale)

  message(paste0("generate ", length(scale), " rows"))

  n <- length(scale)
  mydat <- matrix(NA_real_, nrow = n, ncol = items)

  for (i in seq_len(n)) {
    mydat[i, ] <- makeVector(scale[i], items)
  }

  perm <- sample(1:ncol(mydat))
  mydat <- mydat[, perm]

  colnames(mydat) <- paste0("X", seq_len(ncol(mydat)))

  message(paste0("rearrange ", items, " values within each of ", length(scale), " rows"))

  optimised_dat <- rearrangeRowValues(progress = progress)

  best_alpha <- LikertMakeR::alpha(data = optimised_dat) |> round(4)

  message(paste0("Complete!"))

  message(paste0("desired Cronbach's alpha = ", alpha, " (achieved alpha = ", best_alpha, ")"))

  return(optimised_dat)
}

## Unit tests for makeItemsScale() function

test_that("makeItemsScale runs silently with valid inputs", {
  ## define parameters

  myvalues <- c(2.25, 3.75, 2.50, 2.50, 4.25, 2.50, 4.25, 4.00)

  ## apply function test

  expect_no_warning(makeItemsScale(
    scale = myvalues,
    lowerbound = 1,
    upperbound = 5,
    items = 4,
    summated = FALSE
  ))
})

#' Synthesise rating-scale data with given first and second moments and a
#' predefined correlation matrix
#'
#'
#' Generates a dataframe of random discrete
#'  values so the data replicate a rating scale,
#'  and are correlated close to a predefined correlation matrix.
#'
#'  `makeScales()` is wrapper function for:
#'
#'   * [lfast()], generates a dataframe that best fits the desired
#'    moments, and
#'   * [lcor()], which rearranges values in each column of the dataframe
#'    so they closely match the desired correlation matrix.
#'
#' @param n (positive, int) sample-size - number of observations
#' @param means (real) target means: a vector of length k
#'  of mean values for each scale item
#' @param sds (positive, real) target standard deviations: a vector of length k
#'  of standard deviation values for each scale item
#' @param lowerbound (positive, int) a vector of length k
#'  (same as rows & columns of correlation matrix) of values for lower bound
#'  of each scale item (e.g. '1' for a 1-5 rating scale). Default = 1.
#' @param upperbound (positive, int) a vector of length k
#'  (same as rows & columns of correlation matrix) of values for upper bound
#'  of each scale item (e.g. '5' for a 1-5 rating scale). Default = 5.
#' @param items (positive, int) a vector of length k of number of items in each scale. Default = 1.
#' @param cormatrix (real, matrix) the target correlation matrix:
#'  a square symmetric positive-semi-definite matrix of values ranging
#'  between -1 and +1, and '1' in the diagonal.
#'
#' @return a dataframe of rating-scale values
#'
#' @seealso \code{\link{lfast}}, \code{\link{lcor}}
#'
#' @examples
#'
#' ## Example 1: four correlated items (questions)
#'
#' ### define parameters
#'
#' n <- 16
#' dfMeans <- c(2.5, 3.0, 3.0, 3.5)
#' dfSds <- c(1.0, 1.0, 1.5, 0.75)
#' lowerbound <- rep(1, 4)
#' upperbound <- rep(5, 4)
#'
#' corMat <- matrix(
#'   c(
#'     1.00, 0.30, 0.40, 0.60,
#'     0.30, 1.00, 0.50, 0.70,
#'     0.40, 0.50, 1.00, 0.80,
#'     0.60, 0.70, 0.80, 1.00
#'   ),
#'   nrow = 4, ncol = 4
#' )
#'
#' scale_names <- c("Q1", "Q2", "Q3", "Q4")
#' rownames(corMat) <- scale_names
#' colnames(corMat) <- scale_names
#'
#' ### apply function
#'
#' df1 <- makeScales(
#'   n = n, means = dfMeans, sds = dfSds,
#'   lowerbound = lowerbound, upperbound = upperbound, cormatrix = corMat
#' )
#'
#' ### test function
#'
#' str(df1)
#'
#' #### means
#' apply(df1, 2, mean) |> round(3)
#'
#' #### standard deviations
#' apply(df1, 2, sd) |> round(3)
#'
#' #### correlations
#' cor(df1) |> round(3)
#'
#'
#' ## Example 2: five correlated Likert scales
#'
#' ### a study on employee engagement and organizational climate:
#' # Job Satisfaction (JS)
#' # Organizational Commitment (OC)
#' # Perceived Supervisor Support (PSS)
#' # Work Engagement (WE)
#' # Turnover Intention (TI) (reverse-related to others).
#'
#' ### define parameters
#'
#' n <- 128
#' dfMeans <- c(3.8, 3.6, 3.7, 3.9, 2.2)
#' dfSds <- c(0.7, 0.8, 0.7, 0.6, 0.9)
#' lowerbound <- rep(1, 5)
#' upperbound <- rep(5, 5)
#' items <- c(4, 4, 3, 3, 3)
#'
#' corMat <- matrix(
#'   c(
#'     1.00, 0.72, 0.58, 0.65, -0.55,
#'     0.72, 1.00, 0.54, 0.60, -0.60,
#'     0.58, 0.54, 1.00, 0.57, -0.45,
#'     0.65, 0.60, 0.57, 1.00, -0.50,
#'     -0.55, -0.60, -0.45, -0.50, 1.00
#'   ),
#'   nrow = 5, ncol = 5
#' )
#'
#' scale_names <- c("JS", "OC", "PSS", "WE", "TI")
#' rownames(corMat) <- scale_names
#' colnames(corMat) <- scale_names
#'
#' ### apply function
#'
#' df2 <- makeScales(
#'   n = n, means = dfMeans, sds = dfSds,
#'   lowerbound = lowerbound, upperbound = upperbound,
#'   items = items, cormatrix = corMat
#' )
#'
#' ### test function
#'
#' str(df2)
#'
#' #### means
#' apply(df2, 2, mean) |> round(3)
#'
#' #### standard deviations
#' apply(df2, 2, sd) |> round(3)
#'
#' #### correlations
#' cor(df2) |> round(3)
#'
#' @export
makeScales <- function(n, means, sds,
                       lowerbound = 1, upperbound = 5,
                       items = 1, cormatrix) {
  ####
  ### Basic scalar checks
  ####

  if (!is.numeric(n) || length(n) != 1L || !is.finite(n) || n <= 0 || n != as.integer(n)) {
    stop("'n' must be a single positive integer.")
  }
  n <- as.integer(n)

  ####
  ### Correlation matrix checks
  ####

  if (!is.matrix(cormatrix) || !is.numeric(cormatrix)) {
    stop("'cormatrix' must be a numeric matrix.")
  }
  if (nrow(cormatrix) != ncol(cormatrix)) {
    stop("'cormatrix' must be a square matrix.")
  }

  # Round correlation values to sensible values
  cormatrix <- round(cormatrix, 5L)

  k <- ncol(cormatrix)

  # Symmetry
  if (!isTRUE(all.equal(cormatrix, t(cormatrix), tolerance = 1e-8))) {
    stop("'cormatrix' must be symmetric.")
  }

  # Diagonal close to 1
  if (!all(abs(diag(cormatrix) - 1) < 1e-8)) {
    stop("The diagonal of 'cormatrix' must all be 1.")
  }

  # Values in [-1, 1]
  if (any(cormatrix < -1 - 1e-8 | cormatrix > 1 + 1e-8)) {
    stop("All entries of 'cormatrix' must lie in the interval [-1, 1].")
  }

  # Positive semi-definite check
  check_PD_matrix <- function() {
    eigvals <- eigen(cormatrix, symmetric = TRUE, only.values = TRUE)$values
    if (min(eigvals) < -1e-8) {
      stop(
        "cormatrix is not Positive Semi-Definite.\n",
        "Requested correlations are not possible."
      )
    }
  }
  check_PD_matrix()

  ####
  ### Naming of variables
  ####

  if (is.null(rownames(cormatrix)) || is.null(colnames(cormatrix))) {
    scale_names <- paste0("V", seq_len(k))
    rownames(cormatrix) <- scale_names
    colnames(cormatrix) <- scale_names
  } else {
    if (!identical(rownames(cormatrix), colnames(cormatrix))) {
      warning(
        "Row and column names of 'cormatrix' do not match.",
        "\nUsing rownames for both."
      )
      scale_names <- rownames(cormatrix)
      colnames(cormatrix) <- scale_names
    } else {
      scale_names <- rownames(cormatrix)
    }
  }

  ####
  ### Vector checks for means and sds
  ####

  if (!is.numeric(means) || length(means) != k || any(!is.finite(means))) {
    stop(
      "'means' must be a numeric vector of length ", k,
      " with all finite values."
    )
  }

  if (!is.numeric(sds) || length(sds) != k || any(!is.finite(sds)) || any(sds <= 0)) {
    stop(
      "'sds' must be a numeric vector of positive, ",
      "finite values of length ", k, "."
    )
  }

  ####
  ### Recycle bounds and items, then do integrity checks
  ####

  # Recycle single values to vectors if needed
  if (length(lowerbound) == 1L) lowerbound <- rep(lowerbound, k)
  if (length(upperbound) == 1L) upperbound <- rep(upperbound, k)
  if (length(items) == 1L) items <- rep(items, k)

  parameter_integrity <- function() {
    # lengths
    if (length(lowerbound) != k || length(upperbound) != k ||
      length(items) != k) {
      stop(
        "Parameters have unequal length and dimensions.",
        "\nlowerbound, upperbound, items must each have length k = ", k,
        ", and 'means' and 'sds' must also have length k."
      )
    }

    # types and finiteness
    if (!is.numeric(lowerbound) || any(!is.finite(lowerbound))) {
      stop("'lowerbound' must be a numeric vector of finite values.")
    }
    if (!is.numeric(upperbound) || any(!is.finite(upperbound))) {
      stop("'upperbound' must be a numeric vector of finite values.")
    }
    if (!is.numeric(items) || any(!is.finite(items))) {
      stop("'items' must be a numeric vector of finite values.")
    }

    # bounds order
    if (any(lowerbound >= upperbound)) {
      stop(
        "All 'lowerbound' values must be strictly less than the ",
        "corresponding 'upperbound' values."
      )
    }

    # items as positive integers
    if (any(items <= 0) || any(items != as.integer(items))) {
      stop("'items' must contain positive integers (number of items per scale).")
    }
  }

  parameter_integrity()

  # Coerce items to integer now that they are validated
  items <- as.integer(items)

  # Optional: warn if means are outside bounds (but don't stop)
  if (any(means < lowerbound | means > upperbound)) {
    stop(
      "Some 'means' are outside their corresponding",
      " [lowerbound, upperbound] ranges."
    )
  }

  ####
  ### Data generation
  ####

  # Initialize the dataframe
  df <- as.data.frame(matrix(NA_real_, nrow = n, ncol = k))

  for (i in seq_len(k)) {
    cat("Variable ", i, ": ", scale_names[i], " - ")
    df[, i] <- lfast(
      n          = n,
      mean       = means[i],
      sd         = sds[i],
      lowerbound = lowerbound[i],
      upperbound = upperbound[i],
      items      = items[i]
    )
  }

  cat("\nArranging data to match correlations\n")
  new_df <- lcor(df, cormatrix)

  # if (!is.matrix(new_df) && !is.data.frame(new_df)) {
  #   stop("'lcor()' did not return an object that can be converted to a data.frame.")
  # }

  new_df <- as.data.frame(new_df)
  colnames(new_df) <- scale_names

  cat("\nSuccessfully generated correlated variables\n\n")

  return(new_df)
}

test_that("include = lambda6 adds lambda6 row", {
  skip_if_not_installed("psych")

  set.seed(123)

  cor_mat <- LikertMakeR::makeCorrAlpha(
    items = 4,
    alpha = 0.80
  )

  dat <- LikertMakeR::makeScales(
    n = 50,
    means = c(3, 3, 3, 3),
    sds = c(1, 1, 1, 1),
    lowerbound = rep(1, 4),
    upperbound = rep(5, 4),
    cormatrix = cor_mat
  )

  res <- reliability(dat, include = "lambda6")

  expect_true("lambda6" %in% res$coef_name)
})


test_that("polychoric requested but skipped produces NA ordinal estimates", {
  set.seed(123)

  # force sparsity
  dat <- data.frame(
    i1 = c(rep(1, 49), 5),
    i2 = sample(1:5, 50, replace = TRUE),
    i3 = sample(1:5, 50, replace = TRUE),
    i4 = sample(1:5, 50, replace = TRUE)
  )

  res <- reliability(
    dat,
    include = "polychoric",
    verbose = FALSE
  )

  expect_true("ordinal_alpha" %in% res$coef_name)
  expect_true(all(is.na(res$estimate[res$coef_name == "ordinal_alpha"])))
  expect_true(!is.null(attr(res, "ordinal_diagnostics")))
})

test_that("reliability() returns a valid likert_reliability object", {
  set.seed(123)

  cor_mat <- LikertMakeR::makeCorrAlpha(
    items = 4,
    alpha = 0.80
  )

  dat <- LikertMakeR::makeScales(
    n = 50,
    means = c(3, 3, 3, 3),
    sds = c(1, 1, 1, 1),
    lowerbound = rep(1, 4),
    upperbound = rep(5, 4),
    cormatrix = cor_mat
  )

  res <- reliability(dat)

  # class
  expect_s3_class(res, "likert_reliability")

  # structure
  expect_true(is.data.frame(res))
  expect_true(all(c("coef_name", "estimate", "notes") %in% names(res)))

  # defaults: no CIs
  expect_false("ci_lower" %in% names(res))
  expect_false("ci_upper" %in% names(res))

  # defaults: only alpha + omega
  expect_equal(res$coef_name, c("alpha", "omega_total"))

  # attributes
  expect_identical(attr(res, "include"), "none")
  expect_false(isTRUE(attr(res, "ci")))
})

# tests/testthat/test-makeScales.R

test_that("makeScales: basic generation works and respects shapes, names, bounds", {
  # Ensure testthat is loaded
  if (!requireNamespace("LikertMakeR", quietly = TRUE)) {
    skip("LikertMakeR not installed; skipping this test.")
  }

  set.seed(123)

  n <- 128L
  k <- 4L
  means <- c(2.5, 3.0, 3.0, 3.5)
  sds <- c(1.0, 1.0, 1.5, 0.75)
  lb <- rep(1L, k)
  ub <- rep(5L, k)


  corMat <- matrix(c(
    1.00, 0.30, 0.40, 0.60,
    0.30, 1.00, 0.50, 0.70,
    0.40, 0.50, 1.00, 0.80,
    0.60, 0.70, 0.80, 1.00
  ), nrow = k, byrow = TRUE)
  dimnames(corMat) <- list(paste0("Q", 1:k), paste0("Q", 1:k))


  df <- makeScales(
    n = n, means = means, sds = sds,
    lowerbound = lb, upperbound = ub, cormatrix = corMat
  )

  # structure
  testthat::expect_true(is.data.frame(df))
  testthat::expect_identical(nrow(df), n)
  testthat::expect_identical(ncol(df), k)
  testthat::expect_identical(colnames(df), colnames(corMat))

  # bounds respected
  for (j in seq_len(k)) {
    testthat::expect_true(all(df[[j]] >= lb[j] & df[[j]] <= ub[j]))
  }

  # moments are in the ballpark (tolerances reflect discreteness and finite n)
  m_hat <- vapply(df, mean, numeric(1))
  s_hat <- vapply(df, stats::sd, numeric(1))

  testthat::expect_true(all(abs(m_hat - means) <= 0.25))
  testthat::expect_true(all(abs(s_hat - sds) <= 0.35))

  # correlations roughly match target (off-diagonal)
  Rhat <- stats::cor(df)
  offdiag <- function(M) M[upper.tri(M)]
  testthat::expect_true(all(abs(offdiag(Rhat) - offdiag(corMat)) <= 0.15))
})

testthat::test_that("makeScales: scalar lower/upper/items recycle correctly", {
  set.seed(42)
  n <- 150L
  k <- 5L

  means <- rep(3, k)
  sds <- rep(1, k)
  lb <- 1L # scalar, should recycle
  ub <- 7L # scalar, should recycle
  items <- 3L # scalar, should recycle

  corMat <- diag(k)
  dimnames(corMat) <- list(LETTERS[1:k], LETTERS[1:k])

  df <- makeScales(
    n = n, means = means, sds = sds,
    lowerbound = lb, upperbound = ub, items = items,
    cormatrix = corMat
  )

  testthat::expect_true(is.data.frame(df))
  testthat::expect_identical(nrow(df), n)
  testthat::expect_identical(ncol(df), k)
  testthat::expect_identical(colnames(df), colnames(corMat))
  for (j in seq_len(k)) {
    testthat::expect_true(all(df[[j]] >= 1L & df[[j]] <= 7L))
  }
})


testthat::test_that("makeScales: rejects non-PSD correlation matrices early with a clear error", {
  set.seed(99)
  n <- 50L
  means <- c(3, 3, 3)
  sds <- c(1, 1, 1)
  lb <- c(1, 1, 1)
  ub <- c(5, 5, 5)

  badCor <- matrix(c(
    1, -0.9, -0.9,
    -0.9, 1, -0.9,
    -0.9, -0.9, 1
  ), nrow = 3, byrow = TRUE)

  testthat::expect_error(
    makeScales(n, means, sds, lb, ub, cormatrix = badCor),
    regexp = "not Positive Semi-Definite|Requested correlations are not possible"
  )
})


testthat::test_that("makeScales: mismatched lengths and dimensions fail fast with a descriptive error", {
  set.seed(1)
  n <- 40L
  means <- c(3, 3, 3) # length 3
  sds <- c(1, 1) # length 2 -> mismatch
  lb <- 1
  ub <- 5
  items <- 5L
  corMat <- diag(3)

  testthat::expect_error(
    makeScales(n, means, sds, lb, ub, items, cormatrix = corMat),
    regexp = "sds' must be a numeric vector of positive.*finite values of length 3"
  )
})


testthat::test_that("makeScales: NA/Inf inputs are rejected", {
  # testthat::skip("Enable after adding explicit NA/Inf/type checks that stop().")

  n <- 20L
  means <- c(3, NA)
  sds <- c(1, 1)
  lb <- c(1, 1)
  ub <- c(5, 5)
  corMat <- diag(2)

  testthat::expect_error(
    makeScales(n, means, sds, lb, ub, cormatrix = corMat),
    regexp = "missing|NA|finite|numeric"
  )

  means2 <- c(3, Inf)
  testthat::expect_error(
    makeScales(n, means2, sds, lb, ub, cormatrix = corMat),
    regexp = "finite"
  )
})

testthat::test_that("makeScales: lowerbound < upperbound and within-range means are enforced", {
  # testthat::skip("Enable after adding bound order and mean-range checks that stop().")

  n <- 30L
  means <- c(6, 2) # first mean outside [lb, ub] if lb=1,ub=5
  sds <- c(1, 1)
  lb <- c(1, 3)
  ub <- c(5, 2) # invalid: lb >= ub in 2nd position
  corMat <- diag(2)

  testthat::expect_error(
    makeScales(n, means, sds, lb, ub, cormatrix = corMat),
    regexp = "All 'lowerbound' values must be strictly less than the corresponding 'upperbound' values."
  )


  lb2 <- c(1, 1)
  ub2 <- c(5, 5)
  testthat::expect_error(
    makeScales(n, means, sds, lb2, ub2, cormatrix = corMat),
    regexp = "means.*within.*bounds|outside"
  )
})

#' Rearrange elements in each column of a data-frame to fit a
#' predefined correlation matrix
#'
#' @name lcor
#'
#' @useDynLib LikertMakeR
#'
#' @importFrom Rcpp sourceCpp
#'
#' @description `lcor()` rearranges values in each column of a
#' data-frame so that columns are correlated to match a predefined
#' correlation matrix.
#'
#' @details Values in a column do not change, so univariate
#' statistics remain the same.
#'
#' @param data dataframe that is to be rearranged
#'
#' @param target target correlation matrix.
#' Must have same dimensions as number of columns in data-frame.
#'
#' @param passes Number of optimization passes (default = 10).
#' Increasing this value *MAY* improve results if n-columns
#' (target correlation matrix dimensions) are many.
#' Decreasing the value for 'passes' is faster but may decrease accuracy.
#'
#' @return Returns a dataframe whose column-wise correlations
#' approximate a user-specified correlation matrix
#'
#' @examples
#'
#' ## parameters
#' n <- 32
#' lowerbound <- 1
#' upperbound <- 5
#' items <- 5
#'
#' mydat3 <- data.frame(
#'   x1 = lfast(n, 2.5, 0.75, lowerbound, upperbound, items),
#'   x2 = lfast(n, 3.0, 1.50, lowerbound, upperbound, items),
#'   x3 = lfast(n, 3.5, 1.00, lowerbound, upperbound, items)
#' )
#'
#' cor(mydat3) |> round(3)
#'
#' tgt3 <- matrix(
#'   c(
#'     1.00, 0.50, 0.75,
#'     0.50, 1.00, 0.25,
#'     0.75, 0.25, 1.00
#'   ),
#'   nrow = 3, ncol = 3
#' )
#'
#' ## apply function
#' new3 <- lcor(mydat3, tgt3)
#'
#' ## test output
#' cor(new3) |> round(3)
#'
#' @importFrom stats cor rbeta
#'
#' @export
lcor <- function(data, target, passes = 10) {
  .Call("_LikertMakeR_lcor_C_randomised", data, target, passes,
    PACKAGE = "LikertMakeR"
  ) |>
    data.frame()
}

## Unit tests for lcor() function


test_that("lcor runs silently with valid inputs", {
  n <- 32


  tgt4 <- matrix(
    c(
      1.00, 0.50, 0.50, 0.75,
      0.50, 1.00, 0.25, 0.65,
      0.50, 0.25, 1.00, 0.80,
      0.75, 0.65, 0.80, 1.00
    ),
    nrow = 4
  )

  set.seed(42)

  dat4 <- data.frame(
    x1 = sample(c(1:5), n, replace = TRUE),
    x2 = sample(c(1:5), n, replace = TRUE),
    x3 = sample(c(1:5), n, replace = TRUE),
    x4 = sample(c(1:5), n, replace = TRUE)
  )


  expect_silent(
    lcor(data = dat4, target = tgt4)
  )
})


test_that("lcor runs without errors", {
  n <- 32


  tgt4 <- matrix(
    c(
      1.00, 0.50, 0.50, 0.75,
      0.50, 1.00, 0.25, 0.65,
      0.50, 0.25, 1.00, 0.80,
      0.75, 0.65, 0.80, 1.00
    ),
    nrow = 4
  )

  set.seed(42)

  dat4 <- data.frame(
    x1 = sample(c(1:5), n, replace = TRUE),
    x2 = sample(c(1:5), n, replace = TRUE),
    x3 = sample(c(1:5), n, replace = TRUE),
    x4 = sample(c(1:5), n, replace = TRUE)
  )


  expect_error(
    lcor(data = dat4, target = tgt4),
    regexp = NA
  )
})

#' Correlation matrix from Cronbach's Alpha
#'
#'
#' Generate a Positive-Definite
#' Correlation Matrix for a target _Cronbach's alpha_.
#'
#' Constructs a correlation matrix with a specified number of items and
#' target _Cronbach's alpha_ using a constructive one-factor model.
#'
#' Such a correlation matrix can be applied to the [makeScales()]
#' function to generate synthetic data with the predefined alpha.
#'
#' The algorithm directly builds a positive-definite correlation matrix
#' by solving for item loadings that reproduce the desired average
#' inter-item correlation implied by _alpha_.
#' Unlike earlier versions of this function, this method guarantees positive
#' definiteness by construction, without _post-hoc_ repair.
#'
#' @param items Integer. Number of items (>= 2).
#' @param alpha Numeric. Target Cronbach's alpha (0 < alpha < 1).
#' @param variance Numeric. Controls heterogeneity of item loadings in the
#'   underlying one-factor model.
#'
#'   Larger values produce greater dispersion among item loadings, which
#'   results in a wider spread of inter-item correlations while preserving
#'   the requested Cronbach's alpha.
#'
#'   Typical guidance:
#'   \itemize{
#'     \item \code{0.05} — near-parallel items (very similar correlations)
#'     \item \code{0.10} — modest heterogeneity (default)
#'     \item \code{0.15} — strong heterogeneity
#'     \item \code{0.20} — very strong heterogeneity
#'     \item \code{> 0.25} — extreme dispersion; internal shrinkage may occur
#'   }
#'
#'   For most applied psychometric scales (k < 20), values between
#'   \code{0.05} and \code{0.15} produce realistic correlation structures.
#'   Values above \code{0.30} are automatically reduced to \code{0.30} to
#'   satisfy algorithm constraints.
#'
#' @param alpha_noise Numeric. Controls random variation in the target
#'   Cronbach's alpha before the correlation matrix is constructed.
#'
#'   When \code{alpha_noise = 0} (default), the requested alpha is
#'   reproduced deterministically (subject to numerical tolerance).
#'
#'   When \code{alpha_noise > 0}, a small amount of random variation is
#'   added to the requested alpha prior to constructing the matrix. This
#'   can be useful in teaching or simulation settings where slightly
#'   different reliability values are desired across repeated runs.
#'
#'   Internally, noise is added on the Fisher \emph{z}-transformed scale
#'   to ensure the resulting alpha remains within valid bounds (0, 1).
#'
#'   Typical guidance:
#'   \itemize{
#'     \item \code{0.00} — deterministic alpha (default)
#'     \item \code{0.02} — very small variation
#'     \item \code{0.05} — moderate variation
#'     \item \code{0.10} — substantial variation (caution)
#'   }
#'
#'   Larger values increase the spread of achieved alpha across runs.
#'
#' @param diagnostics Logical. If \code{TRUE}, returns a list containing
#'   the matrix and diagnostic information.
#'
#' @details
#' The function computes the average inter-item correlation implied by the
#' requested alpha and solves for a one-factor loading structure that
#' reproduces this value. A small adaptive reduction in dispersion may be
#' applied when necessary to ensure a valid positive-definite solution.
#'
#' The constructive generator assumes a single common factor structure,
#' consistent with typical psychometric scale construction.
#'
#'
#' @importFrom stats rnorm sd uniroot
#'
#' @return
#' If \code{diagnostics = FALSE}, a positive-definite correlation matrix.
#' If \code{diagnostics = TRUE}, a list containing:
#' \itemize{
#'   \item \code{R}: the correlation matrix
#'   \item \code{diagnostics}: list including achieved alpha,
#'         minimum eigenvalue, and internal variance used
#' }
#'
#'
#' @examples
#'
#' # Example 1
#' # define parameters
#' items <- 4
#' alpha <- 0.85
#'
#' # apply function
#' set.seed(42)
#' cor_matrix <- makeCorrAlpha(
#'   items = items,
#'   alpha = alpha
#' )
#'
#' # test function output
#' print(cor_matrix) |> round(3)
#' alpha(cor_matrix)
#' eigenvalues(cor_matrix, 1)
#'
#' # Example 2
#' # higher alpha, more items, more variability
#' cor_matrix2 <- makeCorrAlpha(
#'   items = 8,
#'   alpha = 0.95,
#'   variance = 0.10
#' )
#'
#' # test output
#' cor_matrix2 |> round(2)
#' alpha(cor_matrix2) |> round(3)
#' eigenvalues(cor_matrix2, 1) |> round(3)
#'
#' # Example 3
#' # large random variation around alpha
#' cor_matrix3 <- makeCorrAlpha(
#'   items = 6,
#'   alpha = 0.85,
#'   alpha_noise = 0.10
#' )
#'
#' # test output
#' cor_matrix3 |> round(2)
#' alpha(cor_matrix3) |> round(3)
#' eigenvalues(cor_matrix3, 1) |> round(3)
#'
#' # Example 4
#' # with diagnostics
#' cor_matrix4 <- makeCorrAlpha(
#'   items = 4,
#'   alpha = 0.80,
#'   diagnostics = TRUE
#' )
#'
#' # test output
#' cor_matrix4
#'
#' @export
makeCorrAlpha <- function(items,
                          alpha,
                          variance = 0.10,
                          alpha_noise = 0,
                          diagnostics = FALSE) {
  k <- items
  if (k < 2) stop("items must be >= 2")

  # -------------------------------
  # Base target mean correlation
  # -------------------------------
  if (!is.numeric(alpha) || length(alpha) != 1 || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single value between 0 and 1.", call. = FALSE)
  }

  target_mean_r <- alpha / (k - alpha * (k - 1))

  if (target_mean_r <= 0) {
    stop("Target alpha implies non-positive mean correlation.")
  }


  # -------------------------------
  # add noise around alpha if wanted
  # -------------------------------

  if (alpha_noise > 0) {
    z_alpha <- atanh(alpha)
    z_alpha_star <- rnorm(1, mean = z_alpha, sd = alpha_noise)
    alpha_target_effective <- tanh(z_alpha_star)
    alpha_target_effective <- max(min(alpha_target_effective, 0.999), 0.001)
  } else {
    alpha_target_effective <- alpha
  }

  target_mean_r <- alpha_target_effective /
    (k - alpha_target_effective * (k - 1))

  ## reporting accuracy
  tolerance <- 0.005
  min_eigen_threshold <- 1e-5


  # variation in correlations

  ## existing code may have old variance level - warn once and continue
  if (variance > 0.30) {
    warning(
      "`variance` cannot exceed 0.30 for the current algorithm; values above this are truncated.",
      call. = FALSE
    )
    variance <- 0.30
  }

  internal_variance <- variance

  best_result <- NULL
  shrink_used <- FALSE

  # -------------------------------
  # Constructive generator
  # -------------------------------

  for (shrink_level in 1:6) {
    for (attempt in 1:3) {
      z <- rnorm(k)
      z <- z - mean(z)
      z <- z / sd(z)
      z <- z * internal_variance

      a_min <- max(-1 - z)
      a_max <- min(1 - z)

      if (a_min >= a_max) next

      lower <- max(0, a_min) # positive-manifold assumption
      upper <- a_max

      mean_r_given_a <- function(a) {
        lambda <- a + z
        if (any(abs(lambda) >= 1)) {
          return(Inf)
        }
        R_tmp <- outer(lambda, lambda)
        mean(R_tmp[lower.tri(R_tmp)])
      }

      f <- function(a) mean_r_given_a(a) - target_mean_r

      if (f(lower) * f(upper) > 0) next

      a_sol <- tryCatch(
        uniroot(f, interval = c(lower, upper))$root,
        error = function(e) NULL
      )

      if (is.null(a_sol)) next

      lambda <- a_sol + z
      if (any(abs(lambda) >= 1)) next

      psi <- 1 - lambda^2
      R <- outer(lambda, lambda) + diag(psi)

      eigvals <- eigen(R, symmetric = TRUE, only.values = TRUE)$values
      min_eig <- min(eigvals)

      avg_r <- mean(R[lower.tri(R)])

      alpha_achieved <- (k * avg_r) / (1 + (k - 1) * avg_r)
      alpha_diff <- alpha_achieved - alpha_target_effective


      if (!is.na(min_eig) &&
        min_eig > min_eigen_threshold &&
        abs(alpha_diff) <= tolerance) {
        best_result <- list(
          R = R,
          alpha_achieved = alpha_achieved,
          min_eigen = min_eig,
          internal_variance = internal_variance
        )

        break
      }
    }

    if (!is.null(best_result)) break

    internal_variance <- internal_variance * 0.85
    shrink_used <- TRUE
  }

  if (is.null(best_result)) {
    stop("Unable to construct feasible correlation matrix for these parameters.")
  }

  cor_matrix <- best_result$R

  item_names <- sprintf("item%02d", seq_len(k))
  rownames(cor_matrix) <- colnames(cor_matrix) <- item_names

  if (shrink_used) {
    warning("Requested variance was reduced internally to ensure feasibility.")
  }

  # message(paste0("Achieved alpha = ", best_result$alpha_achieved |> round(3)))

  if (!diagnostics) {
    return(cor_matrix)
  } else {
    return(list(
      R = cor_matrix,
      diagnostics = list(
        items = items,
        alpha_input = alpha,
        alpha_target_effective = alpha_target_effective,
        alpha_achieved = best_result$alpha_achieved,
        average_r = mean(cor_matrix[lower.tri(cor_matrix)]),
        min_eigenvalue = best_result$min_eigen,
        variance_input = variance,
        internal_variance_used = best_result$internal_variance,
        alpha_noise = alpha_noise
      )
    ))
  }
}

## Unit tests for makeCorrAlpha() function

test_that("makeCorrAlpha runs silently with valid inputs", {
  ## define parameters

  items <- 4
  alpha <- 0.85
  variance <- 0.1

  ## apply function tests

  expect_no_warning(makeCorrAlpha(items, alpha, variance))

  expect_no_warning(makeCorrAlpha(items, alpha))
})


test_that("makeCorrAlpha approximates requested alpha", {
  target <- 0.8

  R <- makeCorrAlpha(items = 6, alpha = target)

  a <- LikertMakeR::alpha(R)

  expect_equal(a, target, tolerance = 0.02)
})

#' Synthesise a dataset from paired-sample t-test summary statistics
#'
#'
#' Generates a dataset from paired-sample *t*-test summary statistics.
#'
#' `makePaired()` generates correlated values so the data replicate
#' rating scales taken, for example, in a before and after experimental design.
#'
#' The function is effectively a wrapper function for
#' `lfast()` and `lcor()` with the addition of a
#' t-statistic from which the between-column correlation is inferred.
#'
#' Paired *t*-tests apply to observations that are associated with each other.
#' For example: the same people before and after a treatment;
#' the same people rating two different objects; ratings by husband & wife;
#' _etc._
#'
#' The paired-samples *t*-test is defined as:
#'
#' \deqn{ t = \frac{\mathrm{mean}(D)}{\mathrm{sd}(D) / \sqrt{n}} }
#'
#' where:
#'
#' - \eqn{D} = differences in values
#' - \eqn{\mathrm{mean}(D)} = mean of the differences
#' - \eqn{\mathrm{sd}(D)} = standard deviation of the differences, where
#'
#' \deqn{ \mathrm{sd}(D)^2 = \mathrm{sd}(X_{\text{before}})^2 +
#'                           \mathrm{sd}(X_{\text{after}})^2 -
#'                           2\,\mathrm{cov}(X_{\text{before}},
#'                           X_{\text{after}}) }
#'
#' A paired-sample *t*-test thus requires an estimate of the covariance between
#' the two sets of observations.
#' `makePaired()` rearranges these formulae so that the covariance is
#' inferred from the t-statistic.
#'
#'
#' @param n (positive, integer) sample size
#' @param means (real) 1:2 vector of target means for two before/after measures
#' @param sds (real) 1:2 vector of target standard deviations
#' @param t_value (real) desired paired *t*-statistic
#' @param lowerbound (integer) lower bound (e.g. '1' for a 1-5 rating scale)
#' @param upperbound (integer) upper bound (e.g. '5' for a 1-5 rating scale)
#' @param items (positive, integer) number of items in the rating scale.
#' Default = 1
#' @param precision (positive, real) relaxes the level of accuracy required.
#' Default = 0
#'
#' @return a dataframe approximating user-specified conditions.
#'
#' @importFrom stats rbeta
#' @importFrom Rcpp sourceCpp
#'
#'
#' @note
#'
#' Larger sample sizes usually result in higher t-statistics,
#' and correspondingly small p-values.
#'
#' Small sample sizes with relatively large standard deviations and
#' relatively high t-statistics can result in impossible correlation values.
#'
#' Similarly, large sample sizes with low *t*-statistics can result in
#' impossible correlations. That is, a correlation outside of the -1:+1 range.
#'
#' If this happens, the function will fail with an _ERROR_ message.
#' The user should review the input parameters and insert more realistic values.
#'
#' @seealso \code{\link{lfast}}, \code{\link{lcor}}
#'
#' @examples
#'
#' n <- 20
#' pair_m <- c(2.5, 3.0)
#' pair_s <- c(1.0, 1.5)
#' lower <- 1
#' upper <- 5
#' k <- 6
#' t <- -2.5
#'
#' pairedDat <- makePaired(
#'   n = n, means = pair_m, sds = pair_s,
#'   t_value = t,
#'   lowerbound = lower, upperbound = upper, items = k
#' )
#'
#' str(pairedDat)
#' cor(pairedDat) |> round(2)
#'
#' t.test(pairedDat$X1, pairedDat$X2, paired = TRUE)
#'
#' @export
#'
makePaired <- function(n, means, sds, t_value,
                       lowerbound, upperbound,
                       items = 1, precision = 0) {
  ## means is a [1:2] vector with before:after mean values
  ## sds is a [1:2] vector with before:after standard deviation values

  mean_before <- means[1]
  mean_after <- means[2]
  sd_before <- sds[1]
  sd_after <- sds[2]


  ### calculate paired correlation from paired t-test information

  ## The logic:
  ##
  ## The t-statistic is directly related to sd(D):
  ## We know t = mean(D) / (sd(D) / sqrt(n))
  ## We have the t-statistic, the sample size (n), and the means for both
  ## before (mean(X_before)) and after (mean(X_after)), and thus, the mean(D).
  ##
  ## Therefore, we can solve for sd(D):
  ##
  ## sd(D) = mean(D) / (t / sqrt(n)) = mean(D) * sqrt(n) / t
  ##
  ## Now we use the sd(D) relationship:
  ##
  ## sd(D)^2 = sd(X_before)^2 + sd(X_after)^2 - 2 * cov(X_before, X_after)
  ##
  ## Solve for Covariance:
  ## Since we now have sd(D), sd(X_before), and sd(X_after),
  ## we can rearrange to find the cov(X_before, X_after):
  ##
  ## cov(X_before, X_after) = (sd(X_before)^2 + sd(X_after)^2 - sd(D)^2) / 2
  ##
  ## Finally, Calculate the Correlation:
  ## calculate the correlation from the covariance and standard deviations to :
  ##
  ## r = cov(X_before, X_after) / (sd(X_before) * sd(X_after))
  ##
  ## NOTES
  ## Small sample sizes with relatively large standard deviations and
  ## relatively high t-statistics can result in impossible correlation values.
  ## Similarly, large sample sizes with low t-statistics
  ## can result in impossible correlations.

  ## Begin paired covariance/correlation function
  calculate_paired_correlation <- function(n, mean_before, mean_after,
                                           sd_before, sd_after, t_statistic) {
    ## Error handling
    if (n <= 3) {
      stop("Sample size must be greater than 3 for a paired t-test.")
    }

    ## Calculate the mean of the differences
    mean_diff <- mean_before - mean_after

    ## Calculate the standard deviation of the differences
    sd_diff <- mean_diff / (t_statistic / sqrt(n))

    ## Check if sd_diff is complex
    if (is.complex(sd_diff)) {
      stop("Error calculating standard deviation of differences.
           \nCheck your t-statistic, means, and sample size.
           \nsd_diff is complex, likely indicating error in input variables")
    }

    ## Calculate the covariance
    covariance <- (sd_before^2 + sd_after^2 - sd_diff^2) / 2

    ## Calculate the correlation
    correlation <- covariance / (sd_before * sd_after)

    ## Validate correlation
    if (correlation > 1 || correlation < -1) {
      stop(paste0("Inputs are inconsistent. \nSmaller (larger) sample size
                  usually means lower (higher) t-value."))
    }

    return(correlation)
  } ## END paired covariance/correlation function


  t_c <- calculate_paired_correlation(
    n = n,
    mean_before = mean_before, mean_after = mean_after,
    sd_before = sd_before, sd_after = sd_after,
    t_statistic = t_value
  )

  ## define target correlation matrix for paired comparisons
  t_cor <- matrix(
    c(
      1, t_c,
      t_c, 1
    ),
    ncol = 2, nrow = 2
  )

  ## define initial data
  message(paste0("Initial data vectors"))
  startDat <- data.frame(
    x1 = lfast(
      n, mean_before, sd_before,
      lowerbound, upperbound,
      items, precision
    ),
    x2 = lfast(
      n, mean_after, sd_after,
      lowerbound, upperbound,
      items, precision
    )
  )

  message("Arranging values to conform with desired t-value")
  correlatedDat <- lcor(data = startDat, target = t_cor)
  message("Complete!")

  return(correlatedDat)
}

## Unit tests for makePaired() function

test_that("makePaired runs without error with valid inputs", {
  n <- 20
  means <- c(2.5, 3.0)
  sds <- c(1.0, 1.5)
  lowerbound <- 1
  upperbound <- 5
  items <- 6
  t <- -2.5

  expect_no_error(
    pairedDat <- makePaired(
      n = n,
      means = means,
      sds = sds,
      lowerbound = lowerbound,
      upperbound = upperbound,
      items = items,
      t_value = t
    )
  )
})


test_that("makePaired runs with errors with unrealistic inputs", {
  n <- 64
  means <- c(2.5, 3.0)
  sds <- c(1.0, 1.5)
  lowerbound <- 1
  upperbound <- 5
  items <- 6
  t <- 0.5

  expect_error(
    pairedDat <- makePaired(
      n = n,
      means = means,
      sds = sds,
      lowerbound = lowerbound,
      upperbound = upperbound,
      items = items,
      t_value = t
    )
  )
})

#' Reproduce Repeated-Measures Data from ANOVA Summary Statistics
#'
#' @description
#' Constructs a synthetic dataset and inter-timepoint correlation matrix
#' from a repeated-measures ANOVA result, based on reported means, standard
#' deviations, and an F-statistic. This is useful when only summary statistics
#' are available from published studies.
#'
#' @details
#' This function estimates the average correlation between repeated measures
#' by matching the reported F-statistic, under one of three assumed
#' correlation structures:
#'
#' - `"cs"` (*Compound Symmetry*): The Default. Assumes all timepoints are
#' equally correlated. Common in standard RM-ANOVA settings.
#' - `"ar1"` (*First-Order Autoregressive*): Assumes correlations decay
#' exponentially with time lag.
#' - `"toeplitz"` (*Linearly Decreasing*): Assumes correlation declines
#' linearly with time lag - a middle ground between `"cs"` and `"ar1"`.
#'
#' The function then generates a data frame of synthetic item-scale ratings
#' using [lfast], and adjusts them to match the estimated correlation
#' structure using [lcor].
#'
#' Set `return_corr_only = TRUE` to extract only the estimated
#' correlation matrix.
#'
#' @param n Integer. Sample size used in the original study.
#' @param k Integer. Number of repeated measures (timepoints).
#' @param means Numeric vector of length `k`.
#' Mean values reported for each timepoint.
#' @param sds Numeric vector of length `k`.
#' Standard deviations reported for each timepoint.
#' @param f_stat Numeric. The reported repeated-measures ANOVA
#' F-statistic for the within-subjects factor.
#' @param df_between Degrees of freedom between conditions (default: `k - 1`.
#' @param df_within Degrees of freedom within-subjects
#' (default: `(n - 1) * (k - 1)`).
#' @param structure Character. Correlation structure to assume:
#' `"cs"`, `"ar1"`, or `"toeplitz"` (default = `"cs"`).
#' @param names Character vector of length `k`. Variable names for each
#' timepoint (default: `"time_1"` to `"time_k"`).
#' @param items Integer. Number of items used to generate each scale score
#' (passed to [lfast]).
#' @param lowerbound, Integer. Lower bounds for Likert-type response
#' scales (default: 1).
#' @param upperbound, Integer. upper bounds for Likert-type response
#' scales (default: 5).
#' @param return_corr_only Logical. If `TRUE`, return only the
#' estimated correlation matrix.
#' @param diagnostics Logical. If `TRUE`, include diagnostic summaries
#' such as feasible F-statistic range and effect sizes.
#' @param ... Reserved for future use.
#'
#' @return A named list with components:
#'
#' \describe{
#'   \item{`data`}{A data frame of simulated repeated-measures responses
#'     (unless `return_corr_only = TRUE`).}
#'   \item{`correlation_matrix`}{The estimated inter-timepoint correlation matrix.}
#'   \item{`structure`}{The correlation structure assumed.}
#'   \item{`achieved_f`}{The F-statistic produced by the estimated `rho` value
#'     (if `diagnostics = TRUE`).}
#'   \item{`feasible_f_range`}{Minimum and maximum achievable F-values under the
#'     chosen structure (shown if diagnostics are requested).}
#'   \item{`recommended_f`}{Conservative, moderate, and strong F-statistic
#'     suggestions for similar designs.}
#'   \item{`effect_size_raw`}{Unstandardised effect size across timepoints.}
#'   \item{`effect_size_standardised`}{Effect size standardised by average variance.}
#' }
#'
#' @examples
#'
#' set.seed(42)
#'
#' out1 <- makeRepeated(
#'   n = 64,
#'   k = 3,
#'   means = c(3.1, 3.5, 3.9),
#'   sds = c(1.0, 1.1, 1.0),
#'   items = 4,
#'   f_stat = 4.87,
#'   structure = "cs",
#'   diagnostics = FALSE
#' )
#'
#' head(out1$data)
#' out1$correlation_matrix
#'
#' out2 <- makeRepeated(
#'   n = 32, k = 4,
#'   means = c(2.75, 3.5, 4.0, 4.4),
#'   sds = c(0.8, 1.0, 1.2, 1.0),
#'   f_stat = 16,
#'   structure = "ar1",
#'   items = 5,
#'   lowerbound = 1, upperbound = 7,
#'   return_corr_only = FALSE,
#'   diagnostics = TRUE
#' )
#'
#' print(out2)
#'
#'
#' out3 <- makeRepeated(
#'   n = 64, k = 4,
#'   means = c(2.0, 2.25, 2.75, 3.0),
#'   sds = c(0.8, 0.9, 1.0, 0.9),
#'   items = 4,
#'   f_stat = 24,
#'   # structure = "toeplitz",
#'   diagnostics = TRUE
#' )
#'
#' str(out3)
#'
#' @seealso \code{\link{lfast}}, \code{\link{lcor}}
#'
#' @importFrom stats median optimize
#'
#' @export
#'
makeRepeated <- function(n, k, means, sds,
                         f_stat,
                         df_between = k - 1,
                         df_within = (n - 1) * (k - 1),
                         structure = c("cs", "ar1", "toeplitz"),
                         names = paste0("time_", 1:k),
                         items = 1,
                         lowerbound = 1, upperbound = 5,
                         return_corr_only = FALSE,
                         diagnostics = FALSE,
                         ...) {
  # Input validation
  structure <- match.arg(structure)

  if (length(means) != k) {
    stop("Length of 'means' must equal k (number of time points)")
  }
  if (length(sds) != k) {
    stop("Length of 'sds' must equal k (number of time points)")
  }
  if (length(names) != k) {
    stop("Length of 'names' must equal k (number of time points)")
  }
  if (n <= 1 || k <= 1) {
    stop("Sample size (n) and number of time points (k) must be > 1")
  }
  if (f_stat <= 0) {
    stop("F-statistic must be positive")
  }

  #####
  ##  Helper functions
  ####

  ##
  ## estimate_rho() calculates mean correlation coefficient
  ##
  estimate_rho <- function(f_stat, df_between, df_within,
                           n, k, means, sds,
                           structure = c("cs", "ar1", "toeplitz"),
                           tolerance = 1e-6) {
    structure <- match.arg(structure)

    ## Create correlation matrix based on structure
    create_corr_matrix <- function(rho, structure, k) {
      R <- matrix(1, nrow = k, ncol = k)

      if (structure == "cs") {
        R[upper.tri(R) | lower.tri(R)] <- rho
      } else if (structure == "ar1") {
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              R[i, j] <- rho^abs(i - j)
            }
          }
        }
      } else if (structure == "toeplitz") {
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              # Improved linear decrease: ensure non-negative correlations
              distance <- abs(i - j)
              R[i, j] <- max(0, rho * (1 - distance / (k - 1)))
            }
          }
        }
      }
      return(R)
    }

    # Calculate expected F-statistic from correlation matrix
    calculate_expected_f <- function(rho, means, sds, n, k, structure) {
      R <- create_corr_matrix(rho, structure, k)

      # Check positive definiteness
      eigenvals <- eigen(R, only.values = TRUE)$values
      if (any(eigenvals <= tolerance)) {
        return(Inf)
      }

      grand_mean <- mean(means)
      SS_between <- n * sum((means - grand_mean)^2)

      # Average correlation calculation
      if (structure == "cs") {
        avg_corr <- rho
      } else if (structure == "ar1") {
        corr_sum <- 0
        count <- 0
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              corr_sum <- corr_sum + rho^abs(i - j)
              count <- count + 1
            }
          }
        }
        avg_corr <- corr_sum / count
      } else if (structure == "toeplitz") {
        corr_sum <- 0
        count <- 0
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              distance <- abs(i - j)
              corr_val <- max(0, rho * (1 - distance / (k - 1)))
              corr_sum <- corr_sum + corr_val
              count <- count + 1
            }
          }
        }
        avg_corr <- corr_sum / count
      }

      pooled_var <- mean(sds^2)
      SS_error <- (n - 1) * k * pooled_var * (1 - avg_corr)

      MS_between <- SS_between / df_between
      MS_error <- SS_error / df_within
      F_calc <- MS_between / MS_error

      return(F_calc)
    }

    # Objective function
    objective <- function(rho) {
      expected_f <- calculate_expected_f(rho, means, sds, n, k, structure)
      return((expected_f - f_stat)^2)
    }

    # Set bounds based on structure
    if (structure == "cs") {
      lower_bound <- max(-0.99, -1 / (k - 1) + 0.01)
      upper_bound <- 0.99
    } else if (structure == "ar1") {
      lower_bound <- -0.99
      upper_bound <- 0.99
    } else if (structure == "toeplitz") {
      # For improved Toeplitz, be more conservative to ensure positive definiteness
      lower_bound <- -0.8
      upper_bound <- 0.99
    }

    # Optimize
    result <- optimize(objective, interval = c(lower_bound, upper_bound))
    final_f <- calculate_expected_f(result$minimum, means, sds, n, k, structure)
    convergence_check <- abs(final_f - f_stat) / f_stat

    return(list(
      rho = result$minimum,
      convergence = convergence_check < 0.01,
      final_f_stat = final_f,
      target_f_stat = f_stat,
      objective_value = result$objective,
      structure = structure,
      correlation_matrix = create_corr_matrix(result$minimum, structure, k)
    ))
  } ## END estimate rho function

  ##
  ### Simplified diagnostic function
  ##
  diagnose_f_range <- function(n, k, means, sds,
                               structure = c("cs", "ar1", "toeplitz"),
                               rho_range = c(-0.8, 0.8), n_points = 20) {
    structure <- match.arg(structure)

    # Helper function to calculate expected F
    calculate_expected_f <- function(rho, means, sds, n, k, structure) {
      # Create correlation matrix
      R <- matrix(1, nrow = k, ncol = k)
      if (structure == "cs") {
        R[upper.tri(R) | lower.tri(R)] <- rho
      } else if (structure == "ar1") {
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              R[i, j] <- rho^abs(i - j)
            }
          }
        }
      } else if (structure == "toeplitz") {
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              distance <- abs(i - j)
              R[i, j] <- max(0, rho * (1 - distance / (k - 1)))
            }
          }
        }
      }

      # Check positive definiteness
      eigenvals <- eigen(R, only.values = TRUE)$values
      if (any(eigenvals <= 1e-6)) {
        return(NA)
      }

      grand_mean <- mean(means)
      SS_between <- n * sum((means - grand_mean)^2)

      # Average correlation calculation
      if (structure == "cs") {
        avg_corr <- rho
      } else if (structure == "ar1") {
        corr_sum <- 0
        count <- 0
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              corr_sum <- corr_sum + rho^abs(i - j)
              count <- count + 1
            }
          }
        }
        avg_corr <- corr_sum / count
      } else if (structure == "toeplitz") {
        corr_sum <- 0
        count <- 0
        for (i in 1:k) {
          for (j in 1:k) {
            if (i != j) {
              distance <- abs(i - j)
              corr_val <- max(0, rho * (1 - distance / (k - 1)))
              corr_sum <- corr_sum + corr_val
              count <- count + 1
            }
          }
        }
        avg_corr <- corr_sum / count
      }

      pooled_var <- mean(sds^2)
      df_between <- k - 1
      df_within <- (n - 1) * (k - 1)
      SS_error <- (n - 1) * k * pooled_var * (1 - avg_corr)

      MS_between <- SS_between / df_between
      MS_error <- SS_error / df_within
      F_calc <- MS_between / MS_error

      return(F_calc)
    }

    # Set reasonable bounds for rho based on structure
    if (structure == "cs") {
      lower_rho <- max(rho_range[1], -1 / (k - 1) + 0.01)
      upper_rho <- min(rho_range[2], 0.99)
    } else if (structure == "ar1") {
      lower_rho <- max(rho_range[1], -0.99)
      upper_rho <- min(rho_range[2], 0.99)
    } else if (structure == "toeplitz") {
      lower_rho <- max(rho_range[1], -0.8)
      upper_rho <- min(rho_range[2], 0.99)
    }

    # Generate sequence of rho values to test
    rho_seq <- seq(lower_rho, upper_rho, length.out = n_points)

    # Calculate F-statistics for each rho
    f_values <- sapply(rho_seq, function(rho) {
      calculate_expected_f(rho, means, sds, n, k, structure)
    })

    # Remove any NA values (from invalid correlation matrices)
    valid_idx <- !is.na(f_values)
    f_values <- f_values[valid_idx]

    # Calculate summary statistics
    f_min <- min(f_values, na.rm = TRUE)
    f_max <- max(f_values, na.rm = TRUE)
    f_median <- median(f_values, na.rm = TRUE)

    # Calculate effect sizes
    grand_mean <- mean(means)
    effect_size_raw <- sum((means - grand_mean)^2) / k
    effect_size_standardised <- effect_size_raw / mean(sds^2)

    # Create simplified results
    return(list(
      feasible_f_range = c(min = f_min, max = f_max),
      recommended_f = list(
        conservative = round((2 * f_min + f_median) / 3, 2),
        moderate = round(f_median, 2),
        strong = round((2 * f_max + f_median) / 3, 2)
      ),
      effect_size_raw = effect_size_raw,
      effect_size_standardised = effect_size_standardised
    ))
  } ## END diagnose_f_range() function

  ##
  ## Main function execution
  ##

  # Estimate correlation parameter
  rho_result <- estimate_rho(
    f_stat = f_stat,
    df_between = df_between,
    df_within = df_within,
    n = n,
    k = k,
    means = means,
    sds = sds,
    structure = structure
  )

  # Check convergence
  if (!rho_result$convergence) {
    warning("Optimization may not have converged. Check results carefully.")
  }

  # Get the correlation matrix
  R <- rho_result$correlation_matrix
  colnames(R) <- rownames(R) <- names

  # If user only wants the correlation matrix, return it
  if (return_corr_only) {
    result <- list(
      correlation_matrix = R,
      structure = structure
    )
    return(result)
  }

  # Generate data using LikertMakeR functions
  likert_data <- data.frame(matrix(nrow = n, ncol = k))
  colnames(likert_data) <- names

  # Generate each column separately with lfast()
  for (i in 1:k) {
    likert_data[, i] <- LikertMakeR::lfast(
      n = n,
      mean = means[i],
      sd = sds[i],
      items = items,
      lowerbound = lowerbound,
      upperbound = upperbound
    )
  }

  # Apply correlation structure to the generated data
  final_data <- LikertMakeR::lcor(data = likert_data, target = R)
  colnames(final_data) <- names

  # Prepare basic output (diagnostics = FALSE)
  result <- list(
    data = final_data,
    correlation_matrix = R,
    structure = structure
  )

  # Add diagnostics if requested (diagnostics = TRUE)
  if (diagnostics) {
    diag_result <- diagnose_f_range(
      n = n, k = k, means = means, sds = sds,
      structure = structure
    )

    result$feasible_f_range <- diag_result$feasible_f_range
    result$recommended_f <- diag_result$recommended_f
    result$achieved_f <- rho_result$final_f_stat
    result$effect_size_raw <- diag_result$effect_size_raw
    result$effect_size_standardised <- diag_result$effect_size_standardised
  }

  return(result)
}

## Unit tests for makeRepeated() function

test_that("makeRepeated returns expected structure", {
  set.seed(42)

  result <- makeRepeated(
    n = 100,
    k = 3,
    means = c(3.1, 3.5, 3.9),
    sds = c(1.0, 1.1, 1.0),
    f_stat = 15,
    structure = "cs",
    diagnostics = TRUE
  )

  # Test class and structure of output
  expect_type(result, "list")
  expect_true(all(c("data", "correlation_matrix", "structure") %in%
    names(result)))

  # Check data frame shape and column names
  expect_s3_class(result$data, "data.frame")
  expect_equal(ncol(result$data), 3)
  expect_equal(nrow(result$data), 100)
  expect_equal(colnames(result$data), paste0("time_", 1:3))

  # Check correlation matrix
  expect_true(is.matrix(result$correlation_matrix))
  expect_equal(dim(result$correlation_matrix), c(3, 3))
  expect_true(all(diag(result$correlation_matrix) == 1))

  # Check structure is correctly labeled
  expect_equal(result$structure, "cs")

  # Check diagnostics elements exist
  expect_true(all(c("feasible_f_range", "recommended_f", "achieved_f") %in%
    names(result)))
  expect_type(result$feasible_f_range, "double")
  expect_type(result$recommended_f, "list")
})

test_that("makeRepeated returns only correlation matrix when requested", {
  result <- makeRepeated(
    n = 50,
    k = 4,
    means = c(2.5, 3.0, 3.2, 3.6),
    sds = c(0.9, 1.0, 1.0, 0.95),
    f_stat = 14.0,
    structure = "ar1",
    return_corr_only = TRUE
  )

  expect_type(result, "list")
  expect_named(result, c("correlation_matrix", "structure"))
  expect_true(is.matrix(result$correlation_matrix))
  expect_equal(dim(result$correlation_matrix), c(4, 4))
})

test_that("makeRepeated catches invalid input", {
  expect_error(
    makeRepeated(
      n = 100, k = 3,
      means = c(3.1, 3.5), sds = c(1, 1, 1),
      f_stat = 4.5
    ),
    "Length of 'means' must equal k"
  )

  expect_error(
    makeRepeated(
      n = 100, k = 3,
      means = c(3.1, 3.5, 3.9), sds = c(1, 1),
      f_stat = 4.5
    ),
    "Length of 'sds' must equal k"
  )

  expect_error(
    makeRepeated(
      n = 100, k = 3,
      means = c(3.1, 3.5, 3.9), sds = c(1, 1, 1),
      f_stat = -2
    ),
    "F-statistic must be positive"
  )

  expect_error(
    makeRepeated(
      n = 1, k = 3,
      means = c(3.1, 3.5, 3.9), sds = c(1, 1, 1),
      f_stat = 4.5
    ),
    "Sample size.*must be > 1"
  )
})

#' Dataframe of correlated scales from different dataframes of scale items
#'
#' @name correlateScales
#'
#' @description `correlateScales()` creates a dataframe of
#' scale items representing correlated constructs,
#' as one might find in a completed questionnaire.
#'
#' @details Correlated rating-scale items generally are summed or
#' averaged to create a measure of an "unobservable", or "latent", construct.
#' `correlateScales()` takes several such dataframes of rating-scale
#' items and rearranges their rows so that the scales are correlated according
#' to a predefined correlation matrix. Univariate statistics for each dataframe
#' of rating-scale items do not change,
#' but their correlations with rating-scale items in other dataframes do.
#'
#' @param dataframes  a list of 'k' dataframes to be rearranged and combined
#' @param scalecors target correlation matrix - should be a symmetric
#' \eqn{k \times k} positive-semi-definite matrix,
#' where 'k' is the number of dataframes
#'
#' @importFrom dplyr bind_cols
#'
#' @return Returns a dataframe whose columns are taken from the starter
#' dataframes and whose summated values are correlated according to a
#' user-specified correlation matrix
#'
#' @examples
#'
#' ## three attitudes and a behavioural intention
#' n <- 32
#' lower <- 1
#' upper <- 5
#'
#' ### attitude #1
#' cor_1 <- makeCorrAlpha(items = 4, alpha = 0.90)
#' means_1 <- c(2.5, 2.5, 3.0, 3.5)
#' sds_1 <- c(0.9, 1.0, 0.9, 1.0)
#'
#' Att_1 <- makeScales(
#'   n = n, means = means_1, sds = sds_1,
#'   lowerbound = rep(lower, 4), upperbound = rep(upper, 4),
#'   items = 4,
#'   cormatrix = cor_1
#' )
#'
#'
#' ### attitude #2
#' cor_2 <- makeCorrAlpha(items = 5, alpha = 0.85)
#' means_2 <- c(2.5, 2.5, 3.0, 3.0, 3.5)
#' sds_2 <- c(1.0, 1.0, 0.9, 1.0, 1.5)
#'
#' Att_2 <- makeScales(
#'   n = n, means = means_2, sds = sds_2,
#'   lowerbound = rep(lower, 5), upperbound = rep(upper, 5),
#'   items = 5,
#'   cormatrix = cor_2
#' )
#'
#'
#' ### attitude #3
#' cor_3 <- makeCorrAlpha(items = 6, alpha = 0.75)
#' means_3 <- c(2.5, 2.5, 3.0, 3.0, 3.5, 3.5)
#' sds_3 <- c(1.0, 1.5, 1.0, 1.5, 1.0, 1.5)
#'
#' Att_3 <- makeScales(
#'   n = n, means = means_3, sds = sds_3,
#'   lowerbound = rep(lower, 6), upperbound = rep(upper, 6),
#'   items = 6,
#'   cormatrix = cor_3
#' )
#'
#'
#' ### behavioural intention
#' intent <- lfast(n, mean = 3.0, sd = 3, lowerbound = 0, upperbound = 10) |>
#'   data.frame()
#' names(intent) <- "int"
#'
#'
#' ### target scale correlation matrix
#' scale_cors <- matrix(
#'   c(
#'     1.0, 0.6, 0.5, 0.3,
#'     0.6, 1.0, 0.4, 0.2,
#'     0.5, 0.4, 1.0, 0.1,
#'     0.3, 0.2, 0.1, 1.0
#'   ),
#'   nrow = 4
#' )
#'
#' data_frames <- list("A1" = Att_1, "A2" = Att_2, "A3" = Att_3, "Int" = intent)
#'
#'
#' ### apply the function
#' my_correlated_scales <- correlateScales(
#'   dataframes = data_frames,
#'   scalecors = scale_cors
#' )
#' head(my_correlated_scales)
#'
#' @export
#'
correlateScales <- function(dataframes, scalecors) {
  ###
  ##   BEGIN helper functions
  ###

  ## Function finding matching sums values and linking rows
  ## Use lookup table for faster matching
  findMatchingRows <- function(sums_vector, df, df_row_sums) {
    n <- nrow(df)
    used_rows <- rep(FALSE, n)

    # Create a lookup table (list) grouped by sum values
    # This avoids repeated searching through the entire vector
    sum_lookup <- split(seq_len(n), df_row_sums)

    # Pre-allocate results matrix for speed (faster than growing dataframe)
    results_matrix <- matrix(nrow = 0, ncol = ncol(df))
    colnames(results_matrix) <- colnames(df)

    for (i in seq_along(sums_vector)) {
      row_sum <- sums_vector[i]

      # Get all rows with this sum value via hash table lookup
      candidate_indices <- sum_lookup[[as.character(row_sum)]]

      if (!is.null(candidate_indices) && length(candidate_indices) > 0) {
        # Find first unused candidate from the small set of candidates
        available <- candidate_indices[!used_rows[candidate_indices]]

        if (length(available) > 0) {
          first_match <- available[1]
          # Add row to results (matrix rbind is faster than dataframe rbind)
          results_matrix <- rbind(results_matrix, df[first_match, , drop = FALSE])
          used_rows[first_match] <- TRUE
        }
      }
    } ## END sums_vector loop

    # Convert matrix back to dataframe
    results_df <- as.data.frame(results_matrix)
    colnames(results_df) <- colnames(df)

    return(results_df)
  } ## END findMatchingRows function


  ## Function to rename columns for their dataframe
  rename_columns <- function(data_list) {
    for (var_name in names(data_list)) {
      new_col_names <- paste(var_name, 1:ncol(data_list[[var_name]]),
        sep = "_"
      )
      colnames(data_list[[var_name]]) <- new_col_names
    }
    return(data_list)
  }

  ## Function to check if all dataframes have the same number of rows
  all_same_rows <- function(data_list) {
    # number of rows from each dataframe
    row_counts <- sapply(data_list, nrow)
    # Check if all values in row_counts are the same
    all_equal <- all(row_counts == row_counts[1])
    return(all_equal)
  } ## END same-row-numbers function

  ## Function to check that all objects are dataframes
  convert_to_df <- function(lst) {
    for (i in seq_along(lst)) {
      if (!is.data.frame(lst[[i]])) {
        warning(paste("Converting object at index ", i, " to a dataframe."))
        lst[[i]] <- data.frame(lst[[i]])
      }
    }
    return(lst)
  } ## END numeric-to-df function

  ## Function to check for valid correlation matrix
  is_valid_correlation_matrix <- function(mat) {
    # Check if the matrix is square
    if (nrow(mat) != ncol(mat)) {
      return(FALSE)
    }

    # Check diagonal elements are all 1
    if (!all(diag(mat) == 1)) {
      return(FALSE)
    }

    # Check all values are between -1 and 1
    if (any(mat < -1 | mat > 1)) {
      return(FALSE)
    }

    # Check the matrix is symmetric
    if (!identical(mat, t(mat))) {
      return(FALSE)
    }

    return(TRUE)
  }

  ###
  ##   END helper functions
  ###


  ###
  ## data integrity checks
  ###
  dataframes <- convert_to_df(dataframes)

  if (all_same_rows(dataframes) == FALSE) {
    stop("Dataframes must have equal number or rows")
  }

  factor_list <- rename_columns(dataframes)

  if (is_valid_correlation_matrix(scalecors) == FALSE) {
    stop("Correlation matrix is not square, symmetrical, or valid values")
  }

  if (min(eigenvalues(scalecors)) < 0) {
    stop("Correlation matrix is not Positive Definite, so not feasible")
  }

  ## END integrity checks


  ## Vectorize row sum calculation using lapply
  ## calculate row sums of the dataframes
  factor_sums <- lapply(factor_list, function(df) {
    rowSums(df)
  })

  ## reorder the sums according  target correlations
  ordered_sums <- lcor(factor_sums, scalecors)

  ## Vectorize row matching using Map (parallel lapply)
  ## allocate dataframe rows to corresponding sum values
  factor <- Map(
    findMatchingRows,
    sums_vector = asplit(ordered_sums, 2), # Split matrix by columns (MARGIN = 2)
    df = factor_list,
    df_row_sums = factor_sums
  )

  ## bring all columns together into one dataframe
  new_df <- dplyr::bind_cols(factor)

  if (!is.null(new_df) && nrow(new_df) > 0) {
    message("New dataframe successfully created")
  }

  return(new_df)
}

#' Calculate Cronbach's Alpha from a correlation matrix or dataframe
#' @name alpha
#' @description `alpha()` calculates Cronbach's Alpha from a given
#'   correlation matrix or a given dataframe.
#'
#' @param cormatrix (real) a square symmetrical matrix with values
#'   ranging from -1 to +1 and '1' in the diagonal
#' @param data (real) a dataframe or matrix
#'
#' @return a single value
#'
#' @seealso \code{\link{alpha_sensitivity}}, \code{\link{reliability}}
#'
#'
#' @examples
#'
#' ## Sample data frame
#' df <- data.frame(
#'   V1  =  c(4, 2, 4, 3, 2, 2, 2, 1),
#'   V2  =  c(4, 1, 3, 4, 4, 3, 2, 3),
#'   V3  =  c(4, 1, 3, 5, 4, 1, 4, 2),
#'   V4  =  c(4, 3, 4, 5, 3, 3, 3, 3)
#' )
#'
#' ## example correlation matrix
#' corMat <- matrix(
#'   c(
#'     1.00, 0.35, 0.45, 0.70,
#'     0.35, 1.00, 0.60, 0.55,
#'     0.45, 0.60, 1.00, 0.65,
#'     0.70, 0.55, 0.65, 1.00
#'   ),
#'   nrow = 4, ncol = 4
#' )
#'
#' ## apply function examples
#'
#' alpha(cormatrix = corMat)
#'
#' alpha(, df)
#'
#' alpha(corMat, df)
#'
#' @export
#'
alpha <- function(cormatrix = NULL, data = NULL) {
  ## input integrity checks
  if (is.null(cormatrix) && is.null(data)) {
    stop(
      "Provide either a correlation matrix or a data argument.",
      call. = FALSE
    )
  } else {
    if (is.null(cormatrix)) {
      cormatrix <- cor(data)
    } else {
      if (!is.null(cormatrix) && !is.null(data)) {
        message("Alert: \nBoth cormatrix and data present.
                \nUsing cormatrix by default.")
      }
    }
  }
  # Catch common user mistake: alpha(df) instead of alpha(, df)
  if (!is.null(cormatrix) && is.null(data)) {
    if (is.data.frame(cormatrix) || is.matrix(cormatrix)) {
      # Check if it's NOT a valid correlation matrix
      is_square <- is.matrix(cormatrix) && nrow(cormatrix) == ncol(cormatrix)
      diag_is_one <- is_square && all(diag(cormatrix) == 1)
      if (!is_square || !diag_is_one) {
        stop(
          "You supplied a dataset as the first argument.\n",
          "Use: alpha(, data = your_data)",
          call. = FALSE
        )
      }
    }
  }
  ## END input integrity checks

  # Calculate Cronbach's Alpha from the correlation matrix
  # find the mean of upper (or lower) triangle
  mean_r <- mean(cormatrix[upper.tri(cormatrix)])

  k <- ncol(cormatrix)
  # calculate alpha
  cronbach_alpha <- (k * mean_r) / (1 + (k - 1) * mean_r)

  return(cronbach_alpha)
}

## Unit tests for alpha() function

test_that("alpha runs silently with valid inputs", {
  ## example data frame
  df <- data.frame(
    V1  =  c(4, 2, 4, 3, 2, 2, 2, 1),
    V2  =  c(4, 1, 3, 4, 4, 3, 2, 3),
    V3  =  c(4, 1, 3, 5, 4, 1, 4, 2),
    V4  =  c(4, 3, 4, 5, 3, 3, 3, 3)
  )

  ## example correlation matrix
  corMat <- matrix(
    c(
      1.00, 0.35, 0.45, 0.70,
      0.35, 1.00, 0.60, 0.55,
      0.45, 0.60, 1.00, 0.65,
      0.70, 0.55, 0.65, 1.00
    ),
    nrow = 4, ncol = 4
  )

  ## apply function examples

  expect_no_warning(alpha(corMat))

  expect_no_warning(alpha(corMat, df))

  expect_no_warning(alpha(NULL, df))
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

lcor_C_randomised <- function(data_df, target, passes = 10L) {
    .Call('_LikertMakeR_lcor_C_randomised', PACKAGE = 'LikertMakeR', data_df, target, passes)
}

score_items_cpp <- function(dat, target_alpha, w_balance, expected_mean) {
    .Call('_LikertMakeR_score_items_cpp', PACKAGE = 'LikertMakeR', dat, target_alpha, w_balance, expected_mean)
}

#' Generate Inter-Item Correlation Matrix from Factor Loadings
#'
#' @description
#' Constructs an inter-item correlation matrix based on a user-supplied
#' matrix of standardised factor loadings and (optionally) a factor
#' correlation matrix.
#' The `makeCorrLoadings()` function does a surprisingly good job of
#' reproducing a target correlation matrix when all item-factor loadings
#' are present, as shown in the _makeCorrLoadings() validation article_.
#'
#' @seealso
#'  - makeCorrLoadings() validation article:
#'  <https://winzarh.github.io/LikertMakeR/articles/makeCorrLoadings_validate.html>
#'  - Package website: <https://winzarh.github.io/LikertMakeR/>
#'
#' @param loadings Numeric matrix. A \eqn{k \times f} matrix of standardized
#' factor loadings \eqn{items \times factors}.
#' Row names and column names are used for diagnostics if present.
#' @param factorCor Optional \eqn{f \times f} matrix of factor
#' correlations (\eqn{\Phi}).
#' If NULL, assumes orthogonal factors.
#' @param uniquenesses Optional vector of length k. If NULL, calculated
#' as \eqn{1 - rowSums(loadings^2)}.
#' @param nearPD Logical.
#' If `TRUE`, attempts to coerce non–positive-definite matrices
#' using [Matrix::nearPD()].
#' @param diagnostics Logical.
#' If `TRUE`, returns diagnostics including McDonald's Omega and
#' item-level summaries.
#'
#' @return If diagnostics = FALSE, returns a correlation matrix (class: matrix).
#'         If diagnostics = TRUE, returns a list with:
#'         - R: correlation matrix
#'         - Omega: per-factor Omega or adjusted Omega
#'         - OmegaTotal: total Omega across all factors
#'         - Diagnostics: dataframe of communalities, uniquenesses, and primary factor
#'
#' @examples
#'
#' # --------------------------------------------------------
#' # Example 1: Basic use without diagnostics
#' # --------------------------------------------------------
#'
#' factorLoadings <- matrix(
#'   c(
#'     0.05, 0.20, 0.70,
#'     0.10, 0.05, 0.80,
#'     0.05, 0.15, 0.85,
#'     0.20, 0.85, 0.15,
#'     0.05, 0.85, 0.10,
#'     0.10, 0.90, 0.05,
#'     0.90, 0.15, 0.05,
#'     0.80, 0.10, 0.10
#'   ),
#'   nrow = 8, ncol = 3, byrow = TRUE
#' )
#'
#' rownames(factorLoadings) <- paste0("Q", 1:8)
#' colnames(factorLoadings) <- c("Factor1", "Factor2", "Factor3")
#'
#' factorCor <- matrix(
#'   c(
#'     1.0,  0.7, 0.6,
#'     0.7,  1.0, 0.4,
#'     0.6,  0.4, 1.0
#'   ),
#'   nrow = 3, byrow = TRUE
#' )
#'
#' itemCor <- makeCorrLoadings(factorLoadings, factorCor)
#' round(itemCor, 3)
#'
#' # --------------------------------------------------------
#' # Example 2: Diagnostics with factor correlations (Adjusted Omega)
#' # --------------------------------------------------------
#'
#' result_adj <- makeCorrLoadings(
#'   loadings = factorLoadings,
#'   factorCor = factorCor,
#'   diagnostics = TRUE
#' )
#'
#' # View outputs
#' round(result_adj$R, 3) # correlation matrix
#' round(result_adj$Omega, 3) # adjusted Omega
#' round(result_adj$OmegaTotal, 3) # total Omega
#' print(result_adj$Diagnostics) # communality and uniqueness per item
#'
#' # --------------------------------------------------------
#' # Example 3: Diagnostics assuming orthogonal factors (Per-Factor Omega)
#' # --------------------------------------------------------
#'
#' result_orth <- makeCorrLoadings(
#'   loadings = factorLoadings,
#'   diagnostics = TRUE
#' )
#'
#' round(result_orth$Omega, 3) # per-factor Omega
#' round(result_orth$OmegaTotal, 3) # total Omega
#' print(result_orth$Diagnostics)
#'
#' @importFrom stats cov2cor
#'
#' @export
makeCorrLoadings <- function(loadings,
                             factorCor = NULL,
                             uniquenesses = NULL,
                             nearPD = FALSE,
                             diagnostics = FALSE) {
  # Ensure loadings is a matrix
  if (!is.matrix(loadings)) {
    warning("loadings should be a matrix. Converting.")
    loadings <- as.matrix(loadings)
  }

  k <- nrow(loadings)
  f <- ncol(loadings)

  # Remove any non-finite loadings
  if (any(!is.finite(loadings))) {
    warning("Non-finite loadings detected. Setting them to 0.")
    loadings[!is.finite(loadings)] <- 0
  }

  # Default: assume orthogonal factors phi = I
  if (is.null(factorCor)) {
    factorCor <- diag(f)
  }

  # Validate factorCor
  if (!all(dim(factorCor) == c(f, f))) {
    stop("factorCor must be a square matrix with dimensions matching ncol(loadings).")
  }
  if (!isSymmetric(factorCor) || any(diag(factorCor) != 1)) {
    warning("factorCor should be a symmetric correlation matrix with 1s on the diagonal.")
  }

  # Apply nearPD if factorCor is not PD
  if (any(eigen(factorCor, only.values = TRUE)$values <= 0)) {
    msg <- "factorCor is not positive definite."
    if (nearPD) {
      warning(paste(msg, "Attempting to fix with nearPD()."))
      factorCor <- as.matrix(Matrix::nearPD(factorCor)$mat)
    } else {
      warning(msg)
    }
  }

  # Communalities = rowSums of squared loadings
  communalities <- rowSums(loadings^2)

  # Uniquenesses = 1 - communalities
  if (is.null(uniquenesses)) {
    uniquenesses <- 1 - communalities
  } else {
    if (length(uniquenesses) != k) {
      stop("uniquenesses must be a vector of length equal to nrow(loadings).")
    }
  }

  # Warn for unusual values
  tol <- 1e-8
  if (any(communalities > 1 + tol)) {
    warning("Some communalities exceed 1. Check your loadings.")
  }
  if (any(uniquenesses < 0 - tol)) {
    warning("Some uniquenesses are negative. Check your loadings.")
  }

  # Construct full covariance matrix
  Psi <- diag(uniquenesses, k)
  Sigma <- loadings %*% factorCor %*% t(loadings) + Psi
  R <- cov2cor(Sigma)

  # Symmetrize correlation matrix
  R[upper.tri(R)] <- t(R)[upper.tri(R)]

  # Coerce to PD if needed
  if (any(eigen(R, only.values = TRUE)$values <= 0)) {
    msg <- "Resulting correlation matrix is not positive definite."
    if (nearPD) {
      warning(paste(msg, "Applying nearPD()."))
      R <- as.matrix(Matrix::nearPD(R)$mat)
    } else {
      warning(paste(msg, "Check your inputs."))
    }
  }

  # Return early if no diagnostics requested
  if (!diagnostics) {
    return(R)
  }

  #### DIAGNOSTICS SECTION ####

  # Identify primary factor (based on max abs loading)
  primary <- apply(abs(loadings), 1, which.max)
  primary_factor <- colnames(loadings)[primary]

  diag_table <- data.frame(
    Item = rownames(loadings),
    Communality = communalities,
    Uniqueness = uniquenesses,
    PrimaryFactor = primary_factor,
    stringsAsFactors = FALSE
  )

  # Per-factor Omega (assuming orthogonal)
  omega_vec <- numeric(f)
  for (j in 1:f) {
    lambda_sq <- loadings[, j]^2
    omega_vec[j] <- sum(lambda_sq) / (sum(lambda_sq) + sum(uniquenesses))
  }
  names(omega_vec) <- colnames(loadings)

  # Omega Total: variance explained across all factors
  total_var_expl <- sum(diag(loadings %*% factorCor %*% t(loadings)))
  total_var <- total_var_expl + sum(uniquenesses)
  omega_total <- total_var_expl / total_var

  # Adjusted Omega (accounts for factorCor)
  use_adjusted <- !is.null(factorCor) && !isTRUE(all.equal(factorCor, diag(f)))

  if (use_adjusted) {
    # Omega adjusted: reliability per factor incorporating \eqn{\Phi}
    omega_adj <- numeric(f)
    for (j in 1:f) {
      # Extract all loadings on factor j and their cross-factor effects
      lambda_j <- loadings[, j]
      phi_j <- factorCor[j, j] # = 1
      lambda_cross <- loadings %*% factorCor[, j]
      var_j <- sum(lambda_cross^2)
      omega_adj[j] <- var_j / (var_j + sum(uniquenesses))
    }
    names(omega_adj) <- colnames(loadings)

    message("Diagnostics returned with Adjusted Omega (accounting for factor correlations).")
    omega_out <- omega_adj
  } else {
    message("Diagnostics returned with Per-Factor Omega (assuming orthogonal factors).")
    omega_out <- omega_vec
  }

  return(list(
    R = R,
    Omega = omega_out,
    OmegaTotal = omega_total,
    Diagnostics = diag_table
  ))
}

# test-makeCorrLoadings.R

test_that("makeCorrLoadings returns a valid correlation matrix by default", {
  Lambda <- matrix(
    c(
      0.05, 0.20, 0.70,
      0.10, 0.05, 0.80,
      0.05, 0.15, 0.85,
      0.20, 0.85, 0.15,
      0.05, 0.85, 0.10,
      0.10, 0.90, 0.05,
      0.90, 0.15, 0.05,
      0.80, 0.10, 0.10
    ),
    nrow = 8, ncol = 3, byrow = TRUE
  )
  Phi <- matrix(
    c(
      1.0, 0.7, 0.6,
      0.7, 1.0, 0.4,
      0.6, 0.4, 1.0
    ),
    nrow = 3, byrow = TRUE
  )

  R <- makeCorrLoadings(Lambda, Phi)

  # Check dimensions: 8 items -> 8x8 matrix
  expect_true(is.matrix(R))
  expect_equal(nrow(R), 8)
  expect_equal(ncol(R), 8)

  # Diagonal elements should be close to 1
  expect_equal(diag(R), rep(1, 8), tolerance = 1e-6)

  # Correlation matrix should be symmetric
  expect_true(isSymmetric(R))
})

#' Sensitivity of Cronbach's alpha to scale design parameters
#'
#' Computes how Cronbach's alpha changes as a function of either
#' the average inter-item correlation (\eqn{\bar{r}}) or the number
#' of items (\eqn{k}), holding the other parameter constant.
#'
#' The function supports two modes:
#' \itemize{
#'   \item Empirical: derive \eqn{k} and \eqn{\bar{r}} from a dataset
#'   \item Theoretical: specify \eqn{k} and \eqn{\bar{r}} directly
#' }
#'
#' @param data A data frame or matrix of item responses. If provided,
#'   \eqn{k} and \eqn{\bar{r}} are computed from the data.
#'
#' @param k Number of items. Required if \code{data} is not supplied.
#'
#' @param r_bar Average inter-item correlation. Required if \code{data}
#'   is not supplied.
#'
#' @param vary Character string indicating which parameter to vary:
#'   \code{"r_bar"} (default) or \code{"k"}.
#'
#' @param k_range Numeric vector of item counts to evaluate when
#'   \code{vary = "k"}. Default is \code{2:20}.
#'
#' @param r_bar_range Numeric vector of average inter-item correlations
#'   to evaluate when \code{vary = "r_bar"}. Default is
#'   \code{seq(0.05, 0.9, by = 0.05)}.
#'
#' @param plot Logical; if \code{TRUE}, a base R plot is produced.
#'   Default is \code{TRUE}.
#'
#' @param digits Number of decimal places for rounding output.
#'   Default is \code{3}.
#'
#' @return A data frame with columns:
#' \itemize{
#'   \item \code{k}: number of items
#'   \item \code{r_bar}: average inter-item correlation
#'   \item \code{alpha}: Cronbach's alpha
#' }
#'
#' The returned object includes an attribute \code{"baseline"} containing
#' the reference \eqn{k} and \eqn{\bar{r}} values.
#'
#' @seealso \code{\link{alpha}}, \code{\link{reliability}}
#'
#' @examples
#' # Theoretical example
#'
#' \dontrun{
#' alpha_sensitivity(k = 6) # produces plot
#' }
#'
#' alpha_sensitivity(k = 6, r_bar = 0.4, plot = FALSE)
#'
#' # Vary number of items
#' alpha_sensitivity(k = 6, r_bar = 0.4, vary = "k", plot = FALSE)
#'
#' # Empirical example
#' df <- data.frame(
#'   V1 = c(1, 2, 3, 4, 5),
#'   V2 = c(3, 2, 4, 2, 5),
#'   V3 = c(2, 1, 5, 4, 3)
#' )
#'
#' \dontrun{
#' alpha_sensitivity(data = df) # produces plot
#' }
#'
#' alpha_sensitivity(df, vary = "r_bar", plot = FALSE)
#'
#' alpha_sensitivity(df, vary = "k", plot = FALSE)
#'
#' @export
alpha_sensitivity <- function(
  data = NULL,
  k = NULL,
  r_bar = NULL,
  vary = c("r_bar", "k"),
  k_range = NULL,
  r_bar_range = NULL,
  plot = TRUE,
  digits = 3
) {
  vary <- match.arg(vary)

  # ------------------ input validation ------------------
  if (!is.null(data)) {
    if (!is.null(k) || !is.null(r_bar)) {
      stop("Provide either 'data' OR ('k' and 'r_bar'), not both.",
        call. = FALSE
      )
    }

    X <- as.data.frame(data)

    if (ncol(X) < 2) {
      stop("data must contain at least 2 items (columns).",
        call. = FALSE
      )
    }

    R <- stats::cor(X, use = "pairwise.complete.obs")

    k <- ncol(R)
    r_bar <- mean(R[upper.tri(R)])
  } else {
    if (is.null(k)) {
      stop("Provide either 'data' or 'k'.",
        call. = FALSE
      )
    }

    # Default value for exploratory use
    if (is.null(r_bar)) {
      r_bar <- 0.3
    }

    if (!is.numeric(k) || length(k) != 1 || k < 2) {
      stop("'k' must be a single integer >= 2.",
        call. = FALSE
      )
    }

    if (!is.numeric(r_bar) || length(r_bar) != 1 || r_bar <= 0 || r_bar >= 1) {
      stop("'r_bar' must be a single value between 0 and 1.",
        call. = FALSE
      )
    }
  }

  # ------------------ defaults ------------------

  if (is.null(r_bar_range)) {
    r_bar_range <- seq(0.05, 0.9, by = 0.05)
  }

  if (is.null(k_range)) {
    k_range <- 2:20
  }

  # ------------------ computation ------------------

  if (vary == "r_bar") {
    alpha_vals <- (k * r_bar_range) / (1 + (k - 1) * r_bar_range)

    out <- data.frame(
      k = k,
      r_bar = r_bar_range,
      alpha = alpha_vals
    )
  } else {
    alpha_vals <- (k_range * r_bar) / (1 + (k_range - 1) * r_bar)

    out <- data.frame(
      k = k_range,
      r_bar = r_bar,
      alpha = alpha_vals
    )
  }

  # rounding
  out$alpha <- round(out$alpha, digits)
  out$r_bar <- round(out$r_bar, digits)

  # baseline attribute
  attr(out, "baseline") <- list(k = k, r_bar = r_bar)

  # ------------------ plotting ------------------

  if (isTRUE(plot)) {
    if (vary == "r_bar") {
      plot(
        out$r_bar, out$alpha,
        type = "l",
        xlab = "Average inter-item correlation (mean_r)",
        ylab = "Cronbach's alpha",
        main = paste0("Alpha sensitivity (k = ", k, ")")
      )

      abline(h = c(0.7, 0.8), lty = 2)
    } else {
      plot(
        out$k, out$alpha,
        type = "l",
        xlab = "Number of items (k)",
        ylab = "Cronbach's alpha",
        main = paste0("Alpha sensitivity (mean_r = ", round(r_bar, digits), ")")
      )

      abline(h = c(0.7, 0.8), lty = 2)
    }
  }

  return(out)
}

### Unit tests for alpha_sensitivity()

## Core correctness (formula check)
test_that("alpha_sensitivity computes correct alpha values", {
  res <- alpha_sensitivity(k = 4, r_bar = 0.5, plot = FALSE)

  expected <- (4 * 0.5) / (1 + (4 - 1) * 0.5)

  expect_equal(
    res$alpha[res$r_bar == 0.5],
    round(expected, 3)
  )
})

## Monotonic behaviour (conceptual integrity)
test_that("alpha increases monotonically with r_bar", {
  res <- alpha_sensitivity(k = 6, vary = "r_bar", plot = FALSE)

  expect_true(all(diff(res$alpha) > 0))
})

## Empirical mode works
test_that("empirical mode derives k and r_bar correctly", {
  df <- data.frame(
    x1 = c(1, 2, 3, 4),
    x2 = c(1, 2, 3, 4),
    x3 = c(1, 2, 3, 4)
  )

  res <- alpha_sensitivity(data = df, plot = FALSE)

  expect_equal(unique(res$k), 3)
  expect_equal(attr(res, "baseline")$r_bar, 1, tolerance = 1e-8)
})

## Input validation (combined)
test_that("input validation works correctly", {
  df <- data.frame(a = 1:3, b = 1:3)

  expect_error(alpha_sensitivity())
  expect_error(alpha_sensitivity(data = df, k = 3, r_bar = 0.3))
  expect_error(alpha_sensitivity(k = 1, r_bar = 0.3))
  expect_error(alpha_sensitivity(k = 4, r_bar = 1))
})

#' Estimate scale reliability for Likert and rating-scale data
#'
#' Computes internal consistency reliability estimates for a single-factor
#' scale, including Cronbach’s alpha, McDonald’s omega (total), and optional
#' ordinal (polychoric-based) variants. Confidence intervals may be obtained
#' via nonparametric bootstrap.
#'
#' The function is designed for Likert-type and rating-scale data and
#' prioritises transparent diagnostics when ordinal reliability estimates
#' are not feasible due to sparse response categories.
#'
#' @details
#' Cronbach’s alpha and McDonald’s omega are computed from Pearson correlations.
#' When \code{include = "polychoric"}, ordinal reliability estimates are computed
#' using polychoric correlations and correspond to Zumbo’s alpha and ordinal omega.
#'
#' Ordinal reliability estimates are skipped if response categories are sparse
#' or if polychoric estimation fails. Diagnostics explaining these decisions
#' are stored in the returned object and may be inspected using
#' \code{\link{ordinal_diagnostics}}.
#'
#' This function assumes a single common factor and is not intended for
#' multidimensional or structural equation modelling contexts.
#'
#'
#' @return
#' A tibble with one row per reliability coefficient and columns:
#' \itemize{
#'   \item \code{coef_name}: Name of the reliability coefficient.
#'   \item \code{estimate}: Point estimate.
#'   \item \code{ci_lower}, \code{ci_upper}: Confidence interval bounds
#'     (only present when \code{ci = TRUE}).
#'   \item \code{notes}: Methodological notes describing how the estimate
#'     was obtained.
#' }
#'
#' The returned object has class \code{"likert_reliability"} and includes
#' additional attributes containing diagnostics and bootstrap information.
#'
#' @seealso
#' \code{\link{ordinal_diagnostics}}
#'
#' @examples
#'
#' ## create dataset
#' my_cor <- LikertMakeR::makeCorrAlpha(
#'   items = 4,
#'   alpha = 0.80
#' )
#'
#' my_data <- LikertMakeR::makeScales(
#'   n = 64,
#'   means = c(2.75, 3.00, 3.25, 3.50),
#'   sds = c(1.25, 1.50, 1.30, 1.25),
#'   lowerbound = rep(1, 4),
#'   upperbound = rep(5, 4),
#'   cormatrix = my_cor
#' )
#'
#' ## run function
#' reliability(my_data)
#'
#' reliability(
#'   my_data,
#'   include = c("lambda6", "polychoric")
#' )
#'
#' \donttest{
#' ## slower (not run on CRAN checks)
#' reliability(
#'   my_data,
#'   include = "polychoric",
#'   ci = TRUE,
#'   n_boot = 200
#' )
#' }
#'
#' @param data A data frame or matrix containing item responses.
#'   Each column represents one item; rows represent respondents.
#'
#' @param include Character vector specifying which additional estimates
#'   to compute. Possible values are:
#'   \itemize{
#'     \item \code{"none"} (default): Pearson-based alpha and omega only.
#'     \item \code{"lambda6"}: Include Guttman’s lambda-6 (requires package \pkg{psych}).
#'     \item \code{"polychoric"}: Include ordinal (polychoric-based) alpha and omega.
#'   }
#'   Multiple options may be supplied.
#'
#' @param ci Logical; if \code{TRUE}, confidence intervals are computed using
#'   nonparametric bootstrap. Default is \code{FALSE}.
#'
#' @param ci_level Confidence level for bootstrap intervals.
#'   Default is \code{0.95}.
#'
#' @param n_boot Number of bootstrap resamples used when \code{ci = TRUE}.
#'   Default is \code{1000}.
#'
#' @param na_method Method for handling missing values. Either
#'   \code{"pairwise"} (default) or \code{"listwise"}.
#'
#' @param min_count Minimum observed frequency per response category required
#'   to attempt polychoric correlations. Ordinal reliability estimates are
#'   skipped if this condition is violated. Default is \code{2}.
#'
#' @param digits Number of decimal places used when printing estimates.
#'   Default is \code{3}.
#'
#' @param verbose Logical; if \code{TRUE}, warnings and progress indicators
#'   are displayed. Default is \code{TRUE}.
#'
#' @seealso \code{\link{alpha_sensitivity}}, \code{\link{alpha}}
#'
#' @export
reliability <- function(
  data,
  include = "none",
  ci = FALSE,
  ci_level = 0.95,
  n_boot = 1000,
  na_method = c("pairwise", "listwise"),
  min_count = 2,
  digits = 3,
  verbose = TRUE
) {
  na_method <- match.arg(na_method)

  include <- match.arg(
    include,
    choices = c("none", "lambda6", "omega_h", "polychoric"),
    several.ok = TRUE
  )

  show_pb <- FALSE
  pb <- NULL

  if (length(include) > 1 && "none" %in% include) include <- setdiff(include, "none")

  do_omega_h <- "omega_h" %in% include
  do_lambda6 <- "lambda6" %in% include
  do_polychor <- "polychoric" %in% include

  X <- as.data.frame(data)

  if (ncol(X) < 2) stop("data must contain at least 2 items (columns).")

  if (na_method == "listwise") X <- stats::na.omit(X)

  n <- nrow(X)
  k <- ncol(X)
  if (n < 3) stop("Not enough observations to compute reliability.")

  # ------------------ helpers ------------------


  get_R_pearson <- function(Xdf) stats::cor(Xdf, use = "pairwise.complete.obs")

  .ordinal_diagnostics_df <- function(Xdf) {
    Xo <- as.data.frame(lapply(Xdf, function(col) droplevels(ordered(col))))
    tabs <- lapply(Xo, function(z) table(z, useNA = "no"))

    data.frame(
      item = names(tabs),
      n_cat_observed = vapply(tabs, length, integer(1)),
      min_count = vapply(tabs, function(tt) min(as.integer(tt)), integer(1)),
      max_count = vapply(tabs, function(tt) max(as.integer(tt)), integer(1)),
      stringsAsFactors = FALSE
    )
  }

  .format_sparse_items <- function(Xdf, min_count_point = 2, max_items = 6) {
    Xo <- as.data.frame(lapply(Xdf, function(col) droplevels(ordered(col))))
    tabs <- lapply(Xo, function(z) table(z, useNA = "no"))

    sparse <- lapply(names(tabs), function(nm) {
      tt <- tabs[[nm]]
      bad_levels <- names(tt)[as.integer(tt) < min_count_point]
      if (length(bad_levels) == 0) {
        return(NULL)
      }

      counts <- as.integer(tt[bad_levels])
      ord <- order(counts)
      bad_levels <- bad_levels[ord]
      counts <- counts[ord]
      take <- seq_len(min(2, length(bad_levels)))

      paste0(nm, " (", paste0(bad_levels[take], "=", counts[take], collapse = ", "), ")")
    })

    sparse <- Filter(Negate(is.null), sparse)
    if (length(sparse) == 0) {
      return(NULL)
    }

    if (length(sparse) > max_items) sparse <- c(sparse[1:max_items], "...")
    paste(sparse, collapse = "; ")
  }


  get_R_polychoric <- function(Xdf, min_count_point = 2, verbose = TRUE) {
    Xo <- as.data.frame(lapply(Xdf, function(col) droplevels(ordered(col))))
    diag_ord <- .ordinal_diagnostics_df(Xo)

    if (any(diag_ord$n_cat_observed < 2)) {
      bad <- diag_ord$item[diag_ord$n_cat_observed < 2]
      if (verbose) {
        warning(
          "Ordinal reliability skipped: item(s) with <2 observed categories: ",
          paste(bad, collapse = ", "),
          call. = FALSE
        )
      }
      return(NULL)
    }

    if (any(diag_ord$min_count < min_count_point)) {
      details <- .format_sparse_items(Xo, min_count_point = min_count_point)
      if (verbose) {
        warning(
          "Ordinal reliability skipped: sparse categories detected (min_count < ",
          min_count_point, "). ",
          if (!is.null(details)) paste0("Examples: ", details, ". ") else "",
          "Consider increasing n, collapsing categories, or using Pearson-based reliability.",
          call. = FALSE
        )
      }
      return(NULL)
    }

    if (!requireNamespace("psych", quietly = TRUE) &&
      !requireNamespace("polycor", quietly = TRUE)) {
      if (verbose) {
        warning(
          "Ordinal reliability skipped: need optional package 'psych' (preferred) or 'polycor' for polychoric correlations.",
          call. = FALSE
        )
      }
      return(NULL)
    }

    if (requireNamespace("psych", quietly = TRUE)) {
      out <- tryCatch(
        suppressMessages(suppressWarnings(psych::polychoric(Xo, global = FALSE))),
        error = function(e) e
      )

      if (!inherits(out, "error")) {
        return(out$rho)
      }

      if (verbose) {
        warning(
          "Ordinal reliability skipped: psych::polychoric() failed (",
          out$message,
          "). This is usually caused by sparse/extreme response distributions.",
          call. = FALSE
        )
      }
      return(NULL)
    }

    if (requireNamespace("polycor", quietly = TRUE)) {
      out2 <- tryCatch(
        suppressMessages(suppressWarnings(polycor::hetcor(Xo, ML = TRUE))),
        error = function(e) e
      )
      if (!inherits(out2, "error")) {
        return(out2$correlations)
      }

      if (verbose) {
        warning(
          "Ordinal reliability skipped: polycor::hetcor() failed (",
          out2$message,
          ").",
          call. = FALSE
        )
      }
      return(NULL)
    }

    NULL
  }

  alpha_from_R <- function(Rmat) {
    k <- ncol(Rmat)
    (k / (k - 1)) * (1 - (sum(diag(Rmat)) / sum(Rmat)))
  }

  omega_from_R <- function(Rmat) {
    eig <- eigen(Rmat, symmetric = TRUE)
    loadings <- sqrt(max(eig$values[1], 0)) * eig$vectors[, 1]
    if (sum(loadings) < 0) loadings <- -loadings
    uni <- 1 - loadings^2
    sum(loadings)^2 / (sum(loadings)^2 + sum(uni))
  }


  ## omega_h function

  H_from_R <- function(Rmat, n_obs = NULL, verbose = TRUE) {
    # Coefficient H ("maximal reliability") for a single-factor model.
    # H = (sum(lambda))^2 / ((sum(lambda))^2 + sum(theta))
    # where lambda are standardized factor loadings and theta are uniquenesses.

    # Prefer psych::fa if available
    if (requireNamespace("psych", quietly = TRUE)) {
      fit <- tryCatch(
        psych::fa(Rmat, nfactors = 1, fm = "minres", rotate = "none"),
        error = function(e) e
      )
      if (inherits(fit, "error")) {
        if (isTRUE(verbose)) warning("omega_h (H) skipped: psych::fa() failed: ", fit$message, call. = FALSE)
        return(NA_real_)
      }

      lam <- as.numeric(fit$loadings[, 1])
      if (!is.finite(sum(lam))) {
        return(NA_real_)
      }

      # Align sign for stability/interpretability
      if (sum(lam, na.rm = TRUE) < 0) lam <- -lam

      theta <- as.numeric(fit$uniquenesses)
      if (length(theta) != length(lam)) {
        return(NA_real_)
      }

      num <- (sum(lam, na.rm = TRUE))^2
      den <- num + sum(theta, na.rm = TRUE)
      if (!is.finite(den) || den <= 0) {
        return(NA_real_)
      }
      return(num / den)
    }

    # Fallback: stats::factanal (needs n.obs if using covmat list)
    if (!is.null(n_obs) && is.finite(n_obs) && n_obs > 2) {
      fit2 <- tryCatch(
        stats::factanal(covmat = list(cov = Rmat, n.obs = n_obs), factors = 1, rotation = "none"),
        error = function(e) e
      )
      if (inherits(fit2, "error")) {
        if (isTRUE(verbose)) warning("omega_h (H) skipped: factanal() failed: ", fit2$message, call. = FALSE)
        return(NA_real_)
      }

      lam <- as.numeric(fit2$loadings[, 1])
      if (sum(lam, na.rm = TRUE) < 0) lam <- -lam
      theta <- as.numeric(fit2$uniquenesses)

      num <- (sum(lam, na.rm = TRUE))^2
      den <- num + sum(theta, na.rm = TRUE)
      if (!is.finite(den) || den <= 0) {
        return(NA_real_)
      }
      return(num / den)
    }

    if (isTRUE(verbose)) {
      warning("omega_h (H) skipped: requires suggested package 'psych' (preferred), or provide n_obs for factanal() fallback.",
        call. = FALSE
      )
    }
    NA_real_
  }


  # ------------------ Pearson estimates ------------------

  R <- get_R_pearson(X)
  alpha_est <- alpha_from_R(R)
  omega_est <- omega_from_R(R)

  lambda6_est <- NA_real_
  if (do_lambda6) {
    if (!requireNamespace("psych", quietly = TRUE)) {
      if (verbose) warning("include='lambda6' requires the suggested package 'psych'.", call. = FALSE)
    } else {
      suppressWarnings({
        lambda6_est <- psych::alpha(X)$total[["G6(smc)"]]
      })
    }
  }


  # ------------------ Ordinal estimates + diagnostics ------------------

  diag_ord <- NULL
  ordinal_alpha_est <- NA_real_
  ordinal_omega_est <- NA_real_
  omega_h_est <- NA_real_

  ordinal_ci_ok <- FALSE

  if (do_polychor) {
    diag_ord <- .ordinal_diagnostics_df(X)
    ordinal_ci_ok <- all(diag_ord$min_count >= min_count)

    R_poly <- get_R_polychoric(X, min_count_point = min_count, verbose = verbose)
    if (!is.null(R_poly)) {
      ordinal_alpha_est <- alpha_from_R(R_poly)
      ordinal_omega_est <- omega_from_R(R_poly)
    }
  }


  if (do_omega_h) {
    omega_h_est <- H_from_R(R, n_obs = n, verbose = verbose)
  }

  # ------------------ Bootstrap CI init ------------------

  ci_alpha <- ci_omega <- ci_lambda6 <- c(NA_real_, NA_real_)
  ci_ord_alpha <- ci_ord_omega <- c(NA_real_, NA_real_)

  ord_fail <- 0L
  ord_attempts <- 0L
  ord_success <- NA_integer_

  # ------------------ CI bootstrap ------

  if (isTRUE(ci)) {
    probs <- c((1 - ci_level) / 2, 1 - (1 - ci_level) / 2)

    # Preallocate bootstrap matrix
    boot_mat <- matrix(NA_real_, nrow = 5, ncol = n_boot)
    rownames(boot_mat) <- c(
      "alpha", "omega", "lambda6",
      "ordinal_alpha", "ordinal_omega"
    )

    show_pb <- isTRUE(verbose) && interactive()

    if (show_pb) {
      pb <- utils::txtProgressBar(min = 0, max = n_boot, style = 3)
      on.exit(try(close(pb), silent = TRUE), add = TRUE)
    }

    for (b in seq_len(n_boot)) {
      idx <- sample.int(n, replace = TRUE)
      Xb <- X[idx, , drop = FALSE]

      # Pearson
      Rb <- get_R_pearson(Xb)
      a <- alpha_from_R(Rb)
      o <- omega_from_R(Rb)

      # lambda6
      l6 <- NA_real_
      if (!is.na(lambda6_est) && do_lambda6 && requireNamespace("psych", quietly = TRUE)) {
        l6 <- psych::alpha(Xb)$total[["G6(smc)"]]
      }

      # Ordinal
      oa <- oo <- NA_real_

      if (do_polychor && isTRUE(ci) && isTRUE(ordinal_ci_ok)) {
        ord_attempts <- ord_attempts + 1L
        poly_try <- tryCatch(
          {
            Rpb <- get_R_polychoric(
              Xb,
              min_count_point = min_count,
              verbose = FALSE
            )
            if (is.null(Rpb)) stop("polychoric unavailable")
            c(
              oa = alpha_from_R(Rpb),
              oo = omega_from_R(Rpb)
            )
          },
          error = function(e) {
            ord_fail <- ord_fail + 1L
            c(oa = NA_real_, oo = NA_real_)
          }
        )

        oa <- unname(as.numeric(poly_try["oa"]))
        oo <- unname(as.numeric(poly_try["oo"]))
      }

      boot_mat[, b] <- c(a, o, l6, oa, oo)

      if (show_pb && (b %% 5 == 0 || b == n_boot)) {
        utils::setTxtProgressBar(pb, b)
      }
    }


    # Success counts
    ord_success_alpha <- sum(!is.na(boot_mat["ordinal_alpha", ]))
    ord_success_omega <- sum(!is.na(boot_mat["ordinal_omega", ]))
    ord_success <- min(ord_success_alpha, ord_success_omega)
    ord_ci_ok <- (do_polychor && isTRUE(ordinal_ci_ok) && isTRUE(ci) && is.finite(ord_success) && ord_success > 0L)


    # Pearson CIs
    ci_alpha <- stats::quantile(boot_mat["alpha", ], probs = probs, na.rm = TRUE)
    ci_omega <- stats::quantile(boot_mat["omega", ], probs = probs, na.rm = TRUE)

    if (!all(is.na(boot_mat["lambda6", ]))) {
      ci_lambda6 <- stats::quantile(
        boot_mat["lambda6", ],
        probs = probs,
        na.rm = TRUE
      )
    }

    # Ordinal CIs (only if feasible)
    if (ord_ci_ok) {
      ci_ord_alpha <- stats::quantile(
        boot_mat["ordinal_alpha", ],
        probs = probs,
        na.rm = TRUE
      )
      ci_ord_omega <- stats::quantile(
        boot_mat["ordinal_omega", ],
        probs = probs,
        na.rm = TRUE
      )
    }
  }


  # ------------------ Output table + notes ------------------

  out <- tibble::tibble(
    coef_name = c("alpha", "omega_total"),
    estimate = c(alpha_est, omega_est),
    ci_lower = c(ci_alpha[1], ci_omega[1]),
    ci_upper = c(ci_alpha[2], ci_omega[2]),
    n_items = k,
    n_obs = n,
    notes = c(
      "Pearson correlations",
      "1-factor eigen omega"
    )
  )

  if (do_lambda6 && !is.na(lambda6_est)) {
    out <- dplyr::bind_rows(
      out,
      tibble::tibble(
        coef_name = "lambda6",
        estimate  = lambda6_est,
        ci_lower  = ci_lambda6[1],
        ci_upper  = ci_lambda6[2],
        n_items   = k,
        n_obs     = n,
        notes     = "psych::alpha()"
      )
    )
  }


  if (do_omega_h && !is.na(omega_h_est)) {
    out <- dplyr::bind_rows(
      out,
      tibble::tibble(
        coef_name = "omega_h",
        estimate  = omega_h_est,
        ci_lower  = NA_real_,
        ci_upper  = NA_real_,
        n_items   = k,
        n_obs     = n,
        notes     = "Coefficient H (1-factor FA, maximal reliability)"
      )
    )
  }


  if (do_polychor) {
    note_ord <- if (is.na(ordinal_alpha_est)) {
      paste0("Ordinal skipped (see diagnostics)")
    } else {
      "Polychoric correlations"
    }

    note_ci <- if (is.na(ordinal_alpha_est)) {
      if (isTRUE(ci)) "Ordinal CIs not computed (ordinal estimate unavailable)" else "Ordinal CIs not requested"
    } else if (isTRUE(ci)) {
      if (!isTRUE(ordinal_ci_ok)) {
        paste0("Ordinal CIs skipped: sparse categories (min_count < ", min_count, ")")
      } else if (ord_attempts > 0L && ord_fail > 0L) {
        paste0("CI bootstrap failures: ", ord_fail, "/", ord_attempts, " draws")
      } else {
        "Ordinal CIs via bootstrap"
      }
    } else {
      "Ordinal CIs not requested"
    }

    out <- dplyr::bind_rows(
      out,
      tibble::tibble(
        coef_name = c("ordinal_alpha", "ordinal_omega_total"),
        estimate  = c(ordinal_alpha_est, ordinal_omega_est),
        ci_lower  = c(ci_ord_alpha[1], ci_ord_omega[1]),
        ci_upper  = c(ci_ord_alpha[2], ci_ord_omega[2]),
        n_items   = k,
        n_obs     = n,
        notes     = c(note_ord, paste(note_ord, "|", note_ci))
      )
    )
  }

  out$estimate <- round(out$estimate, digits)
  if ("ci_lower" %in% names(out)) out$ci_lower <- round(out$ci_lower, digits)
  if ("ci_upper" %in% names(out)) out$ci_upper <- round(out$ci_upper, digits)

  if (!isTRUE(ci)) {
    out <- dplyr::select(out, -dplyr::any_of(c("ci_lower", "ci_upper")))
  }

  class(out) <- c("likert_reliability", class(out))
  attr(out, "include") <- include
  attr(out, "ci") <- ci
  attr(out, "ci_level") <- ci_level
  attr(out, "n_boot") <- n_boot
  attr(out, "min_count") <- min_count

  if (do_polychor) {
    attr(out, "ordinal_diagnostics") <- diag_ord
    attr(out, "ordinal_ci_ok") <- ordinal_ci_ok
    attr(out, "ordinal_boot_failures") <- ord_fail
    attr(out, "ordinal_boot_attempts") <- ord_attempts
  }

  if (verbose && do_polychor && isTRUE(ci) && !isTRUE(ordinal_ci_ok)) {
    warning(
      "Ordinal CIs were skipped due to sparse categories (min_count < ", min_count, "). ",
      "Use ordinal_diagnostics() on the returned object to inspect the cause.",
      call. = FALSE
    )
  }

  out
}
#'
#' Print method for reliability objects
#'
#' @keywords internal
#' @param x An object returned by [reliability()].
#' @param ... Unused.
#' @export
print.likert_reliability <- function(x, ...) {
  print(as.data.frame(x), row.names = FALSE)

  inc <- attr(x, "include")
  if (!is.null(inc) && "polychoric" %in% inc) {
    diag_ord <- attr(x, "ordinal_diagnostics")
    if (!is.null(diag_ord)) {
      minc <- attr(x, "min_count")
      if (is.null(minc) || !is.numeric(minc) || length(minc) != 1) minc <- 2
      sparse <- diag_ord[diag_ord$min_count < minc, , drop = FALSE]

      if (nrow(sparse) > 0) {
        cat("\nOrdinal diagnostics (sparse categories detected):\n")
        print(sparse, row.names = FALSE)
        cat("\n")
      }
    }
  }

  invisible(x)
}
#'
#' Extract ordinal diagnostics from a reliability() result
#'
#' @param x An object returned by [reliability()].
#' @return A data.frame describing observed response categories and sparsity checks,
#'   or NULL if no diagnostics are available.
#' @export
ordinal_diagnostics <- function(x) {
  if (!inherits(x, "likert_reliability")) {
    stop("ordinal_diagnostics() expects an object returned by reliability().", call. = FALSE)
  }

  inc <- attr(x, "include")
  diag <- attr(x, "ordinal_diagnostics")

  # If user didn't request polychoric, just return NULL quietly
  if (is.null(inc) || !"polychoric" %in% inc) {
    return(NULL)
  }

  if (is.null(diag)) {
    warning("No ordinal diagnostics available in this object.", call. = FALSE)
    return(NULL)
  }

  diag
}
#'
#'

#' Generate regression data from summary statistics
#'
#' @name
#' makeScalesRegression
#'
#' @title
#' Generate Data from Multiple-Regression Summary Statistics
#'
#' @description
#' Generates synthetic rating-scale data that replicates reported regression
#' results. This function is useful for reproducing analyses from published
#' research where only summary statistics (standardised regression
#' coefficients and R-squared) are reported.
#'
#' @importFrom stats as.formula coef lm model.frame sd
#' @importFrom Matrix nearPD
#'
#' @details
#' The function can operate in two modes:
#'
#' **Mode 1: With IV correlation matrix provided**
#'
#' When \code{iv_cormatrix} is provided, the function uses the given
#' correlation structure among independent variables and calculates the
#' implied IV-DV correlations from the regression coefficients.
#'
#' **Mode 2: With optimisation (IV correlation matrix not provided)**
#'
#' When \code{iv_cormatrix = NULL}, the function optimises to find a plausible
#' correlation structure among independent variables that matches the reported
#' regression statistics.
#' Initial correlations are sampled using Fisher's z-transformation to ensure
#' proper distribution, then iteratively adjusted to match the target
#' R-squared.
#'
#' The function generates Likert-scale data (not individual items)
#' using \code{lfast()} for each variable with specified moments, then
#' correlates them using \code{lcor()}.
#' Generated data are verified by running a regression and comparing achieved
#' statistics with targets.
#'
#' @param n Integer. Sample size
#' @param beta_std Numeric vector of standardised regression coefficients
#'   (length k)
#' @param r_squared Numeric. R-squared from regression (-1 to 1)
#' @param iv_cormatrix k x k correlation matrix of independent variables.
#'   If missing (NULL), will be optimised.
#' @param iv_cor_mean Numeric. Mean correlation among IVs when optimising
#'  (ignored if iv_cormatrix provided). Default = 0.3
#' @param iv_cor_variance Numeric. Variance of correlations when optimising
#'  (ignored if iv_cormatrix provided). Default = 0.01
#' @param iv_cor_range Numeric vector of length 2.
#'  Min and max constraints on correlations when optimising.
#'  Default = c(-0.7, 0.7)
#' @param iv_means Numeric vector of means for IVs (length k)
#' @param iv_sds Numeric vector of standard deviations for IVs (length k)
#' @param dv_mean Numeric. Mean of dependent variable
#' @param dv_sd Numeric. Standard deviation of dependent variable
#' @param lowerbound_iv Numeric vector of lower bounds for each IV scale
#'   (or single value for all)
#' @param upperbound_iv Numeric vector of upper bounds for each IV scale
#'   (or single value for all)
#' @param lowerbound_dv Numeric. Lower bound for DV scale
#' @param upperbound_dv Numeric. Upper bound for DV scale
#' @param items_iv Integer vector of number of items per IV scale
#'   (or single value for all). Default = 1
#' @param items_dv Integer. Number of items in DV scale. Default = 1
#' @param var_names Character vector of variable names
#'   (length k+1: IVs then DV)
#' @param tolerance Numeric. Acceptable deviation from target R-squared
#'   (default 0.005)
#'
#' @return A list containing:
#'   \item{data}{Generated dataframe with k IVs and 1 DV}
#'   \item{target_stats}{List of target statistics provided}
#'   \item{achieved_stats}{List of achieved statistics from generated data}
#'   \item{diagnostics}{Comparison of target vs achieved}
#'   \item{iv_dv_cors}{Calculated correlations between IVs and DV}
#'   \item{full_cormatrix}{The complete (k+1) x (k+1) correlation matrix used}
#'   \item{optimisation_info}{If IV correlations were optimised,
#'         details about the optimisation}
#'
#'
#' @examples
#'
#' # Example 1: With provided IV correlation matrix
#' set.seed(123)
#' iv_corr <- matrix(c(1.0, 0.3, 0.3, 1.0), nrow = 2)
#'
#' result1 <- makeScalesRegression(
#'   n = 64,
#'   beta_std = c(0.4, 0.3),
#'   r_squared = 0.35,
#'   iv_cormatrix = iv_corr,
#'   iv_means = c(3.0, 3.5),
#'   iv_sds = c(1.0, 0.9),
#'   dv_mean = 3.8,
#'   dv_sd = 1.1,
#'   lowerbound_iv = 1,
#'   upperbound_iv = 5,
#'   lowerbound_dv = 1,
#'   upperbound_dv = 5,
#'   items_iv = 4,
#'   items_dv = 4,
#'   var_names = c("Attitude", "Intention", "Behaviour")
#' )
#'
#' print(result1)
#' head(result1$data)
#'
#'
#' # Example 2: With optimisation (no IV correlation matrix)
#' set.seed(456)
#' result2 <- makeScalesRegression(
#'   n = 128,
#'   beta_std = c(0.3, 0.25, 0.2),
#'   r_squared = 0.40,
#'   iv_cormatrix = NULL, # Will be optimised
#'   iv_cor_mean = 0.3,
#'   iv_cor_variance = 0.02,
#'   iv_means = c(3.0, 3.2, 2.8),
#'   iv_sds = c(1.0, 0.9, 1.1),
#'   dv_mean = 3.5,
#'   dv_sd = 1.0,
#'   lowerbound_iv = 1,
#'   upperbound_iv = 5,
#'   lowerbound_dv = 1,
#'   upperbound_dv = 5,
#'   items_iv = 4,
#'   items_dv = 5
#' )
#'
#' # View optimised correlation matrix
#' print(result2$target_stats$iv_cormatrix)
#' print(result2$optimisation_info)
#'
#' @seealso
#' \code{\link{lfast}} for generating individual rating-scale vectors with exact moments.
#'
#' \code{\link{lcor}} for rearranging values to achieve target correlations.
#'
#' \code{\link{makeCorrAlpha}} for generating correlation matrices from Cronbach's Alpha.
#'
#'
#' @export
makeScalesRegression <- function(
  n,
  beta_std,
  r_squared,
  iv_cormatrix = NULL,
  iv_cor_mean = 0.3,
  iv_cor_variance = 0.01,
  iv_cor_range = c(-0.7, 0.7),
  iv_means,
  iv_sds,
  dv_mean,
  dv_sd,
  lowerbound_iv,
  upperbound_iv,
  lowerbound_dv,
  upperbound_dv,
  items_iv = 1,
  items_dv = 1,
  var_names = NULL,
  tolerance = 0.005
) {
  # Input validation
  k <- length(beta_std) # number of IVs

  # Check if IV correlation matrix needs to be optimised
  optimisation_used <- is.null(iv_cormatrix)
  optimisation_info <- NULL

  if (optimisation_used) {
    message("IV correlation matrix not provided.
            \nOptimising to find plausible structure...")

    opt_result <- optimise_iv_cormatrix(
      k = k,
      beta_std = beta_std,
      r_squared = r_squared,
      iv_cor_mean = iv_cor_mean,
      iv_cor_variance = iv_cor_variance,
      iv_cor_range = iv_cor_range,
      tolerance = tolerance
    )

    iv_cormatrix <- opt_result$cormatrix
    optimisation_info <- list(
      converged = opt_result$converged,
      iterations = opt_result$iterations,
      achieved_r_squared_in_optimisation = opt_result$achieved_r_squared,
      iv_cor_mean_used = iv_cor_mean,
      iv_cor_variance_used = iv_cor_variance,
      iv_cor_range_used = iv_cor_range
    )

    message(sprintf(
      "Optimisation %s after %d iterations
      \n(R-sq target: %.4f, achieved in optimisation: %.4f)",
      ifelse(opt_result$converged, "converged", "completed"),
      opt_result$iterations,
      r_squared,
      opt_result$achieved_r_squared
    ))
  }

  if (nrow(iv_cormatrix) != k || ncol(iv_cormatrix) != k) {
    stop("iv_cormatrix dimensions must match number of IVs
         \n(length of beta_std)")
  }

  if (length(iv_means) != k || length(iv_sds) != k) {
    stop("iv_means and iv_sds must have same length as beta_std")
  }

  if (r_squared < 0 || r_squared > 1) {
    stop("r_squared must be between 0 and 1")
  }

  # Recycle single values to vectors if needed
  if (length(lowerbound_iv) == 1) lowerbound_iv <- rep(lowerbound_iv, k)
  if (length(upperbound_iv) == 1) upperbound_iv <- rep(upperbound_iv, k)
  if (length(items_iv) == 1) items_iv <- rep(items_iv, k)

  # Generate variable names if not provided
  if (is.null(var_names)) {
    var_names <- c(paste0("IV", 1:k), "DV")
  } else if (length(var_names) != k + 1) {
    stop("var_names must have length k+1 (IVs plus DV)")
  }

  # STEP 1: Calculate IV-DV correlations
  # r_XY = R_XX %*% beta_std
  iv_dv_cors <- as.vector(iv_cormatrix %*% beta_std)
  names(iv_dv_cors) <- var_names[1:k]

  # STEP 2: Verify consistency with R-squared
  predicted_r_squared <- sum(beta_std * iv_dv_cors)

  r_squared_diff <- abs(predicted_r_squared - r_squared)

  if (r_squared_diff > tolerance) {
    warning(
      sprintf(
        "Predicted R-squared (%.4f) differs from target (%.4f) by %.4f,
        which exceeds tolerance (%.4f).
        \nInput statistics may be inconsistent.",
        predicted_r_squared, r_squared, r_squared_diff, tolerance
      )
    )
  }

  # STEP 3: Construct full correlation matrix
  full_cormatrix <- matrix(1, nrow = k + 1, ncol = k + 1)
  full_cormatrix[1:k, 1:k] <- iv_cormatrix # IV correlations
  full_cormatrix[1:k, k + 1] <- iv_dv_cors # IV-DV correlations
  full_cormatrix[k + 1, 1:k] <- iv_dv_cors # DV-IV correlations (symmetric)

  rownames(full_cormatrix) <- colnames(full_cormatrix) <- var_names

  # Check if correlation matrix is positive definite
  eigenvals <- eigen(full_cormatrix, only.values = TRUE)$values
  if (any(eigenvals < -1e-8)) {
    warning("Resulting correlation matrix is not positive definite.
            \nGeneration may fail or produce unexpected results.")
  }

  # STEP 4: Generate data using lfast() per variable, then lcor() to correlate
  # Combine all parameters
  all_means <- c(iv_means, dv_mean)
  all_sds <- c(iv_sds, dv_sd)
  all_lowerbound <- c(lowerbound_iv, lowerbound_dv)
  all_upperbound <- c(upperbound_iv, upperbound_dv)
  all_items <- c(items_iv, items_dv)

  # Generate each variable independently with lfast()
  generated_data <- matrix(NA, nrow = n, ncol = k + 1)
  for (i in 1:(k + 1)) {
    generated_data[, i] <- lfast(
      n = n,
      mean = all_means[i],
      sd = all_sds[i],
      lowerbound = all_lowerbound[i],
      upperbound = all_upperbound[i],
      items = all_items[i]
    )
  }

  # Convert to data frame and add column names BEFORE lcor()
  generated_data <- as.data.frame(generated_data)
  colnames(generated_data) <- var_names

  # Correlate the variables using lcor()
  generated_data <- lcor(generated_data, full_cormatrix)

  # Ensure it's still a data frame after lcor() (lcor might return matrix)
  generated_data <- as.data.frame(generated_data)
  colnames(generated_data) <- var_names # Reapply names if lost

  # STEP 5: Verify - run regression on generated data
  formula_str <- paste(var_names[k + 1], "~", paste(var_names[1:k], collapse = " + "))
  reg_model <- lm(as.formula(formula_str), data = as.data.frame(generated_data))

  achieved_beta_std <- std_beta(reg_model)
  achieved_r_squared <- summary(reg_model)$r.squared
  achieved_cors <- cor(generated_data)

  # Calculate achieved IV-DV correlations
  achieved_iv_dv_cors <- achieved_cors[1:k, k + 1]
  names(achieved_iv_dv_cors) <- var_names[1:k]

  # STEP 6: Compile diagnostics
  target_stats <- list(
    beta_std = beta_std,
    r_squared = r_squared,
    iv_dv_cors = iv_dv_cors,
    iv_means = iv_means,
    iv_sds = iv_sds,
    dv_mean = dv_mean,
    dv_sd = dv_sd,
    iv_cormatrix = iv_cormatrix
  )

  achieved_stats <- list(
    beta_std = achieved_beta_std,
    r_squared = achieved_r_squared,
    iv_dv_cors = achieved_iv_dv_cors,
    iv_means = colMeans(generated_data[, 1:k]),
    iv_sds = apply(generated_data[, 1:k], 2, sd),
    dv_mean = mean(generated_data[, k + 1]),
    dv_sd = sd(generated_data[, k + 1]),
    full_cormatrix = achieved_cors
  )

  # Create diagnostics comparison table
  diagnostics <- data.frame(
    Statistic = c(
      paste0("Beta_", var_names[1:k]),
      "R_squared",
      paste0("r_", var_names[1:k], "_DV"),
      paste0("Mean_", var_names),
      paste0("SD_", var_names)
    ),
    Target = c(
      beta_std,
      r_squared,
      iv_dv_cors,
      all_means,
      all_sds
    ),
    Achieved = c(
      achieved_beta_std,
      achieved_r_squared,
      achieved_iv_dv_cors,
      c(achieved_stats$iv_means, achieved_stats$dv_mean),
      c(achieved_stats$iv_sds, achieved_stats$dv_sd)
    )
  )

  diagnostics$Difference <- diagnostics$Achieved - diagnostics$Target
  diagnostics$Pct_Error <- (diagnostics$Difference / diagnostics$Target) * 100

  # Return results
  result <- list(
    data = as.data.frame(generated_data),
    target_stats = target_stats,
    achieved_stats = achieved_stats,
    diagnostics = diagnostics,
    iv_dv_cors = iv_dv_cors,
    full_cormatrix = full_cormatrix,
    optimisation_info = optimisation_info,
    call = match.call()
  )

  class(result) <- c("makeScalesRegression", "list")

  return(result)
}


#' Print method for makeScalesRegression objects
#'
#' @param x An object of class "makeScalesRegression"
#' @param ... Additional arguments (currently unused)
#'
#' @keywords internal
#' @export
print.makeScalesRegression <- function(x, ...) {
  cat("Regression Data Generation Results\n")
  cat("===================================\n\n")
  cat("Sample size:", nrow(x$data), "\n")
  cat("Number of IVs:", ncol(x$data) - 1, "\n")

  # Show optimisation info if used
  if (!is.null(x$optimisation_info)) {
    cat("\nIV Correlation Matrix: OPTIMISED\n")
    cat(sprintf("  Converged: %s\n", x$optimisation_info$converged))
    cat(sprintf("  Iterations: %d\n", x$optimisation_info$iterations))
    cat(sprintf("  Target mean correlation: %.3f\n", x$optimisation_info$iv_cor_mean_used))
    cat(sprintf("  Correlation variance: %.4f\n", x$optimisation_info$iv_cor_variance_used))
  } else {
    cat("\nIV Correlation Matrix: PROVIDED\n")
  }

  cat("\nKey Statistics:\n")
  cat("---------------\n")
  cat(sprintf("Target R-squared:   %.4f\n", x$target_stats$r_squared))
  cat(sprintf("Achieved R-squared: %.4f\n", x$achieved_stats$r_squared))
  cat(sprintf(
    "Difference:         %.4f\n\n",
    x$achieved_stats$r_squared - x$target_stats$r_squared
  ))

  cat("Regression Coefficients (Standardised):\n")
  # Get IV names from the data column names (exclude DV which is last)
  iv_names <- colnames(x$data)[1:(ncol(x$data) - 1)]
  beta_comparison <- data.frame(
    Variable = iv_names,
    Target = round(x$target_stats$beta_std, 4),
    Achieved = round(x$achieved_stats$beta_std, 4),
    Diff = round(x$achieved_stats$beta_std - x$target_stats$beta_std, 4)
  )
  print(beta_comparison, row.names = FALSE)

  cat("\nFor full diagnostics, see $diagnostics\n")
  cat("For generated data, see $data\n")
  if (!is.null(x$optimisation_info)) {
    cat("For IV correlation matrix used, see $target_stats$iv_cormatrix\n")
  }

  invisible(x)
}

## Helper functions

# Helper function to calculate standardised betas
std_beta <- function(model) {
  b <- coef(model)[-1] # unstandardised coefficients (exclude intercept)
  sx <- apply(model.frame(model)[, -1], 2, sd) # SD of predictors
  sy <- sd(model.frame(model)[, 1]) # SD of outcome
  return(b * sx / sy) # standardise: st-beta = beta * (SD_x / SD_y)
}

# Helper function to generate IV correlation matrix via optimisation
optimise_iv_cormatrix <- function(k, beta_std, r_squared,
                                  iv_cor_mean, iv_cor_variance,
                                  iv_cor_range, tolerance) {
  # Fisher's z-transformation functions
  # Transforms correlation [-1, +1] to unbounded z-score
  log_transform <- function(x) {
    log((1 + x) / (1 - x))
  }

  # Inverse Fisher's z-transformation
  # Transforms z-score back to correlation [-1, +1]
  exp_transform <- function(y) {
    (exp(y) - 1) / (exp(y) + 1)
  }

  # Generate initial correlation matrix using Fisher's z-transformation
  # Sample in transformed space, then convert back to correlation space
  z_mean <- log_transform(iv_cor_mean)
  init_z <- rnorm(k * (k - 1) / 2, mean = z_mean, sd = sqrt(iv_cor_variance))
  init_cors <- exp_transform(init_z)

  # Constrain to range as safety check
  init_cors <- pmax(iv_cor_range[1], pmin(iv_cor_range[2], init_cors))

  # Build symmetric correlation matrix
  cor_matrix <- diag(k)
  cor_matrix[lower.tri(cor_matrix)] <- init_cors
  cor_matrix[upper.tri(cor_matrix)] <- t(cor_matrix)[upper.tri(cor_matrix)]

  # Ensure positive definite using nearPD if needed
  if (any(eigen(cor_matrix, only.values = TRUE)$values <= 0)) {
    cor_matrix <- as.matrix(Matrix::nearPD(cor_matrix, corr = TRUE)$mat)
  }

  # Calculate implied R-sq from this correlation matrix
  calc_r_squared <- function(cor_mat) {
    iv_dv_cors <- as.vector(cor_mat %*% beta_std)
    return(sum(beta_std * iv_dv_cors))
  }

  current_r_squared <- calc_r_squared(cor_matrix)

  # If already close enough, return
  if (abs(current_r_squared - r_squared) <= tolerance) {
    return(list(
      cormatrix = cor_matrix,
      achieved_r_squared = current_r_squared,
      iterations = 0,
      converged = TRUE
    ))
  }

  # Optimisation: adjust correlations to match target R-sq
  # Strategy: scale all correlations by a factor to adjust R-sq

  max_iterations <- 100
  best_matrix <- cor_matrix
  best_diff <- abs(current_r_squared - r_squared)

  for (iter in 1:max_iterations) {
    # Calculate scaling factor needed
    # If current R-sq is too low, increase correlations;
    # if too high, decrease
    if (current_r_squared < r_squared) {
      scale_factor <- 1.1 # Increase correlations
    } else {
      scale_factor <- 0.9 # Decrease correlations
    }

    # Scale off-diagonal elements
    new_matrix <- cor_matrix
    for (i in 1:(k - 1)) {
      for (j in (i + 1):k) {
        new_val <- cor_matrix[i, j] * scale_factor
        # Constrain to range
        new_val <- max(iv_cor_range[1], min(iv_cor_range[2], new_val))
        new_matrix[i, j] <- new_val
        new_matrix[j, i] <- new_val
      }
    }

    # Ensure positive definite
    if (any(eigen(new_matrix, only.values = TRUE)$values <= 0)) {
      new_matrix <- as.matrix(Matrix::nearPD(new_matrix, corr = TRUE)$mat)
    }

    # Calculate new R-sq
    new_r_squared <- calc_r_squared(new_matrix)
    new_diff <- abs(new_r_squared - r_squared)

    # Check if improved
    if (new_diff < best_diff) {
      best_matrix <- new_matrix
      best_diff <- new_diff
      cor_matrix <- new_matrix
      current_r_squared <- new_r_squared

      # Check convergence
      if (best_diff <= tolerance) {
        return(list(
          cormatrix = best_matrix,
          achieved_r_squared = current_r_squared,
          iterations = iter,
          converged = TRUE
        ))
      }
    } else {
      # If not improving, try smaller adjustments
      scale_factor <- ifelse(current_r_squared < r_squared, 1.05, 0.95)
    }
  }

  # Return best solution found, even if not converged
  warning(sprintf(
    "Optimisation did not fully converge.
    \nBest R-sq achieved: %.4f (target: %.4f, diff: %.4f)",
    calc_r_squared(best_matrix), r_squared, best_diff
  ))

  return(list(
    cormatrix = best_matrix,
    achieved_r_squared = calc_r_squared(best_matrix),
    iterations = max_iterations,
    converged = FALSE
  ))
}

# tests/testthat/test-makeScalesRegression.R

test_that("makeScalesRegression: basic generation with provided IV correlation matrix", {
  set.seed(123)

  iv_corr <- matrix(c(1.0, 0.3, 0.3, 1.0), nrow = 2, byrow = TRUE)
  n <- 80L

  res <- suppressWarnings(
    makeScalesRegression(
      n = n,
      beta_std = c(0.4, 0.3),
      r_squared = 0.35,
      iv_cormatrix = iv_corr,
      iv_means = c(3.0, 3.5),
      iv_sds = c(1.0, 0.9),
      dv_mean = 3.8,
      dv_sd = 1.1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5,
      items_iv = 4,
      items_dv = 4,
      var_names = c("Attitude", "Intention", "Behaviour")
    )
  )

  # Structure
  testthat::expect_s3_class(res, "makeScalesRegression")
  testthat::expect_true(is.list(res))
  testthat::expect_true(is.data.frame(res$data))

  testthat::expect_identical(nrow(res$data), n)
  testthat::expect_identical(ncol(res$data), 3L)
  testthat::expect_identical(colnames(res$data), c("Attitude", "Intention", "Behaviour"))

  # Full correlation matrix has correct dim and names
  testthat::expect_true(is.matrix(res$full_cormatrix))
  testthat::expect_identical(dim(res$full_cormatrix), c(3L, 3L))
  testthat::expect_identical(
    rownames(res$full_cormatrix),
    c("Attitude", "Intention", "Behaviour")
  )

  # R-squared achieved is close to target (allowing for simulation noise)
  ach_r2 <- res$achieved_stats$r_squared
  testthat::expect_true(is.numeric(ach_r2))
  testthat::expect_lt(abs(ach_r2 - 0.35), 0.05)

  # Standardised betas achieved roughly match target (ignore names)
  testthat::expect_equal(
    unname(res$achieved_stats$beta_std),
    c(0.4, 0.3),
    tolerance = 0.12
  )

  # Means and SDs are in the right ballpark for IVs and DV (ignore names)
  targ_means <- c(res$target_stats$iv_means, res$target_stats$dv_mean)
  targ_sds <- c(res$target_stats$iv_sds, res$target_stats$dv_sd)

  ach_means <- c(res$achieved_stats$iv_means, res$achieved_stats$dv_mean)
  ach_sds <- c(res$achieved_stats$iv_sds, res$achieved_stats$dv_sd)

  testthat::expect_equal(unname(ach_means), targ_means, tolerance = 0.30)
  testthat::expect_equal(unname(ach_sds), targ_sds, tolerance = 0.35)
})


test_that("makeScalesRegression: optimisation mode works when iv_cormatrix is NULL", {
  set.seed(456)

  n <- 120L
  k <- 3L

  res <- suppressMessages(
    makeScalesRegression(
      n = n,
      beta_std = c(0.3, 0.25, 0.2),
      r_squared = 0.40,
      iv_cormatrix = NULL, # optimisation mode
      iv_cor_mean = 0.3,
      iv_cor_variance = 0.02,
      iv_cor_range = c(-0.7, 0.7),
      iv_means = c(3.0, 3.2, 2.8),
      iv_sds = c(1.0, 0.9, 1.1),
      dv_mean = 3.5,
      dv_sd = 1.0,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5,
      items_iv = 4,
      items_dv = 5
    )
  )

  # Structure of result
  testthat::expect_s3_class(res, "makeScalesRegression")
  testthat::expect_true(is.list(res$optimisation_info))
  testthat::expect_true(is.data.frame(res$data))
  testthat::expect_identical(nrow(res$data), n)
  testthat::expect_identical(ncol(res$data), k + 1L)

  # Default variable names are IV1..IVk, DV
  testthat::expect_identical(
    colnames(res$data),
    c(paste0("IV", seq_len(k)), "DV")
  )

  # Optimised IV correlation matrix in target_stats has correct shape
  testthat::expect_true(is.matrix(res$target_stats$iv_cormatrix))
  testthat::expect_identical(dim(res$target_stats$iv_cormatrix), c(k, k))

  # iv_dv_cors has length k and names of IVs
  testthat::expect_length(res$iv_dv_cors, k)
  testthat::expect_identical(names(res$iv_dv_cors), paste0("IV", seq_len(k)))

  # Achieved R-squared is reasonably close to target, given optimisation & simulation
  ach_r2 <- res$achieved_stats$r_squared
  testthat::expect_lt(abs(ach_r2 - 0.40), 0.08)
})

test_that("makeScalesRegression: r_squared outside [0, 1] throws error", {
  set.seed(1)
  iv_corr <- diag(2)

  testthat::expect_error(
    makeScalesRegression(
      n = 50L,
      beta_std = c(0.4, 0.3),
      r_squared = 1.2, # invalid
      iv_cormatrix = iv_corr,
      iv_means = c(3, 3.5),
      iv_sds = c(1, 0.9),
      dv_mean = 4,
      dv_sd = 1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5
    ),
    regexp = "r_squared must be between 0 and 1"
  )
})

test_that("makeScalesRegression: iv_cormatrix dimension mismatch throws error", {
  set.seed(2)
  iv_corr <- diag(3) # 3x3, but only 2 betas

  testthat::expect_error(
    makeScalesRegression(
      n = 40L,
      beta_std = c(0.4, 0.3),
      r_squared = 0.25,
      iv_cormatrix = iv_corr,
      iv_means = c(3, 3),
      iv_sds = c(1, 1),
      dv_mean = 4,
      dv_sd = 1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5
    ),
    regexp = "iv_cormatrix dimensions must match number of IVs"
  )
})

test_that("makeScalesRegression: iv_means / iv_sds length mismatch throws error", {
  set.seed(3)
  iv_corr <- diag(3)

  testthat::expect_error(
    makeScalesRegression(
      n = 40L,
      beta_std = c(0.4, 0.3, 0.2),
      r_squared = 0.25,
      iv_cormatrix = iv_corr,
      iv_means = c(3, 3), # too short
      iv_sds = c(1, 1, 1),
      dv_mean = 4,
      dv_sd = 1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5
    ),
    regexp = "iv_means and iv_sds must have same length as beta_std"
  )

  testthat::expect_error(
    makeScalesRegression(
      n = 40L,
      beta_std = c(0.4, 0.3, 0.2),
      r_squared = 0.25,
      iv_cormatrix = iv_corr,
      iv_means = c(3, 3, 3),
      iv_sds = c(1, 1), # too short
      dv_mean = 4,
      dv_sd = 1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5
    ),
    regexp = "iv_means and iv_sds must have same length as beta_std"
  )
})

test_that("makeScalesRegression: var_names length mismatch throws error", {
  set.seed(4)
  iv_corr <- diag(2)

  testthat::expect_error(
    makeScalesRegression(
      n = 50L,
      beta_std = c(0.4, 0.3),
      r_squared = 0.30,
      iv_cormatrix = iv_corr,
      iv_means = c(3, 3.5),
      iv_sds = c(1, 1),
      dv_mean = 4,
      dv_sd = 1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5,
      var_names = c("TooFew", "NamesOnly") # length 2 instead of k+1 = 3
    ),
    regexp = "var_names must have length k\\+1"
  )
})


test_that("print.makeScalesRegression produces informative output", {
  set.seed(321)

  iv_corr <- matrix(c(1.0, 0.3, 0.3, 1.0), nrow = 2, byrow = TRUE)

  res <- suppressWarnings(
    makeScalesRegression(
      n = 40L,
      beta_std = c(0.4, 0.3),
      r_squared = 0.35,
      iv_cormatrix = iv_corr,
      iv_means = c(3.0, 3.5),
      iv_sds = c(1.0, 0.9),
      dv_mean = 3.8,
      dv_sd = 1.1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5,
      items_iv = 4,
      items_dv = 4,
      var_names = c("Attitude", "Intention", "Behaviour")
    )
  )

  out <- capture.output(print(res))

  testthat::expect_true(any(grepl("Regression Data Generation Results", out)))
  testthat::expect_true(any(grepl("Target R-squared", out)))
  testthat::expect_true(any(grepl("Achieved R-squared", out)))

  # print() should return the object invisibly
  testthat::expect_invisible(print(res))
})

---
title: "LikertMakeR vignette"
author: "Hume Winzar"
date: "February 2026"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{LikertMakeR vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


```{r setup, echo=FALSE}
library(LikertMakeR)
```


```{r logo, fig.align='center', echo=FALSE, out.width = '20%'}
knitr::include_graphics("img/LikertMakeR_3.png")
```


<!-- ![LikertMakeR](LikertMakeR_hex.png){#id .class width=25%} -->


_**LikertMakeR**_ [(Winzar, 2025)](https://cran.r-project.org/package=LikertMakeR) 
lets you create synthetic Likert-scale, or related rating-scale, data.  
Set the mean, standard deviation, and correlations or model coefficients, 
and the package generates data matching those properties. 
It can also rearrange existing data columns to achieve a desired 
correlation structure or generate data based on _Cronbach's Alpha_,
_factor correlations_, _regression_ or _ANOVA coefficients_, 
or other summary statistics.

## Purpose

The package should be useful for teaching in the Social Sciences, 
and for scholars who wish to "replicate" or "reverse engineer" 
rating-scale data for further analysis and visualisation when only 
summary statistics have been reported. 

## Motivation

I was prompted to write the core functions in _LikertMakeR_ after reviewing 
too many journal article submissions where authors presented questionnaire 
results with only means and standard deviations (often only the means), 
with no apparent understanding of scale distributions, 
and their impact on scale properties. 

Hopefully, this tool will help researchers, teachers & students, 
and other reviewers, to better think about rating-scale distributions, 
and the effects of covariation, scale boundaries, 
and number of items in a scale. 
Researchers can also use _LikertMakeR_ to create dummy data to prepare 
analyses ahead of a formal survey.

## Rating scale properties

A _Likert_ scale is the mean, or sum, of several ordinal rating scales. 
Typically, they are bipolar (usually "agree-disagree") responses to 
propositions that are determined to be moderately-to-highly correlated 
and that capture some facet of a theoretical construct.

### Rating scales have bounds and discrete measurement intervals

Rating scales, such as _Likert_ scales, are not continuous or unbounded.

For example, a 5-point Likert scale that is constructed with, 
say, five items (questions) will have a summed range of between 5 
(all rated '1') and 25 (all rated '5') with all integers in between, 
and the mean range will be '1' to '5' with intervals of 1/5=0.20. 
A 7-point Likert scale constructed from eight items will have a summed 
range between 8 (all rated '1') and 56 (all rated '7') with all integers 
in between, 
and the mean range will be '1' to '7' with intervals of 1/8=0.125.

Technically, because they are bounded and not continuous, parametric 
statistics, such as mean, standard deviation, and correlation, should not be 
applied to summated rating scales. 
In practice, however, parametric statistics are commonly used in the social 
sciences because:

  1. they are in common usage and easily understood,
  
  2. In practice, all measures are bounded by the constraints of the 
  measurement tool, meaning that they also have upper and lower boundaries 
  and discrete units of measurement, which means that: 
  
  3. results and conclusions drawn from technically-correct non-parametric statistics are _(almost)_ always the same as for parametric statistics for such data. <br /> 
For example, [D'Alessandro _et al._ (2020)](https://cengage.com.au/sem121/marketing-research-5th-edition-dalessandro-babin-zikmund) 
argue that a summated scale, made with multiple items, 
"approaches" an interval scale measure, implying that parametric statistics 
are quite acceptable. 

### A single 1-5 rating scale is **_NOT_** a Likert scale - it may be an Likert-scale item.

Likert-scale items, such as responses to a single 1-to-5 agree-disagree 
question, should not be analysed by professional or responsible researchers. 
There is too much random error in a single item. 
[Rensis Likert (1932)](https://archive.org/details/likert-1932/mode/2up) 
designed the scale with the logic that a random overstatement on one item 
is likely to be compensated by a random understatement on another 
item, so that, when multiple items are combined, we get a reasonably 
consistent, internally reliable, measure of the target construct.

### Most rating scales are skewed

Rating-scale boundaries define minima and maxima for any scale values. 
If the mean is close to one boundary then data points will gather more 
closely to that boundary. <br />
If the mean is not in the middle of a scale, 
then the data will be always skewed, as shown in the following plots.


```{r skew, fig.align='center', echo=FALSE, out.width = '85%', fig.cap="Off-centre means always give skewed distribution in bounded rating scales"}
knitr::include_graphics("img/skew_chart.png")
```

_____

## _LikertMakeR_ functions

 * *lfast()* generate a vector of values with predefined 
 mean and standard deviation. 
 
 * *lcor()* takes a dataframe of rating-scale values and rearranges the 
 values in each column so that the columns are correlated to match a 
 predefined correlation matrix.
 
 * *makeCorrAlpha* constructs a random correlation matrix of given 
 dimensions from a predefined _Cronbach's Alpha_.
 
 * *makeCorrLoadings* constructs a random correlation matrix from a given 
 _factor loadings_ matrix, and _factor-correlations_ matrix.
 

  * *makeScales()* is a wrapper function for _lfast()_ 
 and _lcor()_ to generate items or summated scales with predefined 
 first and second moments and a predefined correlation matrix. 
 This function replaces _makeItems()_ and now includes multi-item measures. 
 
 * *makeItemsScale()* generates a random dataframe of scale items based 
 on a predefined summated scale with a desired Cronbach's Alpha.
 
 * *makePaired()* generates a dataframe of two correlated columns based 
 on summary data from a paired-sample t-test.
 
 * *makeRepeated()* generates a dataframe of `k` correlated columns based 
 on summary data from a repeated-samples ANOVA.
 
 * *makeScalesRegression()* generates a dataframe based on results of 
 output from multiple-regression - R<sup>2</sup>, standardised betas, and IV 
 correlations (if available).
 
 * *correlateScales()* creates a dataframe of correlated summated scales 
 as one might find in completed survey questionnaire and possibly used 
 in a  Structural Equation model.
 
 * Helper Functions 
 
     - *alpha()* calculates Cronbach's Alpha from a given correlation 
     matrix or a given dataframe.
 
     - *eigenvalues()* calculates eigenvalues of a correlation matrix, 
     reports on positive-definite status of the matrix and, optionally, 
     displays a scree plot to visualise the eigenvalues.
     
     - *reliability()* Computes internal consistency reliability estimates 
     for a single-factor scale, including Cronbach’s alpha, 
     McDonald’s omega (total), and optional ordinal (polychoric-based) 
     variants and Confidence intervals

_____


# Using _LikertMakeR_

## Download and Install _LikertMakeR_

### from _CRAN_

    > ```
    >
    > install.packages("LikertMakeR")
    > library(LikertMakeR)
    >
    > ```

### development version from _GitHub_.

    > ```
    > 
    > library(devtools)
    > install_github("WinzarH/LikertMakeR")
    > library(LikertMakeR)
    >
    > ```

____

## Generate synthetic rating-scale data

### _lfast()_

- **_lfast()_** applies a simple evolutionary algorithm 
which draws repeated random samples from a scaled *Beta* distribution. 
It produces a vector of values with mean and standard 
deviation typically correct to two decimal places.

To synthesise a rating scale with **_lfast()_**, the user must input 
the following parameters:

 - **_n_**: sample size

 - **_mean_**: desired mean

 - **_sd_**: desired standard deviation

 - **_lowerbound_**: desired lower bound

 - **_upperbound_**: desired upper bound

 - **_items_**: number of items making the scale - default = 1

An earlier version of _LikertMakeR_ had a function, _lexact()_, 
which was slow and no more accurate than the latest version of _lfast()_. 
 So, _lexact()_ is now deprecated. 
 

#### _lfast()_ example

##### a four-item, five-point Likert scale


```{r lfastExample}
nItems <- 4
mean <- 2.5
sd <- 0.75

x1 <- lfast(
  n = 512,
  mean = mean,
  sd = sd,
  lowerbound = 1,
  upperbound = 5,
  items = nItems
)
```



```{r fig1, fig.height=4, fig.width=5, fig.align='center', echo = FALSE, crop = TRUE, fig.cap="Example: 4-item, 1-5 Likert scale"}
## distribution of x

hist(x1,
  cex.axis = 0.5, cex.main = 0.75,
  breaks = seq(from = (1 - (1 / 8)), to = (5 + (1 / 8)), by = (1 / 4)),
  col = "skyblue", xlab = NULL, ylab = NULL,
  main = paste0("mu=", round(mean(x1), 2), ", sd=", round(sd(x1), 2))
)
```


##### an 11-point likelihood-of-purchase scale

###### _lfast()_

```{r lfastx2}
x2 <- lfast(256, 3, 2.5, 0, 10)
```


```{r fig2, fig.height=4, fig.width=5, fig.align='center', echo = FALSE, crop = TRUE, fig.cap="Example: likelihood-of-purchase scale"}
## generate histogram

hist(x2,
  cex.axis = 0.5, cex.main = 0.75,
  breaks = seq(from = -0.5, to = 10.5, by = 1),
  col = "skyblue", xlab = NULL, ylab = NULL,
  main = paste0("mu=", round(mean(x2), 2), ", sd=", round(sd(x2), 2))
)
```

____

## Correlating rating scales

The function, **_lcor()_**, rearranges the values in the columns of a 
data-set so that they are correlated at a specified level. 
It does not change the values - it swaps their positions within each 
column so that univariate statistics do not change, 
but their correlations with other vectors do.

### _lcor()_

**_lcor()_** systematically selects pairs of values in a column 
and swaps their places, and checks to see if this swap improves 
the correlation matrix. 
If the revised dataframe produces a correlation matrix closer to the 
target correlation matrix, then the swap is retained. 
Otherwise, the values are returned to their original places. 
This process is iterated across each column. 

To create the desired correlated data, the user must define the 
following parameters:

- **_data_**: a starter data set of rating-scales. 
Number of columns must match the dimensions of the _target_ 
correlation matrix.

- **_target_**: the target correlation matrix. 


### _lcor()_ example

Let's generate some data: three 5-point Likert scales, 
each with five items.

```{r lcorExample}
## generate uncorrelated synthetic data
n <- 128
lowerbound <- 1
upperbound <- 5
items <- 5

mydat3 <- data.frame(
  x1 = lfast(n, 2.5, 0.75, lowerbound, upperbound, items),
  x2 = lfast(n, 3.0, 1.50, lowerbound, upperbound, items),
  x3 = lfast(n, 3.5, 1.00, lowerbound, upperbound, items)
)
```

The first six observations from this dataframe are:

```{r mydat3Head, echo = FALSE}
head(mydat3, 6)
```

And the first and second moments (to 3 decimal places) are:

```{r mydat3Moments, echo = FALSE}
moments <- data.frame(
  mean = apply(mydat3, 2, mean) |> round(3),
  sd = apply(mydat3, 2, sd) |> round(3)
) |> t()

moments
```

We can see that the data have first and second moments are very close 
to what is expected. 

As we should expect, randomly-generated synthetic data have low correlations:

```{r mydat3Cor, echo = FALSE}
cor(mydat3) |> round(2)
```

Now, let's define a target correlation matrix:

```{r tgt3}
## describe a target correlation matrix
tgt3 <- matrix(
  c(
    1.00, 0.85, 0.75,
    0.85, 1.00, 0.65,
    0.75, 0.65, 1.00
  ),
  nrow = 3
)
```

So now we have a dataframe with desired first and second moments, 
and a target correlation matrix.

```{r new3}
## apply lcor() function
new3 <- lcor(data = mydat3, target = tgt3)
```

Values in each column of the new dataframe do not change from the original;  the values are rearranged. 

The first ten observations from this dataframe are:

```{r new3Head, echo = FALSE}
head(new3, 10)
```

And the new dataframe is correlated close to our desired correlation matrix; 
here presented to 3 decimal places:

```{r new3Cor, echo = FALSE}
cor(new3) |> round(3)
```

____


## Generate a correlation matrix from Cronbach's Alpha

### makeCorrAlpha()

**_makeCorrAlpha()_**, constructs a random correlation matrix of given 
 dimensions and predefined _Cronbach's alpha_. 
 
To create the desired correlation matrix, the user must define the 
following parameters:

- **_items_**: or "k" - the number of rows and columns of the 
 desired correlation matrix. 

- **_alpha_**: the target value for Cronbach's Alpha

- **_variance_**: a notional variance coefficient to affect the spread of 
values in the correlation matrix. Default = '0.1'. 
A value of '0' produces a matrix where all off-diagonal correlations 
are equal. 
Setting 'variance = 0.25' or more may be infeasible, so the function should gracefully adjust the `variance` parameter downwards to achieve PD status.

-  **_alpha_noise_**: Controls random variation in the target
 Cronbach's alpha before the correlation matrix is constructed.

When `alpha_noise = 0` (default), the requested alpha is
reproduced deterministically (subject to numerical tolerance).

When `alpha_noise > 0`, a small amount of random variation is
added to the requested alpha prior to constructing the matrix. This
can be useful in teaching or simulation settings where slightly
different reliability values are desired across repeated runs.

Internally, noise is added on the Fisher _z_-transformed scale
to ensure the resulting alpha remains within valid bounds (0, 1).

Typical guidance:

   - 0.00 — deterministic alpha (default)
   - 0.02 — very small variation
   - 0.05 — moderate variation
   - 0.10 — substantial variation (caution)

 Larger values increase the spread of achieved alpha across runs.

- **_diagnostics_**: Logical. 
  If `TRUE`, returns a list containing the correlation matrix and a 
  diagnostics list (target/achieved alpha, average inter-item correlation, 
  eigenvalues, PD flag, and key arguments).
  If `FALSE` (default), returns the correlation matrix only.

#### _makeCorrAlpha()_ examples

##### Four variables, alpha = 0.85

```{r cor_matrix_4}
## define parameters
items <- 4
alpha <- 0.85

## apply makeCorrAlpha() function
set.seed(42)

cor_matrix_4 <- makeCorrAlpha(items, alpha)
```

_makeCorrAlpha()_ produced the following correlation matrix 
(to three decimal places):

```{r cor_matrix_4Print, echo = FALSE}
cor_matrix_4 |> round(3)
```



##### test output with Helper functions

```{r cor_matrix_4Alpha}
## using helper function alpha()

alpha(cor_matrix_4)
```


```{r fig3, fig.height=4, fig.width=5, fig.align='center', echo = TRUE, crop = TRUE}
## using helper function eigenvalues()

eigenvalues(cor_matrix_4, 1)
```



#### makeCorrAlpha() with diagnostics output

```{r cor_matrix_d}
## apply makeCorrAlpha() with diagnostics
set.seed(42)

cor_matrix_5 <- makeCorrAlpha(
  items = 6,
  alpha = 0.90,
  diagnostics = TRUE
)
```

##### diagnostics output

```{r cor_matrix_5_mat}
## output
cor_matrix_5$R |> round(2)

cor_matrix_5$diagnostics
```



____

## Generate a correlation matrix from factor loadings

### makeCorrLoadings

 **_makeCorrLoadings()_** generates a correlation matrix from factor loadings 
 and factor correlations as might be seen in _Exploratory Factor Analysis_ 
 (**EFA**) or a _Structural Equation Model_ (**SEM**).
 
#### makeCorrLoadings() usage

      makeCorrLoadings(loadings, factorCor = NULL, uniquenesses = NULL, nearPD = FALSE)

##### makeCorrLoadings() arguments

  - **_loadings_**:  `k` (items) by `f` (factors)
  matrix of _standardised_ factor loadings. 
  Item names and Factor names can be taken from the row_names (items) 
  and the column_names (factors), if present.

  - **_factorCor_**:  `f` x `f` factor correlation matrix. 
  If not present, then we assume that the factors are uncorrelated 
  (orthogonal), which is rare in practice, and the function applies an 
  identity matrix for _factor_cor_.
 
  - **_uniquenesses_**: length `k` vector of uniquenesses.
     If NULL, the default, compute from the calculated communalities.
 
  - **_nearPD_**: (logical) If TRUE, then the function calls the _nearPD_ 
  function from the _**Matrix**_ package to transform the resulting 
  correlation matrix onto the nearest Positive Definite matrix. 
  Obviously, this only applies if the resulting correlation matrix is not 
  positive definite. (_It should never be needed._)

##### Note

"Censored" loadings (for example, where loadings less than some small value 
(often '0.30'), are removed for ease-of-communication) tend to severely reduce 
the accuracy of the `makeCorrLoadings()` function. 
For a detailed demonstration, see the vignette file,
**makeCorrLoadings_Validate**.


### makeCorrLoadings() examples

####  Typical application from published EFA results

##### define parameters 


```{r parameters_makeCorrLoadings, echo=TRUE}
## Example loadings

factorLoadings <- matrix(
  c(
    0.05, 0.20, 0.70,
    0.10, 0.05, 0.80,
    0.05, 0.15, 0.85,
    0.20, 0.85, 0.15,
    0.05, 0.85, 0.10,
    0.10, 0.90, 0.05,
    0.90, 0.15, 0.05,
    0.80, 0.10, 0.10
  ),
  nrow = 8, ncol = 3, byrow = TRUE
)

## row and column names

rownames(factorLoadings) <- c("Q1", "Q2", "Q3", "Q4", "Q5", "Q6", "Q7", "Q8")
colnames(factorLoadings) <- c("Factor1", "Factor2", "Factor3")

## Factor correlation matrix**

factorCor <- matrix(
  c(
    1.0,  0.5, 0.4,
    0.5,  1.0, 0.3,
    0.4,  0.3, 1.0
  ),
  nrow = 3, byrow = TRUE
)
```
  

##### Apply the function

```{r apply_makeCorrLoadings, echo=TRUE}
## apply makeCorrLoadings() function
itemCorrelations <- makeCorrLoadings(factorLoadings, factorCor)

## derived correlation matrix to two decimal places
round(itemCorrelations, 2)
```

##### Test makeCorrLoadings() output

```{r test_makeCorrLoadings, echo=TRUE}
## correlated factors mean that eigenvalues should suggest two or three factors
eigenvalues(cormatrix = itemCorrelations, scree = TRUE)
```


#####  Assuming orthogonal factors

```{r orthogonal_makeCorrLoadings, echo=TRUE}
## orthogonal factors are assumed when factor correlation matrix is not included
orthogonalItemCors <- makeCorrLoadings(factorLoadings)

## derived correlation matrix to two decimal places
round(orthogonalItemCors, 2)
```

##### Test orthogonal output

```{r test_orthogonal, echo=TRUE}
## eigenvalues should suggest exactly  three factors
eigenvalues(cormatrix = orthogonalItemCors, scree = TRUE)
```
____


## Generate a dataframe of rating scales from a correlation matrix and predefined moments

### makeScales()

**_makeScales()_** generates a dataframe of random discrete
values so the data replicate a set of scale items or summated rating
scales, and are correlated close to a predefined correlation matrix. 

Generally, means, standard deviations, and correlations are correct to 
two decimal places.

_makeScales()_ is a wrapper function for 

 * _lfast()_, which takes repeated samples selecting a vector that
 best fits the desired moments, and
 
 * _lcor()_, which rearranges values in each column of the dataframe
 so they closely match the desired correlation matrix.
 

To create the desired dataframe, the user must define the 
following parameters:

- **_n_**: number of observations 

- **_dfMeans_**: a vector of length `k` of desired means of each variable

- **_dfSds_**: a vector of length `k` of desired standard deviations of 
 each variable
 
 - **_lowerbound_**: a vector of length `k` of values for the lower bound 
 of each variable. default = '1'

- **_upperbound_**: a vector of length `k` of values for the upper bound 
 of each variable. Default = '5'
 
- **_items_**: a vector of length `k` of the number of items in each 
variable. Default = '1'.
 
- **_cormatrix_**: a target correlation matrix with `k` rows and `k` columns. 


### _makeScales()_ examples

#### _makeScales()_ example #1. four correlated items

```{r makeScalesExample1}
## define parameters
n <- 128
dfMeans <- c(2.5, 3.0, 3.0, 3.5)
dfSds <- c(1.0, 1.0, 1.5, 0.75)
lowerbound <- rep(1, 4)
upperbound <- rep(5, 4)

corMat <- matrix(
  c(
    1.00, 0.25, 0.35, 0.45,
    0.25, 1.00, 0.70, 0.75,
    0.35, 0.70, 1.00, 0.85,
    0.45, 0.75, 0.85, 1.00
  ),
  nrow = 4, ncol = 4
)

var_names <- c("var1", "var2", "var3", "var4")
colnames(corMat) <- var_names
rownames(corMat) <- var_names

## apply makeScales() function
df <- makeScales(
  n = n,
  means = dfMeans,
  sds = dfSds,
  lowerbound = lowerbound,
  upperbound = upperbound,
  cormatrix = corMat
)
```

##### Structure of new dataframe

```{r str_makeScalesExample1}
## test the function
str(df)
```

##### Means should be correct to two decimal places

```{r moments_makeScalesExample1}
### means should be correct to two decimal places
dfmoments <- data.frame(
  mean = apply(df, 2, mean) |> round(3),
  sd = apply(df, 2, sd) |> round(3)
) |> t()

dfmoments
```

##### Correlations should be correct to two decimal places

```{r cors_makeScalesExample1}
### correlations should be correct to two decimal places
cor(df) |> round(3)
```

#### _makeScales()_ example #2. four Likert scales

  - **Brand Trust (BT)**  - The confidence a consumer has in a brand’s reliability and honesty.
  - **Brand Satisfaction (BS)** - Overall affective evaluation of the brand experience. 
  - **Brand Love (BL)** - Deep emotional attachment toward the brand. 
  - **Brand Loyalty (BLY)**  - Intention to repurchase and recommend the brand. 


```{r makeScalesExample2}
## define parameters
n <- 256
dfMeans <- c(3.9, 4.1, 3.6, 4.0)
dfSds <- c(0.6, 0.5, 0.8, 0.7)
lowerbound <- rep(1, 4)
upperbound <- rep(5, 4)
items <- c(4, 3, 4, 3)

corMat <- matrix(
  c(
    1.00, 0.75, 0.60, 0.70,
    0.75, 1.00, 0.65, 0.72,
    0.60, 0.65, 1.00, 0.68,
    0.70, 0.72, 0.68, 1.00
  ),
  nrow = 4, ncol = 4
)

scale_names <- c("BT", "BS", "BL", "BLY")
rownames(corMat) <- scale_names
colnames(corMat) <- scale_names

## apply makeScales() function
df <- makeScales(
  n = n,
  means = dfMeans,
  sds = dfSds,
  lowerbound = lowerbound,
  upperbound = upperbound,
  items = items,
  cormatrix = corMat
)

## test the function
head(df)
tail(df)

### means should be correct to two decimal places
dfmoments <- data.frame(
  mean = apply(df, 2, mean) |> round(3),
  sd = apply(df, 2, sd) |> round(3)
) |> t()

dfmoments

### correlations should be correct to two decimal places
cor(df) |> round(3)
```


____


## Generate a dataframe from Cronbach's Alpha and predefined moments

This is a two-step process:
  
  1. apply **_makeCorrAlpha()_** to generate a correlation matrix from 
   desired alpha,
  
  2. apply **_makeScales()_** to generate rating-scale items from the 
    correlation matrix and desired moments
  
Required parameters are:

  - **_k_**: number items/ columns 
  
  - **_alpha_**: a target Cronbach's Alpha. 

  - **_n_**: number of observations 

  - **_lowerbound_**: a vector of length `k` of values for the lower bound 
   of each variable 

  - **_upperbound_**: a vector of length `k` of values for the upper bound 
   of each variable

  - **_means_**: a vector of length `k` of desired means of each variable

  - **_sds_**: a vector of length `k` of desired standard deviations of 
   each variable
 
### Step 1: Generate a correlation matrix

```{r genCorrelation}
## define parameters
k <- 6
myAlpha <- 0.85

## generate correlation matrix
set.seed(42)
myCorr <- makeCorrAlpha(items = k, alpha = myAlpha)

## display correlation matrix
myCorr |> round(3)

### checking Cronbach's Alpha
alpha(cormatrix = myCorr)
```


### Step 2: Generate dataframe

```{r gendataframe}
## define parameters
n <- 256
myMeans <- c(2.75, 3.00, 3.00, 3.25, 3.50, 3.5)
mySds <- c(1.00, 0.75, 1.00, 1.00, 1.00, 1.5)
lowerbound <- rep(1, k)
upperbound <- rep(5, k)

## Generate Items
myItems <- makeScales(
  n = n, means = myMeans, sds = mySds,
  lowerbound = lowerbound, upperbound = upperbound,
  items = 1,
  cormatrix = myCorr
)

## resulting dataframe
head(myItems)
tail(myItems)

## means and standard deviations
myMoments <- data.frame(
  means = apply(myItems, 2, mean) |> round(3),
  sds = apply(myItems, 2, sd) |> round(3)
) |> t()
myMoments

## Cronbach's Alpha of dataframe
alpha(NULL, myItems)
```

#### Summary plots of new dataframe

```{r fig5, fig.height=5, fig.width=5, fig.align='center', echo=FALSE, warning=FALSE, crop = TRUE, fig.cap="Summary of dataframe from makeScales() function"}
# Correlation panel

panel.cor <- function(x, y) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))
  r <- round(cor(x, y), digits = 2)
  txt <- paste0(r)
  cex.cor <- 0.8 / strwidth(txt)
  text(0.5, 0.5, txt, cex = 1.25)
}

# Customize upper panel

upper.panel <- function(x, y) {
  points(x, y, pch = 19, col = "#0000ff11")
}

# diagonals

panel.hist <- function(x, ...) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(usr[1:2], 0, 1.5))
  h <- hist(x, plot = FALSE)
  breaks <- h$breaks
  nB <- length(breaks)
  y <- h$counts
  y <- y / max(y)
  rect(breaks[-nB], 0, breaks[-1], y, col = "#87ceeb66")
}

# Create the plots

pairs(myItems,
  lower.panel = panel.cor,
  upper.panel = upper.panel,
  diag.panel = panel.hist
)
```

____


## Generate a dataframe of rating-scale items from a summated rating scale

### makeItemsScale()

  -  **_makeItemsScale()_** generates a dataframe of rating-scale items from a summated rating scale and desired _Cronbach's Alpha_.


To create the desired dataframe, define the 
following parameters:

  - **_scale_**: a vector or dataframe of the summated rating scale.
   Should range from ('lowerbound' * 'items') to ('upperbound' * 'items') 
 
  - **_lowerbound_**: lower bound of the scale item
   (example: '1' in a '1' to '5' rating) 
 
  - **_upperbound_**:  upper bound of the scale item
   (example: '5' in a '1' to '5' rating) 
 
  - **_items_**: k, or number of columns to generate
  
  - **_alpha_**: desired Cronbach's Alpha. Default = '0.8'
  
  - **_summated_**: (logical) If TRUE, the scale is treated as a summed
   score (e.g., 4–20 for four 5-point items).
   If FALSE, it is treated as an averaged score
   (e.g., 1–5 in 0.25 increments). Default = TRUE.
   
  - **_progress_**: (logical) If TRUE (default is `interactive()`), 
   a progress bar is displayed during optimisation. 
   Set to FALSE for slightly faster execution. 
  

#### _makeItemsScale()_ Example:



#### create items with _makeItemsScale()_

```{r makeItemsScale_example_1, fig.height=4, fig.width=5, fig.align='center', crop = TRUE}
summatedScale <- c(11, 8, 14, 12, 9, 12, 10, 9, 8, 18, 16, 16, 14, 8, 15, 12)

## apply makeItemsScale() function

newItems <- makeItemsScale(
  scale = summatedScale,
  lowerbound = 1,
  upperbound = 5,
  items = 4
)

###  observations and summated scale
head(cbind(newItems, summatedScale), 16)

### correlation matrix
cor(newItems) |> round(2)

### default Cronbach's alpha = 0.80
alpha(data = newItems) |> round(4)

### calculate eigenvalues and print scree plot
eigenvalues(cor(newItems), 1) |> round(3)
```


####  _makeItemsScale()_ with same summated values and higher _alpha_

```{r makeItemsScale_example_2, fig.height=4, fig.width=5, fig.align='center', crop = TRUE}
## apply makeItemsScale() function
newItems_2 <- makeItemsScale(
  scale = summatedScale,
  lowerbound = 1,
  upperbound = 5,
  items = 4,
  alpha = 0.9
)

### observations and summated scale
head(cbind(newItems_2, summatedScale), 16)

### correlation matrix
cor(newItems_2) |> round(2)

### requested Cronbach's alpha = 0.90
alpha(data = newItems_2) |> round(4)

### calculate eigenvalues and print scree plot
eigenvalues(cor(newItems_2), 1) |> round(3)
```


----

## Create a dataframe for a t-test

Generating a data for an **independent-samples t-test** is trivial 
with _LikertMakeR_. 
But a dataframe for a **paired-sample t-test** is tricky because the 
observations are related to each other. 
That is, we must generate a dataframe of correlated observations.

### Independent-samples t-test

Note that such tests don't even require the same sample-size.

```{r independent-samples_t-test, echo = TRUE}
## define parameters
lower <- 1
upper <- 5
items <- 6

## generate two independent samples
x1 <- lfast(
  n = 20, mean = 2.5, sd = 0.75,
  lowerbound = lower, upperbound = upper, items = items
)
x2 <- lfast(
  n = 30, mean = 3.0, sd = 0.85,
  lowerbound = lower, upperbound = upper, items = items
)

## run independent-samples t-test
t.test(x1, x2)
```

### makePaired() paired-sample t-test

_**makePaired()**_ generates correlated values so the data replicate rating 
scales taken, for example, in a before and after experimental design. 
The function is effectively a wrapper function for _lfast()_ and _lcor()_ with 
the addition of a _t-statistic_ from which the between-column correlation is 
inferred.

Paired t-tests apply to observations that are associated with each other. 
For example: the same people rating the same object before and after a 
treatment, the same people rating two different objects, 
ratings by husband & wife, _etc._

_**makePaired()**_ has similar parameters as for the _lfast()_ function 
with the addition of a value for the desired t-statistic.

 - _**n**_ sample size
 
 - _**means**_ a [1:2] vector of target means for two before/after measures

 - _**sds**_ a [1:2] vector of target standard deviations

 - _**t_value**_ desired paired t-statistic

 - _**lowerbound**_ lower bound (e.g. '1' for a 1-5 rating scale)

 - _**upperbound**_ upper bound (e.g. '5' for a 1-5 rating scale)

 - _**items**_ number of items in the rating scale.

 - _**precision**_ can relax the level of accuracy required, as in _lfast()_.

### makePaired() examples

```{r paired-sample_t-test_data, , echo = TRUE}
## define parameters
n <- 20
means <- c(2.5, 3.0)
sds <- c(0.75, 0.85)
lower <- 1
upper <- 5
items <- 6
t <- -2.5

## run the function
pairedDat <- makePaired(
  n = n, means = means, sds = sds,
  t_value = t,
  lowerbound = lower, upperbound = upper, items = items
)
```

###### check properties of new data

```{r makePaired_output_properties, echo=TRUE}
## test function output
str(pairedDat)

cor(pairedDat) |> round(2)

pairedMoments <- data.frame(
  mean = apply(pairedDat, MARGIN = 2, FUN = mean) |> round(3),
  sd = apply(pairedDat, MARGIN = 2, FUN = sd) |> round(3)
) |> t()

pairedMoments
```


###### run a paired-sample t-test with the new data

```{r paired-sample_t-test, echo=TRUE}
## run a paired-sample t-test

paired_t <- t.test(x = pairedDat$X1, y = pairedDat$X2, paired = TRUE)

# paired_t <- t.test(pairedDat$X1, pairedDat$X2, paired = TRUE)


paired_t
```


----

## Create a dataframe for Repeated-Measures ANOVA  

### makeRepeated()

_makeRepeated()_ Reconstructs a synthetic dataset and inter-timepoint correlation matrix from a repeated-measures ANOVA result, based on reported means, standard deviations, and an F-statistic. 

This function estimates the average correlation between repeated measures by matching the reported F-statistic, under one of three assumed correlation structures:

 - `"cs"` (*Compound Symmetry*): Compound Symmetry assumes that all repeated measures are equally correlated with each other. That is, the correlation between time 1 and time 2 is the same as between time 1 and time 3, and so on. This structure is commonly used in repeated-measures ANOVA by default. It's mathematically simple and reflects the idea that all timepoints are equally related. However, it may not be realistic for data where correlations decrease as time intervals increase (e.g., memory decay or learning effects).
 
 - `"ar1"` (*First-Order Autoregressive*): first-order autoregressive, assumes that measurements closer together in time are more highly correlated than those further apart. For example, the correlation between time 1 and time 2 is stronger than between time 1 and time 3. This pattern is often realistic in longitudinal or time-series studies where change is gradual. The correlation drops off exponentially with each time step. 
 
 - `"toeplitz"` (*Linearly Decreasing*): Toeplitz structure is a more flexible option that allows the correlation between measurements to decrease linearly as the time gap increases. Unlike AR(1), where the decline is exponential, the Toeplitz structure assumes a straight-line drop in correlation.



#### makeRepeated() usage

    makeRepeated(
      n, 
      k, 
      means, 
      sds,
      f_stat,
      df_between = k - 1,
      df_within = (n - 1) * (k - 1),
      structure = c("cs", "ar1", "toeplitz"),
      names = paste0("time_", 1:k),
      items = 1,
      lowerbound = 1, upperbound = 5,
      return_corr_only = FALSE,
      diagnostics = FALSE,
      ...
    )


#### makeRepeated() arguments

  -  _**n**_ Integer. Sample size used in the original study.
  -  _**k**_ Integer. Number of repeated measures (timepoints).
  -  _**means**_ Numeric vector of length `k`. 
  Mean values reported for each timepoint.
  -  _**sds**_ Numeric vector of length `k`. 
  Standard deviations reported for each timepoint.
  -  _**f_stat**_ Numeric. The reported repeated-measures ANOVA F-statistic for the within-subjects factor.
  -  _**df_between**_, Degrees of freedom between conditions (default: `k - 1`).
  -  _**df_within**_, Degrees of freedom within-subjects 
  (default: `(n - 1) * (k - 1)`).
  -  _**structure**_ Character. Correlation structure to assume: `"cs"`, `"ar1"`, or `"toeplitz"` (default).
  -  _**names**_ Character vector of length `k`. Variable names for each timepoint (default: `"time_1"` to `"time_k"`).
  -  _**items**_ Integer. Number of items used to generate each scale score (passed to `link{lfast}`). (default: `1`).
  -  _**lowerbound**_, Integer. Lower bounds for Likert-type response scales (default: `1`).
  -  _**upperbound**_, Integer. upper bounds for Likert-type response scales (default: `5`).
  -  _**return_corr_only**_ Logical. If `TRUE`, return only the estimated correlation matrix.
  -  _**diagnostics**_ Logical. If `TRUE`, include diagnostic summaries such as feasible F-statistic range and effect sizes.


#### makeRepeated() examples

```{r  makeRepeated-test, echo=TRUE}
out1 <- makeRepeated(
  n = 128,
  k = 3,
  means = c(3.1, 3.5, 3.9),
  sds = c(1.0, 1.1, 1.0),
  items = 4,
  f_stat = 4.87,
  structure = "cs",
  diagnostics = FALSE
)

head(out1$data)
out1$correlation_matrix


out2 <- makeRepeated(
  n = 32, k = 4,
  means = c(2.75, 3.5, 4.0, 4.4),
  sds = c(0.8, 1.0, 1.2, 1.0),
  f_stat = 16,
  structure = "ar1",
  items = 5,
  lowerbound = 1, upperbound = 7,
  return_corr_only = FALSE,
  diagnostics = TRUE
)

print(out2)


out3 <- makeRepeated(
  n = 32, k = 4,
  means = c(2.0, 2.5, 3.0, 2.8),
  sds = c(0.8, 0.9, 1.0, 0.9),
  items = 4,
  f_stat = 24,
  structure = "toeplitz",
  diagnostics = TRUE
)

str(out3)
```


----


## Generate rating-scale data from multiple regression results    

### makeScalesRegression()

Generates synthetic rating-scale data that replicates reported regression
results: standardised betas, `R^2`, and correlation matrix of 
independent variables (if available).  

#### makeScalesRegression() usage

    makeScalesRegression <- (
       n, 
       beta_std, 
       r_squared, 
       iv_cormatrix = NULL, 
       iv_cor_mean = 0.3,  
       iv_cor_variance = 0.01, 
       iv_cor_range = c(-0.7, 0.7), 
       iv_means, 
       iv_sds,  
       dv_mean,  
       dv_sd, 
       lowerbound_iv,  
       upperbound_iv,  
       lowerbound_dv, 
       upperbound_dv,  
       items_iv = 1,  
       items_dv = 1,  
       var_names = NULL,  
       tolerance = 0.005 
    )

#### makeScalesRegression() arguments

  -  _**n**_ sample size.
  -  _**beta_std**_  a vector of length `k` 
   (number of independent variables) of standardised betas.
  -  _**r_squared**_ model `R^2`
  -  _**iv_cormatrix**_ independent variables correlation matrix. 
   Default= `NULL`
  -  _**iv_cor_mean**_ if no iv_cormatrix, average IV correlations. 
   Default = `0.3`
  -  _**iv_cor_variance**_ if no iv_cormatrix, variation in iv_cormatrix. 
   Default = `0.01`
  -  _**iv_cor_range**_ if no iv_cormatrix, range in iv_cormatrix. 
   Default = `c(-0.7, 0.7)`
  -  _**iv_means**_ a vector of length `k` of IV mean values
  -  _**iv_sds**_  a vector of length `k` of IV standard deviations
  -  _**dv_mean**_  mean of Dependent Variable (DV)
  -  _**dv_sd**_  standard deviation of DV
  -  _**lowerbound_iv**_  a vector of length `k` of lowerbounds for IV's
  -  _**upperbound_iv**_  a vector of length `k` of upperbounds for IV's
  -  _**lowerbound_dv**_  lowerbound for DV
  -  _**upperbound_dv**_  upperbound for DV
  -  _**items_iv**_ a vector of length `k` of number of items in the IV's. 
  Default = `1`.
  -  _**items_dv**_ number of items in DV. Default = `1`.
  -  _**var_names**_ a vector of variable names 
   (Independent Variables first then the Dependent Variable). Default = `NULL`
  -  _**tolerance**_ close to target R-squared. Default = `0.005`


#### makeScalesRegression() examples

##### Example 1: With provided IV correlation matrix
    set.seed(123)
    iv_corr <- matrix(c(1.0, 0.3, 0.3, 1.0), nrow = 2)
    
    result1 <- makeScalesRegression(
      n = 64,
      beta_std = c(0.4, 0.3),
      r_squared = 0.35,
      iv_cormatrix = iv_corr,
      iv_means = c(3.0, 3.5),
      iv_sds = c(1.0, 0.9),
      dv_mean = 3.8,
      dv_sd = 1.1,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5,
      items_iv = 4,
      items_dv = 4,
      var_names = c("Attitude", "Intention", "Behaviour")
    )
    
    print(result1)
    head(result1$data)
    
##### Example 2: With optimisation (no IV correlation matrix)

    set.seed(456)
    result2 <- makeScalesRegression(
      n = 64,
      beta_std = c(0.3, 0.25, 0.2),
      r_squared = 0.40,
      iv_cormatrix = NULL, # Will be optimised
      iv_cor_mean = 0.3,
      iv_cor_variance = 0.02,
      iv_means = c(3.0, 3.2, 2.8),
      iv_sds = c(1.0, 0.9, 1.1),
      dv_mean = 3.5,
      dv_sd = 1.0,
      lowerbound_iv = 1,
      upperbound_iv = 5,
      lowerbound_dv = 1,
      upperbound_dv = 5,
      items_iv = 4,
      items_dv = 5
    )
    
    # View optimised correlation matrix
    print(result2$target_stats$iv_cormatrix)
    print(result2$optimisation_info)


----

## Create a multidimensional dataframe of correlated scale items 

### correlateScales()

Correlated rating-scale items generally are summed or averaged to create a measure of an "unobservable", or "latent", construct.

_**correlateScales()**_ takes several such dataframes of rating-scale
items and rearranges their rows so that the scales are correlated according
to a predefined correlation matrix. Univariate statistics for each
dataframe of rating-scale items do not change, but their correlations with
rating-scale items in other dataframes do.

To run _**correlateScales()**_, parameters are:

 - _**dataframes**_: a list of `k` dataframes to be rearranged and combined 

 - _**scalecors**_: target correlation matrix - should be a symmetric k*k 
  positive-semi-definite matrix, where `k` is the number of dataframes

As with other functions in _LikertMakeR_, _correlateScales()_ focuses on item and scale moments (mean and standard deviation) rather than on covariance structure. 
If you wish to simulate data for teaching or experimenting with Structural Equation modelling, then I recommend the _sim.item()_ and _sim.congeneric()_ functions from the [psych package](https://CRAN.R-project.org/package=psych)

### correlateScales() examples

#### three attitudes and a behavioural intention

##### create dataframes of Likert-scale items
 
```{r correlateScales_dataframes, echo = TRUE}
n <- 128
lower <- 1
upper <- 5

### attitude #1

#### generate a correlation matrix
cor_1 <- makeCorrAlpha(items = 4, alpha = 0.80)

#### specify moments as vectors
means_1 <- c(2.5, 2.5, 3.0, 3.5)
sds_1 <- c(0.75, 0.85, 0.85, 0.75)

#### apply makeScales() function
Att_1 <- makeScales(
  n = n, means = means_1, sds = sds_1,
  lowerbound = rep(lower, 4), upperbound = rep(upper, 4),
  items = 1,
  cormatrix = cor_1
)

### attitude #2

#### generate a correlation matrix
cor_2 <- makeCorrAlpha(items = 5, alpha = 0.85)

#### specify moments as vectors
means_2 <- c(2.5, 2.5, 3.0, 3.0, 3.5)
sds_2 <- c(0.75, 0.85, 0.75, 0.85, 0.75)

#### apply makeScales() function
Att_2 <- makeScales(
  n, means_2, sds_2,
  rep(lower, 5), rep(upper, 5),
  items = 1,
  cor_2
)

### attitude #3

#### generate a correlation matrix
cor_3 <- makeCorrAlpha(items = 6, alpha = 0.90)

#### specify moments as vectors
means_3 <- c(2.5, 2.5, 3.0, 3.0, 3.5, 3.5)
sds_3 <- c(0.75, 0.85, 0.85, 1.0, 0.75, 0.85)

#### apply makeScales() function
Att_3 <- makeScales(
  n, means_3, sds_3,
  rep(lower, 6), rep(upper, 6),
  items = 1,
  cor_3
)

### behavioural intention
intent <- lfast(n, mean = 4.0, sd = 3, lowerbound = 0, upperbound = 10) |>
  data.frame()
names(intent) <- "int"
```

###### check properties of item dataframes

```{r dataframe_properties}
## Attitude #1
A1_moments <- data.frame(
  means = apply(Att_1, 2, mean) |> round(2),
  sds = apply(Att_1, 2, sd) |> round(2)
) |> t()

### Attitude #1 moments
A1_moments

### Attitude #1 correlations
cor(Att_1) |> round(2)

### Attitude #1 cronbach's alpha
alpha(cor(Att_1)) |> round(3)

## Attitude #2
A2_moments <- data.frame(
  means = apply(Att_2, 2, mean) |> round(2),
  sds = apply(Att_2, 2, sd) |> round(2)
) |> t()

### Attitude #2 moments
A2_moments

### Attitude #2 correlations
cor(Att_2) |> round(2)

### Attitude #2 cronbach's alpha
alpha(cor(Att_2)) |> round(3)

## Attitude #3
A3_moments <- data.frame(
  means = apply(Att_3, 2, mean) |> round(2),
  sds = apply(Att_3, 2, sd) |> round(2)
) |> t()

### Attitude #3 moments
A3_moments

### Attitude #3 correlations
cor(Att_3) |> round(2)

### Attitude #2 cronbach's alpha
alpha(cor(Att_3)) |> round(3)


## Behavioural Intention

intent_moments <- data.frame(
  mean = apply(intent, 2, mean) |> round(3),
  sd = apply(intent, 2, sd) |> round(3)
) |> t()

### Intention moments
intent_moments
```



##### correlateScales parameters

```{r correlateScales_parameters}
### target scale correlation matrix
scale_cors <- matrix(
  c(
    1.0, 0.7, 0.6, 0.5,
    0.7, 1.0, 0.4, 0.3,
    0.6, 0.4, 1.0, 0.2,
    0.5, 0.3, 0.2, 1.0
  ),
  nrow = 4
)

### bring dataframes into a list
data_frames <- list("A1" = Att_1, "A2" = Att_2, "A3" = Att_3, "Int" = intent)
```


#### apply the correlateScales() function

```{r my_correlated_scales}
### apply correlateScales() function
my_correlated_scales <- correlateScales(
  dataframes = data_frames,
  scalecors = scale_cors
)
```

#### plot the new correlated scale items

```{r fig6, fig.height=9, fig.width=9, fig.align='center', echo=FALSE, warning=FALSE,  crop = TRUE}
# Correlation panel
panel.cor <- function(x, y) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))
  r <- round(cor(x, y), digits = 2)
  txt <- paste0(r)
  cex.cor <- 0.8 / strwidth(txt)
  text(0.5, 0.5, txt, cex = 1.25)
}
# Customize upper panel
upper.panel <- function(x, y) {
  points(x, y, pch = 19, col = "#0000ff11")
}
# diagonals
panel.hist <- function(x, ...) {
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(usr[1:2], 0, 1.5))
  h <- hist(x, plot = FALSE)
  breaks <- h$breaks
  nB <- length(breaks)
  y <- h$counts
  y <- y / max(y)
  rect(breaks[-nB], 0, breaks[-1], y, col = "#0000ff50")
}
# Create the plots
pairs(my_correlated_scales,
  lower.panel = panel.cor,
  upper.panel = upper.panel,
  diag.panel = panel.hist
)
```


###### Check the properties of our derived dataframe

```{r newdata_check}
## data structure
str(my_correlated_scales)
```


```{r fig7, fig.height=4, fig.width=5, fig.align='center', echo = TRUE,  crop = TRUE}
## eigenvalues of dataframe correlations
Cor_Correlated_Scales <- cor(my_correlated_scales)
eigenvalues(cormatrix = Cor_Correlated_Scales, scree = TRUE) |> round(2)
```

```{r fig7a, fig.height=4, fig.width=5, fig.align='center', echo = TRUE,  crop = TRUE}
#### Eigenvalues of predictor variable items only
Cor_Attitude_items <- cor(my_correlated_scales[, -16])
eigenvalues(cormatrix = Cor_Attitude_items, scree = TRUE) |> round(2)
```



____

## Helper functions

_likertMakeR()_ includes two additional functions that may be of help 
 when examining parameters and output.

 * **_alpha()_** calculates Cronbach's Alpha from a given correlation 
 matrix or a given dataframe
 
 * **_eigenvalues()_** calculates eigenvalues of a correlation matrix, 
 a report on whether the correlation matrix is positive definite, and 
 produces an optional scree plot.
 
 * **_reliability_** presents a table of internal consistency statistics
 

### alpha()

_alpha()_ accepts, as input, either a correlation matrix or a dataframe. 
If both are submitted, then the correlation matrix is used by default, 
with a message to that effect.

### alpha() examples

```{r alphaExample}
## define parameters
df <- data.frame(
  V1 = c(4, 2, 4, 3, 2, 2, 2, 1),
  V2 = c(3, 1, 3, 4, 4, 3, 2, 3),
  V3 = c(4, 1, 3, 5, 4, 1, 4, 2),
  V4 = c(4, 3, 4, 5, 3, 3, 3, 3)
)

corMat <- matrix(
  c(
    1.00, 0.35, 0.45, 0.75,
    0.35, 1.00, 0.65, 0.55,
    0.45, 0.65, 1.00, 0.65,
    0.75, 0.55, 0.65, 1.00
  ),
  nrow = 4, ncol = 4
)

## apply function examples
alpha(cormatrix = corMat)
alpha(data = df)
alpha(NULL, df)
alpha(corMat, df)
```



### eigenvalues()

_eigenvalues()_ calculates eigenvalues of a correlation
 matrix, reports on whether the matrix is positive-definite,
 and optionally produces a scree plot.

### eigenvalues() examples


```{r eigenExample}
## define parameters
correlationMatrix <- matrix(
  c(
    1.00, 0.25, 0.35, 0.45,
    0.25, 1.00, 0.70, 0.75,
    0.35, 0.70, 1.00, 0.85,
    0.45, 0.75, 0.85, 1.00
  ),
  nrow = 4, ncol = 4
)

## apply function
evals <- eigenvalues(cormatrix = correlationMatrix)

print(evals)
```

##### eigenvalues() function with optional scree plot

```{r fig8, fig.height=4, fig.width=5, fig.align='center', echo = TRUE, crop = TRUE}
evals <- eigenvalues(correlationMatrix, 1)
print(evals)
```



### reliability()

_reliabiity()_ Computes internal consistency reliability estimates for a single-factor scale, including Cronbach’s alpha, McDonald’s omega (total), and optional ordinal (polychoric-based) variants and Confidence intervals.

### reliability() examples

```{r reliabilityExample, warning=FALSE, message=FALSE}
## create dataset
my_cor <- LikertMakeR::makeCorrAlpha(
  items = 4,
  alpha = 0.80
)

my_data <- LikertMakeR::makeScales(
  n = 64,
  means = c(2.75, 3.00, 3.25, 3.50),
  sds = c(1.25, 1.50, 1.30, 1.25),
  lowerbound = rep(1, 4),
  upperbound = rep(5, 4),
  cormatrix = my_cor
)

## run function
reliability(my_data)

reliability(
  my_data,
  include = c("lambda6", "polychoric")
)


## bootstrapped Confidence intervals can be slow!
reliability(
  my_data,
  include = "polychoric",
  ci = TRUE,
  n_boot = 64
)
```

____


# Alternative methods & packages

_LikertMakeR_ is intended for synthesising & correlating 
rating-scale data with means, standard deviations, and 
correlations as close as possible to predefined parameters. 
If you don't need your data to be close to exact, then other 
options may be faster or more flexible. 

Different approaches include:

- sampling from a _truncated normal_ distribution
 
- sampling with a predetermined probability distribution
 
- marginal model specification
 

### sampling from a _truncated normal_ distribution 

Data are sampled from a normal distribution, and then truncated to suit 
the rating-scale boundaries, and rounded to set discrete values as we see 
in rating scales. 

See [Heinz (2021)](https://glaswasser.github.io/simulating-correlated-likert-scale-data/) 
for an excellent and short example using the following packages:

- [truncnorm](https://cran.r-project.org/package=truncnorm)
 
- [faux](https://cran.r-project.org/package=faux) 
 
- See also the _rLikert()_ function from the excellent
 [latent2likert](https://github.com/markolalovic/latent2likert) package, 
 [Lalovic (2024)](https://latent2likert.lalovic.io/), for an 
 approach using optimal discretization and skew-normal distribution. 
 _latent2likert()_ converts continuous latent variables into ordinal 
 categories to generate Likert scale item responses.
 
### sampling with a predetermined probability distribution

- the following code will generate a vector of values with approximately 
 the given probabilities. 
 Good for simulating a single item.
 
 
```{r, eval = FALSE}
n <- 128
sample(1:5, n,
  replace = TRUE,
  prob = c(0.1, 0.2, 0.4, 0.2, 0.1)
)
```


### marginal model specification 

Marginal model specification extends the idea of a predefined probability 
distribution to multivariate and correlated dataframes.

- [SimCorrMix: Simulation of Correlated Data with Multiple Variable Types Including Continuous and Count Mixture Distributions](https://CRAN.R-project.org/package=SimCorrMix) on CRAN.
 
- [SimMultiCorrData: Simulation of Correlated Data with Multiple Variable Types]( https://CRAN.R-project.org/package=SimMultiCorrData) on CRAN.

- [lsasim: Functions to Facilitate the Simulation of Large Scale Assessment Data](https://CRAN.R-project.org/package=lsasim) on CRAN. 
See [Matta et al. (2018)](https://doi.org/10.1186/s40536-018-0068-8)

- [GenOrd:Simulation of Discrete Random Variables with Given Correlation Matrix and Marginal Distributions](https://CRAN.R-project.org/package=GenOrd) on CRAN.

- [SimCorMultRes: Simulates Correlated Multinomial Responses](https://cran.r-project.org/package=SimCorMultRes) on CRAN. 
See [Touloumis (2016)](https://doi.org/10.32614/RJ-2016-034)
 
- [covsim: VITA, IG and PLSIM Simulation for Given Covariance and Marginals](https://cran.r-project.org/package=covsim) on CRAN. 
See [Grønneberg et al. (2022)](https://www.jstatsoft.org/article/view/v102i03/)

### Factor Models: Classical Test Theory (CTT) 

The [latentFactoR](https://CRAN.R-project.org/package=latentFactoR) package is 
ideal for generating multi-factor items. <br /> 
`latentFactoR::simulate_factors()` 
generates data based on latent factor models, which in turn can be adjusted to 
continuous, polytomous, dichotomous, or mixed. Skews, cross-loadings, wording 
effects, population errors, and local dependencies can be added.  <br />
**High recommended!**



The [psych package](https://CRAN.R-project.org/package=psych) has several 
excellent functions for simulating rating-scale data based on factor loadings. <br /> 
These focus on factor and item correlations rather than item moments. <br />
**Highly recommended.**

- [**_psych::sim.item_**  Generate simulated data structures for circumplex, spherical, or simple structure](https://CRAN.R-project.org/package=psych)

- [**_psych::sim.congeneric_** Simulate a congeneric data set with or without minor factors](https://CRAN.R-project.org/package=psych)
See [Revelle (in prep)](https://personality-project.org/r/book/)




Also:

[**_simsem_**](https://CRAN.R-project.org/package=simsem) has many 
functions for simulating and testing data for application in 
Structural Equation modelling. 
See examples at [https://simsem.org/](https://simsem.org/) 


____


# References

D'Alessandro, S., H. Winzar, B. Lowe, B.J. Babin, W. Zikmund (2020). 
_Marketing Research_ 5<sup>ed</sup>, Cengage Australia.  <https://cengage.com.au/sem121/marketing-research-5th-edition-dalessandro-babin-zikmund> 

Grønneberg, S., Foldnes, N., & Marcoulides, K. M. (2022).
covsim: An R Package for Simulating Non-Normal Data for Structural Equation Models Using Copulas. _Journal of Statistical Software_, 102(1), 1–45. <doi:10.18637/jss.v102.i03>

Heinz, A. (2021), Simulating Correlated Likert-Scale Data In R: 
3 Simple Steps (blog post)
<https://glaswasser.github.io/simulating-correlated-likert-scale-data/>

Lalovic M (2024). latent2likert: Converting Latent Variables into Likert Scale Responses. R package version 1.2.2, <https://latent2likert.lalovic.io/>. 

Matta, T.H., Rutkowski, L., Rutkowski, D. & Liaw, Y.L. (2018), 
lsasim: an R package for simulating large-scale assessment data. 
_Large-scale Assessments in Education_ 6, 15. 
<doi:10.1186/s40536-018-0068-8>

Pornprasertmanit, S., Miller, P., & Schoemann, A. (2021).
simsem: R package for simulated structural equation modeling
<https://simsem.org/>

Revelle, W. (in prep) _An introduction to psychometric theory with applications in R_. Springer. (working draft available at <https://personality-project.org/r/book/> )

Touloumis, A. (2016), Simulating Correlated Binary and Multinomial Responses 
under Marginal Model Specification: The SimCorMultRes Package, 
_The R Journal_ 8:2, 79-91. 
<https://doi.org/10.32614/RJ-2016-034>

Winzar, H. (2025). LikertMakeR (V 1.4.0): Synthesise and correlate Likert 
scale and related rating-scale data with predefined first and second moments. 
CRAN: <https://CRAN.R-project.org/package=LikertMakeR>

---
title: "makeCorrLoadings validation"
author: "Hume Winzar"
date: "April 2025"
output: rmarkdown::html_document
vignette: >
  %\VignetteIndexEntry{makeCorrLoadings validation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup-packages, message=FALSE, warning=FALSE}
# Load required packages only if installed
if (requireNamespace("rosetta", quietly = TRUE)) library(rosetta)
if (requireNamespace("dplyr", quietly = TRUE)) library(dplyr)
if (requireNamespace("psych", quietly = TRUE)) library(psych)
if (requireNamespace("psychTools", quietly = TRUE)) library(psychTools)
if (requireNamespace("knitr", quietly = TRUE)) library(knitr)
if (requireNamespace("kableExtra", quietly = TRUE)) library(kableExtra)

library(LikertMakeR)
```



## LikertMakeR package

*LikertMakeR* is a package for the R statistical programming language
[@winzar2022]. 
It's purpose is to synthesise and correlate Likert-scale
and similar rating-scale data with predefined first & second moments
(mean and standard deviation), *Cronbach's Alpha*, *Factor Loadings*,
and other summary statistics.

## The makeCorrLoadings() function

The `makeCorrLoadings()` function generates a correlation matrix from
factor loadings and factor correlations as might be published in the
results of *Exploratory Factor Analysis* (**EFA**) or a *Structural
Equation Model* (**SEM**). The resulting correlation matrix then can be
applied to the `makeItems()` function to generate a synthetic data set
of rating-scale items that closely resemble the original data that
created the factor loadings summary table.

This paper tests how well the `makeCorrLoadings()` function achieves
that goal.

## Study design

A valid `makeCorrLoadings()` function should be able to produce a
correlation matrix that is identical to the original correlation matrix.
Further, subsequent treatment with `makeItems()` should produce a
dataframe that appears to come from the same population as the original
sample.

So we need an original, _True_, dataframe to test the function. 
Preferably, we should have several different original dataframes to ensure
that results are generalisable.



# Study #1: Party Panel 2015

### Original data

We use the pp15 dataset from the `rosetta` package [@rosetta]. This is a
subset of the Party Panel 2015 dataset. Party Panel is an annual
semi-panel study among Dutch nightlife patrons, where every year, the
determinants of another nightlife-related risk behavior is mapped. In
2015, determinants were measured of behaviours related to using highly
dosed ecstasy pills.

Nine items are relevant to this study. Each is a 7-point likert-style
question scored in the range *-3* to *+3*.

The questions and the item labels are presented as follows:

```{r}
#| echo: false

itemQuestions <- c(
  "Expectation that a high dose results in a longer trip",
  "Expectation that a high dose results in a more intense trip",
  "Expectation that a high dose makes you more intoxicated",
  "Expectation that a high dose provides more energy",
  "Expectation that a high dose produces more euphoria",
  "Expectation that a high dose yields more insight",
  "Expectation that a high dose strengthens your connection with others",
  "Expectation that a high dose facilitates making contact with others",
  "Expectation that a high dose improves sex"
)

itemLabels <- c(
  "long",
  "intensity",
  "intoxicated",
  "energy",
  "euphoria",
  "insight",
  "connection",
  "contact",
  "sex"
)

labels <- data.frame(
  Questions = itemQuestions,
  Labels = itemLabels
)

kable(labels) |>
  kable_classic(full_width = F)
```

#### Extract and clean the original data file

```{r}
## variable names
item_list <- c(
  "highDose_AttBeliefs_long",
  "highDose_AttBeliefs_intensity",
  "highDose_AttBeliefs_intoxicated",
  "highDose_AttBeliefs_energy",
  "highDose_AttBeliefs_euphoria",
  "highDose_AttBeliefs_insight",
  "highDose_AttBeliefs_connection",
  "highDose_AttBeliefs_contact",
  "highDose_AttBeliefs_sex"
)

## read the data/ select desired variables/ remove obs with missing values
dat <- read.csv2(file = "data/pp15.csv") |>
  select(all_of(item_list)) |>
  na.omit()

## give variables shorter names
names(dat) <- itemLabels

sampleSize <- nrow(dat)
```

### Target correlation matrix

The correlations among these nine items should be reproducable by the
`makeCorrLoadings()` function.

```{r}
## correlation matrix
pp15_cor <- cor(dat)
```

```{r}
#| echo: false

kable(pp15_cor, digits = 2) |>
  kable_classic(full_width = F)
```

#### Test cases

We shall produce the following options for factor correlation reporting:

1.  ***Full information***: factor loadings and factor correlations to
    five decimal places, plus uniquenesses.

2.  ***Full information - No uniquenesses***: factor loadings and factor
    correlations to 5 decimal places but without uniquenesses.

3.  ***Rounded loadings***: factor loadings and factor correlations to
    two decimal places, plus uniquenesses.

4.  ***Rounded loadings - No uniquenesses***: factor loadings and factor
    correlations to two decimal places but without uniquenesses.

5.  ***Censored loadings***: factor loadings to two decimal places, with
    loadings less than some arbitrary value removed for clarity of
    presentation, with uniquenesses.

6.  ***Censored loadings - No uniqueness***: factor loadings to two
    decimal places, with loadings less some arbitrary value removed for
    clarity, but without uniquenesses.

7.  ***Censored loadings, no uniqueness, no factor correlations***:
    factor loadings to two decimal places, with loadings less than some
    arbitrary value removed for clarity, no uniquenesses or factor
    correlations. Functionally, this is the equivalent of claiming
    orthogonal factors even with factor loadings from non-orthogonal
    rotation.

#### Evaluation

To compare the *True* correlation matrix with a *Synthetic* matrix, we
employ the `cortest.jennrich()` function from the `psych` package
[@psych]. This is a *Chi-square* test of whether a pair of matrices are
equal [@Jennrich1970]. We report the raw $\chi^2$ statistic and
corresponding p-value for each test.

### Exploratory Factor Analysis

Some pretesting suggest that two factors are appropriate for this
sample. And we're confident that the factors will be correlated, so we
use promax rotation.

```{r}
## factor analysis from `rosetta` package
rfaDose <- rosetta::factorAnalysis(
  data = dat,
  nfactors = 2,
  rotate = "promax"
)

factorLoadings <- rfaDose$output$loadings
factorCorrs <- rfaDose$output$correlations
```

#### Factor Loadings

```{r}
#| echo: false

kable(factorLoadings, digits = 2) |>
  kable_classic(full_width = F)
```

#### Factor Correlations

```{r}
#| echo: false

kable(factorCorrs, digits = 2) |>
  kable_classic(full_width = F)
```

### Test Case #1: Full information

```{r}
## round input values to 5 decimal places
# factor loadings
fl1 <- factorLoadings[, 1:2] |>
  round(5) |>
  as.matrix()
# item uniquenesses
un1 <- factorLoadings[, 3] |> round(5)
# factor correlations
fc1 <- round(factorCorrs, 5) |> as.matrix()
# run makeCorrLoadings() function
itemCors_1 <- makeCorrLoadings(
  loadings = fl1,
  factorCor = fc1,
  uniquenesses = un1
)
## Compare the two matrices
chiSq_1 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_1,
  n1 = sampleSize, n2 = sampleSize
)
```

### Test case #2: Full information - No uniquenesses

factor loadings and factor correlations to 5 decimal places but without
uniquenesses

```{r}
## round input values to 2 decimal places
# factor loadings
fl2 <- factorLoadings[, 1:2] |>
  round(5) |>
  as.matrix()
# factor correlations
fc2 <- factorCorrs |>
  round(5) |>
  as.matrix()
itemCors_2 <- makeCorrLoadings(
  loadings = fl2,
  factorCor = fc2,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_2 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_2,
  n1 = sampleSize, n2 = sampleSize
)
```

### Test case #3: Rounded loadings

factor loadings and factor correlations to two decimal places.

```{r}
## round input values to 2 decimal places
# factor loadings
fl3 <- factorLoadings[, 1:2] |>
  round(2) |>
  as.matrix()
# item uniquenesses
un3 <- factorLoadings[, 3] |>
  round(2)
## factor correlations
fc3 <- factorCorrs |>
  round(2) |>
  as.matrix()
## Compare the two matrices
itemCors_3 <- makeCorrLoadings(
  loadings = fl3,
  factorCor = fc3,
  uniquenesses = un3
)
## Compare the two matrices
chiSq_3 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_3,
  n1 = sampleSize, n2 = sampleSize
)
```

### Test case #4: Rounded loadings - No uniquenesses

Factor loadings and factor correlations to two decimal places, and no
uniquenesses

```{r}
## round input values to 2 decimal places
# factor loadings
fl4 <- factorLoadings[, 1:2] |>
  round(2) |>
  as.matrix()
## factor correlations
fc4 <- factorCorrs |>
  round(2) |>
  as.matrix()
# apply the function
itemCors_4 <- makeCorrLoadings(
  loadings = fl4,
  factorCor = fc4,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_4 <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_4,
  n1 = sampleSize, n2 = sampleSize
)
```

### Censored loadings

Often, item-factor loadings are presented with lower values removed for
ease of reading. I'm calling these "Censored" loadings. This is usually
acceptable, because the purpose is to show the reader where the larger
loadings are. But such missing information may affect the results of
reverse-engineering that we're employing with the `makeCorrLoadings()`
function.

In this study, we set the level of hidden loadings at values less than
'0.1', '0.2' and '0.3'.

```{r}
#| echo: false

fl_0 <- factorLoadings[, 1:2] |>
  round(2)
fl_a <- fl_0
# convert factor loadings < '0.1' to '0'
censor_a <- 0.1
fl_a[abs(fl_a) < censor_a] <- 0
fl_a_mean <- mean(fl_a == 0, na.rm = TRUE) |> round(2)
fl_a[fl_a == 0] <- " "
fl_b <- fl_0
# convert factor loadings < '0.2' to '0'
censor_b <- 0.2
fl_b[abs(fl_b) < censor_b] <- 0
fl_b_mean <- mean(fl_b == 0, na.rm = TRUE) |> round(2)
fl_b[fl_b == 0] <- " "
fl_c <- fl_0
# convert factor loadings < '0.3' to '0'
censor_c <- 0.3
fl_c[abs(fl_c) < censor_c] <- 0
fl_c_mean <- mean(fl_c == 0, na.rm = TRUE) |> round(2)
fl_c[fl_c == 0] <- " "
#  bring factor loadings together
fl <- cbind(fl_0, fl_a, fl_b, fl_c)
colnames(fl) <- c("f1", "f2", "f1", "f2", "f1", "f2", "f1", "f2")
header_text <- c("Item" = 1, "all values" = 2, "< 0.1 out" = 2, "< 0.2 out" = 2, "<0.3 out" = 2)
# print summary factor loadings
kable(fl, digits = 2, align = rep("c", 8)) |>
  column_spec(1:9, border_left = T, border_right = T) |>
  kable_styling() |>
  add_header_above(
    header = header_text,
    align = "c"
  )
```

### Test case #5: Censored loadings

Factor loadings less than '0.1', '0.2', '0.3' are removed for clarity,
presented to two decimal places.

```{r}
## round input values to 2 decimal places
# factor loadings
fl5a <- factorLoadings[, 1:2] |>
  round(2)
# convert factor loadings < '0.1' to '0'
fl5a[abs(fl5a) < 0.1] <- 0
fl5a <- as.matrix(fl5a)
# item uniquenesses
un5 <- factorLoadings[, 3] |>
  round(2)
# factor correlations
fc5 <- factorCorrs |>
  round(2) |>
  as.matrix()
# apply the function
itemCors_5a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5a <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_5a,
  n1 = sampleSize, n2 = sampleSize
)

# factor loadings
fl5b <- factorLoadings[, 1:2] |>
  round(2)
# convert factor loadings < '0.2' to '0'
fl5b[abs(fl5b) < 0.2] <- 0
fl5b <- as.matrix(fl5b)
# apply the function
itemCors_5b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5b <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_5b,
  n1 = sampleSize, n2 = sampleSize
)

# factor loadings
fl5c <- factorLoadings[, 1:2] |>
  round(2)
# convert factor loadings < '0.2' to '0'
fl5c[abs(fl5c) < 0.3] <- 0
fl5c <- as.matrix(fl5c)
# apply the function
itemCors_5c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5c <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_5c,
  n1 = sampleSize, n2 = sampleSize
)
# kable(itemCors_5, digits = 2)
```

### Test case #6: Censored loadings - no uniquenesses

With no declared uniquenesses, they are inferred from estimated
communalities.

```{r}
itemCors_6a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6a <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_6a,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_6b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6b <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_6b,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_6c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6c <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_6c,
  n1 = sampleSize, n2 = sampleSize
)
```

### Test case #7 Censored loadings, no uniqueness, no factor correlations

No uniquenesses. That is, Uniquenesses are estimated from
1-communalities, where communalities = sum(factor-loadings\^2). And no
factor correlations. That is, we assume orthogonal factors.

```{r}
itemCors_7a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7a <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_7a,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_7b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7b <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_7b,
  n1 = sampleSize, n2 = sampleSize
)
# apply the function
itemCors_7c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7c <- cortest.jennrich(
  R1 = pp15_cor, R2 = itemCors_7c,
  n1 = sampleSize, n2 = sampleSize
)
```

## Summary results

```{r}
#| echo: false

cases <- c(
  "Full information",
  "Full information - No uniqueness",
  "Rounded loadings",
  "Rounded loadings - No uniqueness",
  "Censored loadings <0.1",
  "Censored loadings <0.2",
  "Censored loadings <0.3",
  "Censored loadings <0.1 - no uniqueness",
  "Censored loadings <0.2 - no uniqueness",
  "Censored loadings <0.3 - no uniqueness",
  "Censored loadings <0.1 - no uniqueness, factor cors",
  "Censored loadings <0.2 - no uniqueness, factor cors",
  "Censored loadings <0.3 - no uniqueness, factor cors"
)

chi2 <- c(
  chiSq_1$chi2,
  chiSq_2$chi2,
  chiSq_3$chi2,
  chiSq_4$chi2,
  chiSq_5a$chi2,
  chiSq_5b$chi2,
  chiSq_5c$chi2,
  chiSq_6a$chi2,
  chiSq_6b$chi2,
  chiSq_6c$chi2,
  chiSq_7a$chi2,
  chiSq_7b$chi2,
  chiSq_7c$chi2
) |> round(2)

p <- c(
  chiSq_1$prob,
  chiSq_2$prob,
  chiSq_3$prob,
  chiSq_4$prob,
  chiSq_5a$prob,
  chiSq_5b$prob,
  chiSq_5c$prob,
  chiSq_6a$prob,
  chiSq_6b$prob,
  chiSq_6c$prob,
  chiSq_7a$prob,
  chiSq_7b$prob,
  chiSq_7c$prob
) |> round(3)

summary_results_1 <- data.frame(
  Treatment = cases,
  chi2 = chi2,
  p = p
)

kable(summary_results_1) |>
  kable_classic(full_width = F)
```

## Conclusion

The `makeCorrLoadings` function works quite well when it has information
on factor loadings, but much less well when "summary" (censored) factor
loadings are given.

------------------------------------------------------------------------

# Study #2: Big Five (bfi personality items)

### Original data: bfi 25 Personality items representing 5 factors

25 personality self report items taken from the [International
Personality Item Pool (ipip.ori.org)](https://ipip.ori.org) were
included as part of the *Synthetic Aperture Personality Assessment*
*(SAPA)* web based personality assessment project. The data are
available through the `psychTools` package [@psychTools].

2800 subjects are included as a demonstration set for scale
construction, factor analysis, and Item Response Theory analysis. Three
additional demographic variables (sex, education, and age) are also
included.

For purposes of this study, we confine ourselves to women (sex==2) with
postgraduate qualifications (education==5), giving us a sample size of
229 subjects.

The dataset contains the following 28 variables. (The q numbers are the
SAPA item numbers).

-   A1 Am indifferent to the feelings of others. (q_146)
-   A2 Inquire about others’ well-being. (q_1162)
-   A3 Know how to comfort others. (q_1206)
-   A4 Love children. (q_1364)
-   A5 Make people feel at ease. (q_1419)
-   C1 Am exacting in my work. (q_124)
-   C2 Continue until everything is perfect. (q_530)
-   C3 Do things according to a plan. (q_619)
-   C4 Do things in a half-way manner. (q_626)
-   C5 Waste my time. (q_1949)
-   E1 Don’t talk a lot. (q_712)
-   E2 Find it difficult to approach others. (q_901)
-   E3 Know how to captivate people. (q_1205)
-   E4 Make friends easily. (q_1410)
-   E5 Take charge. (q_1768)
-   N1 Get angry easily. (q_952)
-   N2 Get irritated easily. (q_974)
-   N3 Have frequent mood swings. (q_1099
-   N4 Often feel blue. (q_1479)
-   N5 Panic easily. (q_1505)
-   O1 Am full of ideas. (q_128)
-   O2 Avoid difficult reading material.(q_316)
-   O3 Carry the conversation to a higher level. (q_492)
-   O4 Spend time reflecting on things. (q_1738)
-   O5 Will not probe deeply into a subject. (q_1964)
-   gender (Male=1, Female=2)
-   education (1=HS, 2=finished_HS, 3=some_college, 4=college_graduate
    5=graduate_degree)
-   age age in years

### Second target correlation matrix

The correlations among these 25 items should be reproducable by the
`makeCorrLoadings()` function.

```{r}
## download data
data(bfi)
## filter for highly-educated women
bfi_short <- bfi |>
  filter(education == 5 & gender == 2) |>
  na.omit()
## keep just the 25 items
bfi_short <- bfi_short[, 1:25]
sampleSize <- nrow(bfi_short)
## derive correlation matrix
bfi_cor <- cor(bfi_short)
```

#### Test cases and Evaluation

As in the first study, we shall produce the following options for factor
correlation reporting:

1.  Full information
2.  Full information - No uniquenesses
3.  Rounded loadings
4.  Rounded loadings - No uniquenesses
5.  Censored loadings
6.  Censored loadings - No uniqueness
7.  Censored loadings - no uniqueness or factor cors

And, as in the first study, we compare the *True* correlation matrix
with a *Synthetic* matrix, employing a *Jennrich test*.

### Exploratory Factor Analysis

Five correlated factors are appropriate for this sample, so we use
promax rotation.

```{r}
## factor analysis from `rosetta` package is a less messy version of the `psych::fa()` function

fa_bfi <- rosetta::factorAnalysis(
  data = bfi_short,
  nfactors = 5,
  rotate = "promax"
)

bfiLoadings <- fa_bfi$output$loadings
bfiCorrs <- fa_bfi$output$correlations
```

#### item-factor loadings & uniquenesses

```{r}
#| echo: false

kable(bfiLoadings, digits = 2) |>
  kable_classic(full_width = F)
```

#### factor correlations

```{r}
#| echo: false

kable(bfiCorrs, digits = 2) |>
  kable_classic(full_width = F)
```

### Test Case #1: Full information

```{r}
## round input values to 5 decimal places
# factor loadings
fl1 <- bfiLoadings[, 1:5] |>
  round(5) |>
  as.matrix()
# item uniquenesses
un1 <- bfiLoadings[, 6] |> round(5)
# factor correlations
fc1 <- round(bfiCorrs, 5) |> as.matrix()
# run makeCorrLoadings() function
itemCors_1 <- makeCorrLoadings(
  loadings = fl1,
  factorCor = fc1,
  uniquenesses = un1
)
## Compare the two matrices
chiSq_1 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_1,
  n1 = sampleSize, n2 = sampleSize
)
```

### Other test cases as for the first study

The remaining test cases are as for the first study and this last test
case. Changes are made to the number of decimal places considered, the
presence of uniquenesses, presence of factor correlation matrix, and
level of item-factor loading that is included.

<!-- ### Test case #2: Full information - No uniquenesses  -->

<!-- factor loadings and factor correlations to 5 decimal places but without uniquenesses -->

```{r}
#| echo: false

## round input values to 5 decimal places
# factor loadings
fl2 <- bfiLoadings[, 1:5] |>
  round(5) |>
  as.matrix()
# factor correlations
fc2 <- bfiCorrs |>
  round(5) |>
  as.matrix()
# run makeCorrLoadings() function
itemCors_2 <- makeCorrLoadings(
  loadings = fl2,
  factorCor = fc2,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_2 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_2,
  n1 = sampleSize, n2 = sampleSize
)
```

<!-- ### Test case #3: Rounded loadings -->

<!-- factor loadings and factor correlations to two decimal places. -->

```{r}
#| echo: false

## round input values to 2 decimal places
# factor loadings
fl3 <- bfiLoadings[, 1:5] |>
  round(2) |>
  as.matrix()

# item uniquenesses
un3 <- bfiLoadings[, 6] |>
  round(2)
## factor correlations
fc3 <- bfiCorrs |>
  round(2) |>
  as.matrix()
# run makeCorrLoadings() function
itemCors_3 <- makeCorrLoadings(
  loadings = fl3,
  factorCor = fc3,
  uniquenesses = un3
)

## Compare the two matrices
chiSq_3 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_3,
  n1 = sampleSize, n2 = sampleSize
)
```

<!-- ### Test case #4: Rounded loadings - No uniquenesses -->

<!-- Factor loadings and factor correlations to two decimal places, and no uniquenesses -->

```{r}
#| echo: false

## round input values to 2 decimal places
# factor loadings
fl4 <- bfiLoadings[, 1:5] |>
  round(2) |>
  as.matrix()
## factor correlations
fc4 <- bfiCorrs |>
  round(2) |>
  as.matrix()
# run makeCorrLoadings() function
itemCors_4 <- makeCorrLoadings(
  loadings = fl4,
  factorCor = fc4,
  uniquenesses = NULL
)

## Compare the two matrices
chiSq_4 <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_4,
  n1 = sampleSize, n2 = sampleSize
)
```

<!-- ### Test case #5: Censored loadings -->

<!-- Factor loadings less than '0.3' are removed for clarity, presented to two decimal places. Uniquenesses are inferred from estimated communalities. -->

```{r}
#| echo: false

## round input values to 2 decimal places
# factor loadings
fl5a <- bfiLoadings[, 1:5] |>
  round(2)
# convert factor loadings < '0.1' to '0'
fl5a[abs(fl5a) < 0.1] <- 0
fl5a <- as.matrix(fl5a)
# factor correlations
fc5 <- bfiCorrs |>
  round(2) |>
  as.matrix()
# item uniquenesses
un5 <- bfiLoadings[, 6] |>
  round(2)
# run makeCorrLoadings() function
itemCors_5a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5a <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_5a,
  n1 = sampleSize, n2 = sampleSize
)
# factor loadings
fl5b <- bfiLoadings[, 1:5] |>
  round(2)
# convert factor loadings < '0.2' to '0'
fl5b[abs(fl5b) < 0.2] <- 0
fl5b <- as.matrix(fl5b)
# run makeCorrLoadings() function
itemCors_5b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5b <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_5b,
  n1 = sampleSize, n2 = sampleSize
)
# factor loadings
fl5c <- bfiLoadings[, 1:5] |>
  round(2)
# convert factor loadings < '0.3' to '0'
fl5c[abs(fl5c) < 0.3] <- 0
fl5c <- as.matrix(fl5c)
# run makeCorrLoadings() function
itemCors_5c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = un5
)
## Compare the two matrices
chiSq_5c <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_5c,
  n1 = sampleSize, n2 = sampleSize
)
```

<!-- ### Test case #6: Censored loadings - No uniquenesses -->

<!-- Factor loadings less than '0.1', '0.2', '0.3' are removed for clarity, presented to two decimal places. -->

```{r}
#| echo: false

## Loadings and factor correlations are the same, so we only need to change
## parameters of the makeCorrLoadings() application.

# run makeCorrLoadings() function
itemCors_6a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6a <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_6a,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_6b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6b <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_6b,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_6c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = fc5,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_6c <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_6c,
  n1 = sampleSize, n2 = sampleSize
)
```

<!-- ### Test case #7 Censored loadings, no uniqueness, no factor correlations -->

<!-- Factor loadings less than '0.2' are removed for clarity, but no uniquenesses. That is, Uniquenesses are estimated from 1-communalities, where communalities = sum(factor-loadings\^2). -->

```{r}
#| echo: false

## Loadings and factor correlations are the same, so we only need to change
## parameters of the makeCorrLoadings() application.

# run makeCorrLoadings() function
itemCors_7a <- makeCorrLoadings(
  loadings = fl5a,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7a <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_7a,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_7b <- makeCorrLoadings(
  loadings = fl5b,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7b <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_7b,
  n1 = sampleSize, n2 = sampleSize
)
# run makeCorrLoadings() function
itemCors_7c <- makeCorrLoadings(
  loadings = fl5c,
  factorCor = NULL,
  uniquenesses = NULL
)
## Compare the two matrices
chiSq_7c <- cortest.jennrich(
  R1 = bfi_cor, R2 = itemCors_7c,
  n1 = sampleSize, n2 = sampleSize
)
```

## Study #2 Summary results

```{r}
#| echo: false

cases <- c(
  "Full information",
  "Full information - No uniquenesses",
  "Rounded loadings",
  "Rounded loadings - No uniquenesses",
  "Censored loadings <0.1",
  "Censored loadings <0.2",
  "Censored loadings <0.3",
  "Censored loadings <0.1 - no uniqueness",
  "Censored loadings <0.2 - no uniqueness",
  "Censored loadings <0.3 - no uniqueness",
  "Censored loadings <0.1 - no uniqueness, no factor cors",
  "Censored loadings <0.2 - no uniqueness, no factor cors",
  "Censored loadings <0.3 - no uniqueness, no factor cors"
)

chi2 <- c(
  chiSq_1$chi2,
  chiSq_2$chi2,
  chiSq_3$chi2,
  chiSq_4$chi2,
  chiSq_5a$chi2,
  chiSq_5b$chi2,
  chiSq_5c$chi2,
  chiSq_6a$chi2,
  chiSq_6b$chi2,
  chiSq_6c$chi2,
  chiSq_7a$chi2,
  chiSq_7b$chi2,
  chiSq_7c$chi2
) |> round(2)

p <- c(
  chiSq_1$prob,
  chiSq_2$prob,
  chiSq_3$prob,
  chiSq_4$prob,
  chiSq_5a$prob,
  chiSq_5b$prob,
  chiSq_5c$prob,
  chiSq_6a$prob,
  chiSq_6b$prob,
  chiSq_6c$prob,
  chiSq_7a$prob,
  chiSq_7b$prob,
  chiSq_7c$prob
) |> round(3)

summary_results_2 <- data.frame(
  Treatment = cases,
  chi2 = chi2,
  p = p
)

# summary_results_2
kable(summary_results_2, digits = c(0, 1, 5)) |>
  kable_classic(full_width = F)
```

# Overall Results

Both studies show consistent results.

```{r}
#| echo: false

overall_summary <- data.frame(
  treatment = cases,
  chi2.1 = summary_results_1[, 2],
  p.1 = summary_results_1[, 3],
  chi2.2 = summary_results_2[, 2],
  p.2 = summary_results_2[, 3]
)

names(overall_summary) <- c("treatment", "chi2", "p", "chi2", "p")

kable(overall_summary, digits = c(0, 1, 3, 1, 3)) |>
  column_spec(4, border_left = T) |>
  kable_classic(full_width = F) |>
  add_header_above(c(" " = 1, "Party panel" = 2, "Big 5 (bfi)" = 2))
```

## Conclusions

The `makeCorrLoadings()` function is designed to produce an item
correlation matrix based on item-factor loadings and factor correlations
as one might see in the results of Exploratory Factor Analysis (EFA) or
Structural Equation Modelling (SEM).

Results of this study suggest that the `makeCorrLoadings()` function
does a surprisingly good job of reproducing the target correlation
matrix when all item-factor loadings are present.

A correlation matrix created with `makeCorrLoadings()` seems to be
robust even with the absence of specified *uniquenesses*, or even
without *factor correlations*. But a valid reproduction of a correlation
matrix should have complete item-factor loadings.

------------------------------------------------------------------------

## References

---
title: "likertMakeR::reliability()"
author: "Hume Winzar"
date: "December 2025"
output: rmarkdown::html_vignette
bibliography: references_2.bib  
link-citations: true
vignette: >
  %\VignetteIndexEntry{likertMakeR::reliability()}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


```{r}
#| label: load_packages
#| echo: false
#| warning: false
#| message: false

# library(dplyr)
# library(tibble)
# library(kableExtra)
# library(tidyr)
library(LikertMakeR)
```

<!-- END setup chunks -->

## Reliability estimation with `LikertMakeR::reliability()`

The `reliability()` function estimates a range of internal consistency
reliability coefficients 
for **single-factor Likert and rating-scale measures**. 
It is designed to work naturally with synthetic data generated
by **LikertMakeR**, but applies equally to real survey data.

Unlike many reliability functions, `reliability()`:

- presents **multiple coefficients in a tidy table**,
- provides **bootstrap confidence intervals** when requested,
- supports **ordinal (polychoric-based) reliability**, and
- includes **explicit diagnostics** explaining when ordinal estimates
  are not feasible.

### When should you use `reliability()`?

Use `reliability()` when:

- your scale is intended to measure **one underlying construct**,
- items are **Likert-type or bounded rating scales**, and
- you want **transparent, reproducible reliability estimates**, especially
  for teaching, simulation, or methods work.

The function is **not intended for multidimensional scales or SEM models**;
excellent alternatives already exist for those purposes (e.g. `lavaan`,
`semTools`).

---

## Function usage

```{r}
#| label: usage
#| eval: false

reliability(
  data,
  include = "none",
  ci = FALSE,
  ci_level = 0.95,
  n_boot = 1000,
  na_method = c("pairwise", "listwise"),
  min_count = 2,
  digits = 3,
  verbose = TRUE
)
```

## Arguments

### `data`

An `n × k` data frame or matrix containing item responses,
where rows correspond to respondents and columns correspond to items.

---

### `include`

A character vector specifying which additional reliability coefficients
to compute.

Possible values are:

- `"none"` (default)  
  Computes **Cronbach’s alpha** and **McDonald’s omega (total)** using
  Pearson correlations.

- `"lambda6"`  
  Adds **Guttman’s lambda-6**, computed via `psych::alpha()`  
  (requires the optional package **psych**).
  
- `"omega_h"`  
  Adds **McDonald’s omega hierarchical ($\omega_h$)**, also known as **Coefficient H**.  
  This coefficient estimates the **maximum reliability of the general factor**
  under a single-factor model, assuming optimal weighting of items.

- `"polychoric"`  
  Adds **ordinal reliability estimates**, computed from polychoric
  correlations:
  - ordinal alpha (Zumbo’s alpha),
  - ordinal omega (total).

Multiple options may be supplied, for example:

```{r}
#| label: include_vector
#| eval: false

include <- c("lambda6", "polychoric")
```

### `ci`

Logical.  
If `TRUE`, confidence intervals are computed using a
**nonparametric bootstrap**.

Default is `FALSE`.

---

### `ci_level`

Confidence level for bootstrap intervals.  
Default is `0.95`.

---

### `n_boot`

Number of bootstrap resamples used when `ci = TRUE`.  
Default is `1000`.

Larger values reduce _Monte Carlo_ error but increase computation time,
especially for ordinal (polychoric-based) reliability estimates.

---

### `na_method`

How missing values are handled:

- `"pairwise"` (default): correlations use all available pairs,
- `"listwise"`: rows with any missing values are removed before analysis.

---

### `min_count`

Minimum observed frequency per response category required to attempt
polychoric correlations.  
Default is `2`.

Ordinal reliability estimates are skipped if any item contains categories
with fewer than `min_count` observations. When this occurs, diagnostics
are stored in the returned object and may be inspected using
`ordinal_diagnostics()`.

---

### `digits`

Number of decimal places used when printing estimates.  
Default is `3`.

---

### `verbose`

Logical.  
If `TRUE`, warnings and progress indicators are displayed.

Default is `TRUE`.

---

## Reliability coefficients returned

### Pearson-based coefficients (always available)

- **Cronbach’s alpha**  
  Computed from the Pearson correlation matrix.

- **McDonald’s omega (total)**  
  Computed from the leading eigenvalue of the correlation matrix, assuming
  a single common factor.

These estimates are appropriate when Likert-scale responses are treated
as approximately interval-scaled.

---

### Ordinal (polychoric-based) coefficients

When `include = "polychoric"`:

- **Ordinal alpha (Zumbo’s alpha)**  
  Cronbach’s alpha computed from the **polychoric correlation matrix**.

- **Ordinal omega (total)**  
  McDonald’s omega computed from the polychoric correlation matrix.

These estimates are often preferred when items are clearly ordinal,
response distributions are skewed, or floor/ceiling effects are present.

---

## Ordinal diagnostics and safeguards

Ordinal reliability estimation can fail when response categories are sparse
(e.g., very few observations in extreme categories).

When this occurs:

- ordinal estimates are **skipped rather than forced**,
- a warning is issued,
- diagnostics are stored in the returned object.

Diagnostics may be inspected using:

```{r}
#| label: ordinal_diagnostics
#| eval: false

ordinal_diagnostics(result)
```


---

### Hierarchical reliability: $\omega_h$ (Coefficient H)

When `include = "omega_h"`, `reliability()` reports 
**McDonald’s omega hierarchical** ($\omega_h$), also known as **Coefficient H**.

$\omega_h$ answers a different question from $\alpha$ or $\omega$ (total):

> *How well would the underlying latent factor be measured if the best possible
linear combination of items were used?*

Key characteristics of $\omega_h$:

- It is a **model-based upper bound** on reliability
- It reflects **factor determinacy**, not observed-score reliability
- It assumes a **single dominant common factor**
- It is insensitive to scale length but sensitive to factor structure

$\omega_h$ is therefore best interpreted as a **diagnostic index**, 
rather than as a direct estimate of the reliability of observed summed scores.

#### Why no confidence intervals for $\omega_h$?

Confidence intervals are **not reported** for $\omega_h$.

This is intentional:

- $\omega_h$ is a **maximal reliability bound**, not a descriptive statistic
- Its sampling distribution is **highly non-normal**
- Bootstrap confidence intervals are often **unstable or misleading**
- There is **no agreed inferential framework** for $\omega_h$ in the literature

Accordingly, $\omega_h$ is reported as a **point estimate only**, with 
explanatory notes in the output table.

---


## Examples

### Create a synthetic dataset

The example below generates a four-item single-factor scale with a target
Cronbach’s alpha of 0.80, using functions from **LikertMakeR**.

```{r}
#| label: dataset

# example correlation matrix
my_cor <- LikertMakeR::makeCorrAlpha(
  items = 4,
  alpha = 0.80
)

# example correlated dataframe
my_data <- LikertMakeR::makeScales(
  n = 64,
  means = c(2.75, 3.00, 3.25, 3.50),
  sds = c(1.25, 1.50, 1.30, 1.25),
  lowerbound = rep(1, 4),
  upperbound = rep(5, 4),
  cormatrix = my_cor
)
```


### Basic reliability estimates

By default, `reliability()` returns Pearson-based Cronbach’s alpha and
McDonald’s omega (total), assuming a single common factor.

```{r}
#| label: simple_function

# $\alpha$ and $\omega$

reliability(my_data)
```

### Including additional coefficients

Additional reliability coefficients may be requested using the
`include` argument.

```{r}
#| label: include_parameter

# $\alpha$, $\omega$ (total), $\lambda 6$, $\omega_h$, and ordinal variants

reliability(
  my_data,
  include = c("lambda6", "omega_h", "polychoric")
)
```

The available options are:

- `"lambda6"`  
  Adds **Guttman’s lambda-6**, computed using `psych::alpha()`.  
  This option requires the suggested package **psych**.

- `"omega_h"`  
  Adds **omega hierarchical (Coefficient H)**, a model-based upper bound on
  reliability that reflects how well the general factor is measured.
  $\omega_h$ is reported as a point estimate only and is best used as a 
  diagnostic   indicator of factor strength rather than as observed-score 
  reliability.
  
- `"polychoric"`  
  Adds **ordinal (polychoric-based) reliability estimates**, including
  ordinal alpha (Zumbo’s alpha) and ordinal omega (total).

Multiple options may be supplied simultaneously. If `"none"` is included
alongside other options, it is ignored.

If ordinal reliability estimates cannot be computed — most commonly due to
sparse response categories — they are skipped automatically. In such cases,
the returned object contains diagnostic information explaining why the
estimates were omitted.

### When should I use each option?

By default, `reliability()` reports Cronbach’s alpha and McDonald’s omega
computed from Pearson correlations. 
This is appropriate for most teaching, exploratory, and applied settings, 
especially when Likert items have five or more categories and reasonably 
symmetric distributions.

Use `include = "lambda6"` when you want an additional lower-bound reliability
estimate that is less sensitive to tau-equivalence assumptions. 
Guttman’s lambda-6 is often reported alongside alpha and omega in 
methodological comparisons and requires the **psych** package.

Use `include = "omega_h"` when you want to assess the 
**strength and clarity of the general factor** underlying a scale.
$\omega_h$ is particularly useful when evaluating whether a set of items 
meaningfully reflects a single latent construct, but it should not be 
interpreted as the reliability of summed or averaged scores.

Use `include = "polychoric"` when item responses are clearly ordinal and
category distributions are well populated. In this case, the function
computes ordinal alpha (Zumbo’s alpha) and ordinal omega based on polychoric
correlations. 
Ordinal methods are most appropriate when response categories
are few (e.g., 4–5 points) and when treating items as continuous may be
questionable. If response categories are sparse, ordinal estimates are
skipped and diagnostics are provided to explain why.


### Notes on computation

All reliability coefficients in `reliability()` are computed under the
assumption of a **single common factor**. The function is intended for
unidimensional scales and does not perform factor extraction or
dimensionality testing.

Cronbach’s alpha and McDonald’s omega are computed from 
**Pearson correlations** by default. 
When `include = "polychoric"` is specified,
ordinal reliability estimates are computed 
using **polychoric correlations**, corresponding 
to *Zumbo’s ordinal alpha* and *ordinal omega total*.

Ordinal reliability estimates may be **skipped automatically** when:

- an item has fewer than two observed response categories, or
- one or more response categories occur fewer than `min_count` times.

In these cases, the function returns `NA` for ordinal estimates and stores
diagnostic information explaining the decision. These diagnostics can be
inspected using `ordinal_diagnostics()`.

When `ci = TRUE`, confidence intervals are obtained using a **nonparametric
bootstrap**. 
For ordinal reliability estimates, bootstrap resamples may fail if polychoric 
correlations cannot be estimated in some resampled datasets. 
Such failures are tracked internally and reported in the output notes. 
Increasing `n_boot` can improve the stability of ordinal confidence
intervals when the proportion of successful bootstrap draws is high but
not complete.

For transparency, methodological details about estimation methods and
bootstrap performance are reported alongside point estimates in the
returned table.

## Choosing a Reliability Coefficient: A Practical Decision Guide

Researchers and students are often faced with multiple reliability coefficients and little guidance on when each should be used. This section provides a practical, defensible guide for choosing among Cronbach’s alpha, McDonald’s omega, and their ordinal counterparts when working with Likert-type and rating-scale data.

This guidance assumes a single-factor scale, which is the design focus of 
LikertMakeR.

### Step 1: What kind of data do you have?

#### Continuous or approximately continuous items

Examples:

  - Scale scores with many response options

  - Visual analogue scales

  - Aggregated or averaged ratings

  → Pearson correlations are usually appropriate.

#### Ordinal (Likert-type) items

Examples:

  - Single 5-point or 7-point agreement scales

  - Frequency scales with clear category boundaries

  → Ordinal (polychoric-based) methods are often more appropriate, especially when responses are skewed or unevenly distributed.

### Step 2: Choosing between $\alpha$ and $\omega$

#### Cronbach’s alpha ($\alpha$)

Cronbach’s alpha is the most widely reported reliability coefficient and is 
based on average inter-item correlations.

Use alpha when:

  - You need comparability with legacy literature

  - Items are roughly tau-equivalent 
  (all items make equal contributions to the underlying factor)

  - You want a simple baseline estimate

Limitations:

  - Assumes equal factor loadings

  - Can underestimate reliability when loadings differ

  - Sensitive to the number of items

Alpha should be viewed as a descriptive lower bound, not a definitive measure of internal consistency.

#### McDonald’s omega ($\omega$)

McDonald’s omega estimates the proportion of variance attributable to a single common factor, allowing items to have different loadings.

Use omega when:

  - Items vary in strength or discrimination

  - You want a model-based reliability estimate

  - A single factor is theoretically justified

Advantages:

  - Fewer restrictive assumptions than alpha

  - Better behaved in simulations

  - Increasingly recommended in methodological literature

As a general rule, omega is preferred to alpha for single-factor scales when factor loadings are unequal.


#### Where does Guttman’s $\lambda_6$ fit?

Guttman’s lambda-6 ($\lambda_6$) is a lower-bound estimate of reliability that 
relaxes Cronbach’s assumption of equal error variances across items.

Use $\lambda_6$ when:

- You want a reliability estimate that is:
  - more defensible than $\alpha$,
  - but does not rely on a factor model
- You are comparing multiple lower-bound estimates
- You want a conservative benchmark alongside $\omega$

Key points:

- $\lambda_6$ is always $\geqslant$ $\alpha$ for the same data
- Like $\alpha$, it is a lower bound — not an estimate of true reliability
- Unlike $\omega$, it does not assume a latent factor structure

In practice, $\lambda_6$ is most useful when reported 
**alongside $\alpha$ and $\omega$** to show
how sensitive conclusions are to different reliability assumptions.


### Step 3: When should I use ordinal reliability?

Ordinal reliability coefficients are computed from polychoric correlations, 
which estimate associations between latent continuous variables underlying 
ordinal responses.

In reliability(), these correspond to:

  - Ordinal alpha (often called _Zumbo’s alpha_)

  - Ordinal omega

Use ordinal reliability when:

  - Items are ordinal (e.g., 5- or 7-point Likert scales)

  - Response distributions are skewed or uneven

  - You wish to respect the ordinal measurement scale

Important caveats:

  - Polychoric correlations require sufficient observations per category

  - Sparse categories can cause estimation failure

  - Diagnostics should always be inspected

If ordinal estimation is not feasible, reliability() reports this transparently 
and falls back to Pearson-based estimates.

### Step 4: $\alpha$ vs $\omega$ vs ordinal $\omega$ — a practical summary

<!-- | Situation | Recommended coefficient | -->
<!-- |----------|--------------------------| -->
<!-- | Legacy comparison, simple reporting | $\alpha$ | -->
<!-- | Single-factor scale, unequal loadings | $\omega$ | -->
<!-- | Strength of general factor | $\omega_h$ | -->
<!-- | Likert items with skew or ceiling effects | Ordinal $\omega$ | -->
<!-- | Teaching or demonstration | $\alpha$ *and* $\omega$ | -->
<!-- | Ordinal data, small samples or sparse categories | $\omega$ (Pearson-based) | -->

```{r}
#| label: step4_decision_table
#| echo: FALSE
#| results: asis

library(knitr)
library(kableExtra)

decision_table <- data.frame(
  Situation = c(
    "Legacy comparison, simple reporting",
    "Single-factor scale, unequal loadings",
    "Strength of general factor",
    "Likert items with skew or ceiling effects",
    "Teaching or demonstration",
    "Ordinal data, small samples or sparse categories"
  ),
  `Recommended coefficient` = c(
    "$\\alpha$, Cronbach's alpha",
    "$\\omega$, McDonalds omega",
    "$\\omega_h$, Coefficient H",
    "Ordinal $\\omega$",
    "$\\alpha$ and $\\omega$",
    "$\\omega$ (Pearson-based)"
  )
)

kable(
  decision_table,
  format = "html",
  escape = FALSE,
  align = c("l", "l")
) |>
  column_spec(1, width = "60%") |>
  column_spec(2, width = "40%")
```

When in doubt:

    Report omega, and optionally alpha for comparison.

If your data are clearly ordinal and diagnostics permit:

    Ordinal omega is the most defensible choice.
    
    “ordinal $\omega$” refers to omega total computed from the 
    polychoric correlation matrix.

### Step 5: Confidence intervals

When `ci = TRUE`, LikertMakeR computes nonparametric bootstrap confidence intervals.

Why bootstrap?

  - No closed-form CI exists for omega

  - Ordinal reliability has no reliable analytic CI

  - Bootstrap intervals are flexible and robust

Practical advice:

  - Use at least 1,000 resamples for stable intervals

  - Expect longer runtimes for ordinal bootstraps

  - Always report the method used to compute CIs


Confidence intervals are intentionally **not provided** for $\omega_h$, 
as it represents a model-based upper bound on reliability rather than an 
inferential estimate.


## Recommended reading

 - For readers who want to go a little deeper

If you use reliability() for teaching or applied research, the following
sources provide accessible explanations of the ideas behind the coefficients
reported here.

### Understanding Cronbach’s alpha and its limitations

  - Cronbach, L. J. (1951). _Coefficient alpha and the internal structure of tests._ (@cronbach1951)  
The original source for alpha; still worth reading to understand what alpha
does—and does not—measure.

  - Revelle, W., & Zinbarg, R. E. (2009). _Coefficients alpha, beta, omega, and the glb._ (@revelle2009)  
A clear discussion of why alpha can be misleading and when omega is preferable.

### Omega and factor-based reliability

  - McDonald, R. P. (1999). _Test theory: A unified treatment._ (@mcdonald2013test)  
The definitive reference for omega; recommended for readers comfortable with
factor analysis concepts.

  - Hancock, G. R., & Mueller, R. O. (2001).
  _Rethinking construct reliability within latent variable systems._ (@hancock2001)  
  Introduces Coefficient H and discusses its interpretation as factor
  determinacy rather than observed-score reliability.

### Comparative studies

  - Xiao L, Hau KT. (2022). _Performance of Coefficient Alpha and Its Alternatives: Effects of Different Types of Non-Normality._ (@xiao2023)

### Ordinal reliability for Likert-type data

  - Zumbo, B. D., Gadermann, A. M., & Zeisser, C. (2007).
_Ordinal versions of coefficients alpha and theta for Likert rating scales._ (@zumbo2007)  
Introduces ordinal (polychoric-based) alpha—often called Zumbo’s alpha.

  - Gadermann, A. M., Guhn, M., & Zumbo, B. D. (2012).
_Estimating ordinal reliability for Likert-type and ordinal item response data._ (@gadermann2012)  
A practical, non-technical guide that is especially suitable for teaching.

### Polychoric correlations in practice

  - Holgado–Tello, F. P., et al. (2010).
_Polychoric versus Pearson correlations in factor analysis of ordinal variables._ (@holgado2010)  
A helpful applied comparison explaining why Pearson correlations can distort
analyses of Likert-type data.

## Teaching tip

For most classroom examples, start with Pearson-based alpha and omega.
Introduce ordinal reliability only after students understand:

(a) factor models, and
(b) why Likert responses are not truly continuous.

This mirrors the progressive structure used in reliability() and helps
students see why additional assumptions are required for ordinal methods.

## Citations

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeItemsScale.R
\name{makeItemsScale}
\alias{makeItemsScale}
\title{Generate scale items from a summated scale, with desired Cronbach's Alpha}
\usage{
makeItemsScale(
  scale,
  lowerbound,
  upperbound,
  items,
  alpha = 0.8,
  summated = TRUE,
  progress = TRUE
)
}
\arguments{
\item{scale}{(int) a vector or dataframe of the summated rating scale.
Should range from \eqn{lowerbound \times items} to
\eqn{upperbound \times items}}

\item{lowerbound}{(int) lower bound of the scale item
(example: '1' in a '1' to '5' rating)}

\item{upperbound}{(int) upper bound of the scale item
(example: '5' in a '1' to '5' rating)}

\item{items}{(positive, int) k, or number of columns to generate}

\item{alpha}{(posiitve, real) desired \emph{Cronbach's Alpha} for the
new dataframe of items.
Default = '0.8'.}

\item{summated}{logical. If TRUE (default), \code{scale} is assumed to be a
summated scale. If FALSE, the input is treated as a mean scale and will
be multiplied by \code{items} internally.}

\item{progress}{logical. If TRUE (default is \code{interactive()}), a progress
bar is displayed during optimisation.
Set to FALSE for slightly faster execution.}
}
\value{
a dataframe with 'items' columns and 'length(scale)' rows
}
\description{
\code{makeItemsScale()} generates a random dataframe
of scale items based on a predefined summated scale
(such as created by the \code{\link[=lfast]{lfast()}} function),
and a desired \emph{Cronbach's Alpha}.
}
\details{
\subsection{simulated annealing}{

Simulated annealing is used in \code{makeItemsScale()} to search for item
configurations that satisfy several competing objectives simultaneously.
These include achieving a target Cronbach’s alpha, maintaining similar
item means, and preserving constant row sums. Because improving one
objective can worsen another, a simple greedy search often becomes
trapped in suboptimal solutions.

Simulated annealing addresses this by allowing occasional acceptance of
worse solutions early in the search, promoting exploration of the solution
space. As the algorithm progresses, a cooling schedule reduces this
randomness, increasingly favouring improvements across all criteria.
This process enables the function to converge on a balanced solution that
jointly satisfies the desired scale properties, rather than optimizing any
single objective at the expense of others.
}
}
\examples{

summatedScale <- c(16, 11, 14, 7, 10, 13, 16, 13, 12, 14, 8, 11, 8, 18, 8, 13)

newItems <- makeItemsScale(
  scale = summatedScale,
  lowerbound = 1,
  upperbound = 5,
  items = 4,
  alpha = 0.8
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeRepeated.R
\name{makeRepeated}
\alias{makeRepeated}
\title{Reproduce Repeated-Measures Data from ANOVA Summary Statistics}
\usage{
makeRepeated(
  n,
  k,
  means,
  sds,
  f_stat,
  df_between = k - 1,
  df_within = (n - 1) * (k - 1),
  structure = c("cs", "ar1", "toeplitz"),
  names = paste0("time_", 1:k),
  items = 1,
  lowerbound = 1,
  upperbound = 5,
  return_corr_only = FALSE,
  diagnostics = FALSE,
  ...
)
}
\arguments{
\item{n}{Integer. Sample size used in the original study.}

\item{k}{Integer. Number of repeated measures (timepoints).}

\item{means}{Numeric vector of length \code{k}.
Mean values reported for each timepoint.}

\item{sds}{Numeric vector of length \code{k}.
Standard deviations reported for each timepoint.}

\item{f_stat}{Numeric. The reported repeated-measures ANOVA
F-statistic for the within-subjects factor.}

\item{df_between}{Degrees of freedom between conditions (default: \code{k - 1}.}

\item{df_within}{Degrees of freedom within-subjects
(default: \code{(n - 1) * (k - 1)}).}

\item{structure}{Character. Correlation structure to assume:
\code{"cs"}, \code{"ar1"}, or \code{"toeplitz"} (default = \code{"cs"}).}

\item{names}{Character vector of length \code{k}. Variable names for each
timepoint (default: \code{"time_1"} to \code{"time_k"}).}

\item{items}{Integer. Number of items used to generate each scale score
(passed to \link{lfast}).}

\item{lowerbound, }{Integer. Lower bounds for Likert-type response
scales (default: 1).}

\item{upperbound, }{Integer. upper bounds for Likert-type response
scales (default: 5).}

\item{return_corr_only}{Logical. If \code{TRUE}, return only the
estimated correlation matrix.}

\item{diagnostics}{Logical. If \code{TRUE}, include diagnostic summaries
such as feasible F-statistic range and effect sizes.}

\item{...}{Reserved for future use.}
}
\value{
A named list with components:

\describe{
\item{\code{data}}{A data frame of simulated repeated-measures responses
(unless \code{return_corr_only = TRUE}).}
\item{\code{correlation_matrix}}{The estimated inter-timepoint correlation matrix.}
\item{\code{structure}}{The correlation structure assumed.}
\item{\code{achieved_f}}{The F-statistic produced by the estimated \code{rho} value
(if \code{diagnostics = TRUE}).}
\item{\code{feasible_f_range}}{Minimum and maximum achievable F-values under the
chosen structure (shown if diagnostics are requested).}
\item{\code{recommended_f}}{Conservative, moderate, and strong F-statistic
suggestions for similar designs.}
\item{\code{effect_size_raw}}{Unstandardised effect size across timepoints.}
\item{\code{effect_size_standardised}}{Effect size standardised by average variance.}
}
}
\description{
Constructs a synthetic dataset and inter-timepoint correlation matrix
from a repeated-measures ANOVA result, based on reported means, standard
deviations, and an F-statistic. This is useful when only summary statistics
are available from published studies.
}
\details{
This function estimates the average correlation between repeated measures
by matching the reported F-statistic, under one of three assumed
correlation structures:
\itemize{
\item \code{"cs"} (\emph{Compound Symmetry}): The Default. Assumes all timepoints are
equally correlated. Common in standard RM-ANOVA settings.
\item \code{"ar1"} (\emph{First-Order Autoregressive}): Assumes correlations decay
exponentially with time lag.
\item \code{"toeplitz"} (\emph{Linearly Decreasing}): Assumes correlation declines
linearly with time lag - a middle ground between \code{"cs"} and \code{"ar1"}.
}

The function then generates a data frame of synthetic item-scale ratings
using \link{lfast}, and adjusts them to match the estimated correlation
structure using \link{lcor}.

Set \code{return_corr_only = TRUE} to extract only the estimated
correlation matrix.
}
\examples{

set.seed(42)

out1 <- makeRepeated(
  n = 64,
  k = 3,
  means = c(3.1, 3.5, 3.9),
  sds = c(1.0, 1.1, 1.0),
  items = 4,
  f_stat = 4.87,
  structure = "cs",
  diagnostics = FALSE
)

head(out1$data)
out1$correlation_matrix

out2 <- makeRepeated(
  n = 32, k = 4,
  means = c(2.75, 3.5, 4.0, 4.4),
  sds = c(0.8, 1.0, 1.2, 1.0),
  f_stat = 16,
  structure = "ar1",
  items = 5,
  lowerbound = 1, upperbound = 7,
  return_corr_only = FALSE,
  diagnostics = TRUE
)

print(out2)


out3 <- makeRepeated(
  n = 64, k = 4,
  means = c(2.0, 2.25, 2.75, 3.0),
  sds = c(0.8, 0.9, 1.0, 0.9),
  items = 4,
  f_stat = 24,
  # structure = "toeplitz",
  diagnostics = TRUE
)

str(out3)

}
\seealso{
\code{\link{lfast}}, \code{\link{lcor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makePaired.R
\name{makePaired}
\alias{makePaired}
\title{Synthesise a dataset from paired-sample t-test summary statistics}
\usage{
makePaired(
  n,
  means,
  sds,
  t_value,
  lowerbound,
  upperbound,
  items = 1,
  precision = 0
)
}
\arguments{
\item{n}{(positive, integer) sample size}

\item{means}{(real) 1:2 vector of target means for two before/after measures}

\item{sds}{(real) 1:2 vector of target standard deviations}

\item{t_value}{(real) desired paired \emph{t}-statistic}

\item{lowerbound}{(integer) lower bound (e.g. '1' for a 1-5 rating scale)}

\item{upperbound}{(integer) upper bound (e.g. '5' for a 1-5 rating scale)}

\item{items}{(positive, integer) number of items in the rating scale.
Default = 1}

\item{precision}{(positive, real) relaxes the level of accuracy required.
Default = 0}
}
\value{
a dataframe approximating user-specified conditions.
}
\description{
Generates a dataset from paired-sample \emph{t}-test summary statistics.
}
\details{
\code{makePaired()} generates correlated values so the data replicate
rating scales taken, for example, in a before and after experimental design.

The function is effectively a wrapper function for
\code{lfast()} and \code{lcor()} with the addition of a
t-statistic from which the between-column correlation is inferred.

Paired \emph{t}-tests apply to observations that are associated with each other.
For example: the same people before and after a treatment;
the same people rating two different objects; ratings by husband & wife;
\emph{etc.}

The paired-samples \emph{t}-test is defined as:

\deqn{ t = \frac{\mathrm{mean}(D)}{\mathrm{sd}(D) / \sqrt{n}} }

where:
\itemize{
\item \eqn{D} = differences in values
\item \eqn{\mathrm{mean}(D)} = mean of the differences
\item \eqn{\mathrm{sd}(D)} = standard deviation of the differences, where
}

\deqn{ \mathrm{sd}(D)^2 = \mathrm{sd}(X_{\text{before}})^2 +
                          \mathrm{sd}(X_{\text{after}})^2 -
                          2\,\mathrm{cov}(X_{\text{before}},
                          X_{\text{after}}) }

A paired-sample \emph{t}-test thus requires an estimate of the covariance between
the two sets of observations.
\code{makePaired()} rearranges these formulae so that the covariance is
inferred from the t-statistic.
}
\note{
Larger sample sizes usually result in higher t-statistics,
and correspondingly small p-values.

Small sample sizes with relatively large standard deviations and
relatively high t-statistics can result in impossible correlation values.

Similarly, large sample sizes with low \emph{t}-statistics can result in
impossible correlations. That is, a correlation outside of the -1:+1 range.

If this happens, the function will fail with an \emph{ERROR} message.
The user should review the input parameters and insert more realistic values.
}
\examples{

n <- 20
pair_m <- c(2.5, 3.0)
pair_s <- c(1.0, 1.5)
lower <- 1
upper <- 5
k <- 6
t <- -2.5

pairedDat <- makePaired(
  n = n, means = pair_m, sds = pair_s,
  t_value = t,
  lowerbound = lower, upperbound = upper, items = k
)

str(pairedDat)
cor(pairedDat) |> round(2)

t.test(pairedDat$X1, pairedDat$X2, paired = TRUE)

}
\seealso{
\code{\link{lfast}}, \code{\link{lcor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlateScales.R
\name{correlateScales}
\alias{correlateScales}
\title{Dataframe of correlated scales from different dataframes of scale items}
\usage{
correlateScales(dataframes, scalecors)
}
\arguments{
\item{dataframes}{a list of 'k' dataframes to be rearranged and combined}

\item{scalecors}{target correlation matrix - should be a symmetric
\eqn{k \times k} positive-semi-definite matrix,
where 'k' is the number of dataframes}
}
\value{
Returns a dataframe whose columns are taken from the starter
dataframes and whose summated values are correlated according to a
user-specified correlation matrix
}
\description{
\code{correlateScales()} creates a dataframe of
scale items representing correlated constructs,
as one might find in a completed questionnaire.
}
\details{
Correlated rating-scale items generally are summed or
averaged to create a measure of an "unobservable", or "latent", construct.
\code{correlateScales()} takes several such dataframes of rating-scale
items and rearranges their rows so that the scales are correlated according
to a predefined correlation matrix. Univariate statistics for each dataframe
of rating-scale items do not change,
but their correlations with rating-scale items in other dataframes do.
}
\examples{

## three attitudes and a behavioural intention
n <- 32
lower <- 1
upper <- 5

### attitude #1
cor_1 <- makeCorrAlpha(items = 4, alpha = 0.90)
means_1 <- c(2.5, 2.5, 3.0, 3.5)
sds_1 <- c(0.9, 1.0, 0.9, 1.0)

Att_1 <- makeScales(
  n = n, means = means_1, sds = sds_1,
  lowerbound = rep(lower, 4), upperbound = rep(upper, 4),
  items = 4,
  cormatrix = cor_1
)


### attitude #2
cor_2 <- makeCorrAlpha(items = 5, alpha = 0.85)
means_2 <- c(2.5, 2.5, 3.0, 3.0, 3.5)
sds_2 <- c(1.0, 1.0, 0.9, 1.0, 1.5)

Att_2 <- makeScales(
  n = n, means = means_2, sds = sds_2,
  lowerbound = rep(lower, 5), upperbound = rep(upper, 5),
  items = 5,
  cormatrix = cor_2
)


### attitude #3
cor_3 <- makeCorrAlpha(items = 6, alpha = 0.75)
means_3 <- c(2.5, 2.5, 3.0, 3.0, 3.5, 3.5)
sds_3 <- c(1.0, 1.5, 1.0, 1.5, 1.0, 1.5)

Att_3 <- makeScales(
  n = n, means = means_3, sds = sds_3,
  lowerbound = rep(lower, 6), upperbound = rep(upper, 6),
  items = 6,
  cormatrix = cor_3
)


### behavioural intention
intent <- lfast(n, mean = 3.0, sd = 3, lowerbound = 0, upperbound = 10) |>
  data.frame()
names(intent) <- "int"


### target scale correlation matrix
scale_cors <- matrix(
  c(
    1.0, 0.6, 0.5, 0.3,
    0.6, 1.0, 0.4, 0.2,
    0.5, 0.4, 1.0, 0.1,
    0.3, 0.2, 0.1, 1.0
  ),
  nrow = 4
)

data_frames <- list("A1" = Att_1, "A2" = Att_2, "A3" = Att_3, "Int" = intent)


### apply the function
my_correlated_scales <- correlateScales(
  dataframes = data_frames,
  scalecors = scale_cors
)
head(my_correlated_scales)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigenvalues.R
\name{eigenvalues}
\alias{eigenvalues}
\title{calculate eigenvalues of a correlation matrix with optional scree plot}
\usage{
eigenvalues(cormatrix, scree = FALSE)
}
\arguments{
\item{cormatrix}{(real, matrix) a correlation matrix}

\item{scree}{(logical) default = \code{FALSE}. If \code{TRUE} (or \code{1}), then
\code{eigenvalues()} produces a scree plot to illustrate the eigenvalues}
}
\value{
a vector of eigenvalues

report on positive-definite status of cormatrix
}
\description{
\code{eigenvalues()} calculates eigenvalues of a correlation
matrix and optionally produces a scree plot.
}
\examples{

## define parameters

correlationMatrix <- matrix(
  c(
    1.00, 0.25, 0.35, 0.40,
    0.25, 1.00, 0.70, 0.75,
    0.35, 0.70, 1.00, 0.80,
    0.40, 0.75, 0.80, 1.00
  ),
  nrow = 4, ncol = 4
)

## apply function

evals <- eigenvalues(cormatrix = correlationMatrix)
evals <- eigenvalues(correlationMatrix, 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lcor.R
\name{lcor}
\alias{lcor}
\title{Rearrange elements in each column of a data-frame to fit a
predefined correlation matrix}
\usage{
lcor(data, target, passes = 10)
}
\arguments{
\item{data}{dataframe that is to be rearranged}

\item{target}{target correlation matrix.
Must have same dimensions as number of columns in data-frame.}

\item{passes}{Number of optimization passes (default = 10).
Increasing this value \emph{MAY} improve results if n-columns
(target correlation matrix dimensions) are many.
Decreasing the value for 'passes' is faster but may decrease accuracy.}
}
\value{
Returns a dataframe whose column-wise correlations
approximate a user-specified correlation matrix
}
\description{
\code{lcor()} rearranges values in each column of a
data-frame so that columns are correlated to match a predefined
correlation matrix.
}
\details{
Values in a column do not change, so univariate
statistics remain the same.
}
\examples{

## parameters
n <- 32
lowerbound <- 1
upperbound <- 5
items <- 5

mydat3 <- data.frame(
  x1 = lfast(n, 2.5, 0.75, lowerbound, upperbound, items),
  x2 = lfast(n, 3.0, 1.50, lowerbound, upperbound, items),
  x3 = lfast(n, 3.5, 1.00, lowerbound, upperbound, items)
)

cor(mydat3) |> round(3)

tgt3 <- matrix(
  c(
    1.00, 0.50, 0.75,
    0.50, 1.00, 0.25,
    0.75, 0.25, 1.00
  ),
  nrow = 3, ncol = 3
)

## apply function
new3 <- lcor(mydat3, tgt3)

## test output
cor(new3) |> round(3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeScalesRegression.R
\name{makeScalesRegression}
\alias{makeScalesRegression}
\title{Generate Data from Multiple-Regression Summary Statistics}
\usage{
makeScalesRegression(
  n,
  beta_std,
  r_squared,
  iv_cormatrix = NULL,
  iv_cor_mean = 0.3,
  iv_cor_variance = 0.01,
  iv_cor_range = c(-0.7, 0.7),
  iv_means,
  iv_sds,
  dv_mean,
  dv_sd,
  lowerbound_iv,
  upperbound_iv,
  lowerbound_dv,
  upperbound_dv,
  items_iv = 1,
  items_dv = 1,
  var_names = NULL,
  tolerance = 0.005
)
}
\arguments{
\item{n}{Integer. Sample size}

\item{beta_std}{Numeric vector of standardised regression coefficients
(length k)}

\item{r_squared}{Numeric. R-squared from regression (-1 to 1)}

\item{iv_cormatrix}{k x k correlation matrix of independent variables.
If missing (NULL), will be optimised.}

\item{iv_cor_mean}{Numeric. Mean correlation among IVs when optimising
(ignored if iv_cormatrix provided). Default = 0.3}

\item{iv_cor_variance}{Numeric. Variance of correlations when optimising
(ignored if iv_cormatrix provided). Default = 0.01}

\item{iv_cor_range}{Numeric vector of length 2.
Min and max constraints on correlations when optimising.
Default = c(-0.7, 0.7)}

\item{iv_means}{Numeric vector of means for IVs (length k)}

\item{iv_sds}{Numeric vector of standard deviations for IVs (length k)}

\item{dv_mean}{Numeric. Mean of dependent variable}

\item{dv_sd}{Numeric. Standard deviation of dependent variable}

\item{lowerbound_iv}{Numeric vector of lower bounds for each IV scale
(or single value for all)}

\item{upperbound_iv}{Numeric vector of upper bounds for each IV scale
(or single value for all)}

\item{lowerbound_dv}{Numeric. Lower bound for DV scale}

\item{upperbound_dv}{Numeric. Upper bound for DV scale}

\item{items_iv}{Integer vector of number of items per IV scale
(or single value for all). Default = 1}

\item{items_dv}{Integer. Number of items in DV scale. Default = 1}

\item{var_names}{Character vector of variable names
(length k+1: IVs then DV)}

\item{tolerance}{Numeric. Acceptable deviation from target R-squared
(default 0.005)}
}
\value{
A list containing:
\item{data}{Generated dataframe with k IVs and 1 DV}
\item{target_stats}{List of target statistics provided}
\item{achieved_stats}{List of achieved statistics from generated data}
\item{diagnostics}{Comparison of target vs achieved}
\item{iv_dv_cors}{Calculated correlations between IVs and DV}
\item{full_cormatrix}{The complete (k+1) x (k+1) correlation matrix used}
\item{optimisation_info}{If IV correlations were optimised,
details about the optimisation}
}
\description{
Generates synthetic rating-scale data that replicates reported regression
results. This function is useful for reproducing analyses from published
research where only summary statistics (standardised regression
coefficients and R-squared) are reported.
}
\details{
Generate regression data from summary statistics

The function can operate in two modes:

\strong{Mode 1: With IV correlation matrix provided}

When \code{iv_cormatrix} is provided, the function uses the given
correlation structure among independent variables and calculates the
implied IV-DV correlations from the regression coefficients.

\strong{Mode 2: With optimisation (IV correlation matrix not provided)}

When \code{iv_cormatrix = NULL}, the function optimises to find a plausible
correlation structure among independent variables that matches the reported
regression statistics.
Initial correlations are sampled using Fisher's z-transformation to ensure
proper distribution, then iteratively adjusted to match the target
R-squared.

The function generates Likert-scale data (not individual items)
using \code{lfast()} for each variable with specified moments, then
correlates them using \code{lcor()}.
Generated data are verified by running a regression and comparing achieved
statistics with targets.
}
\examples{

# Example 1: With provided IV correlation matrix
set.seed(123)
iv_corr <- matrix(c(1.0, 0.3, 0.3, 1.0), nrow = 2)

result1 <- makeScalesRegression(
  n = 64,
  beta_std = c(0.4, 0.3),
  r_squared = 0.35,
  iv_cormatrix = iv_corr,
  iv_means = c(3.0, 3.5),
  iv_sds = c(1.0, 0.9),
  dv_mean = 3.8,
  dv_sd = 1.1,
  lowerbound_iv = 1,
  upperbound_iv = 5,
  lowerbound_dv = 1,
  upperbound_dv = 5,
  items_iv = 4,
  items_dv = 4,
  var_names = c("Attitude", "Intention", "Behaviour")
)

print(result1)
head(result1$data)


# Example 2: With optimisation (no IV correlation matrix)
set.seed(456)
result2 <- makeScalesRegression(
  n = 128,
  beta_std = c(0.3, 0.25, 0.2),
  r_squared = 0.40,
  iv_cormatrix = NULL, # Will be optimised
  iv_cor_mean = 0.3,
  iv_cor_variance = 0.02,
  iv_means = c(3.0, 3.2, 2.8),
  iv_sds = c(1.0, 0.9, 1.1),
  dv_mean = 3.5,
  dv_sd = 1.0,
  lowerbound_iv = 1,
  upperbound_iv = 5,
  lowerbound_dv = 1,
  upperbound_dv = 5,
  items_iv = 4,
  items_dv = 5
)

# View optimised correlation matrix
print(result2$target_stats$iv_cormatrix)
print(result2$optimisation_info)

}
\seealso{
\code{\link{lfast}} for generating individual rating-scale vectors with exact moments.

\code{\link{lcor}} for rearranging values to achieve target correlations.

\code{\link{makeCorrAlpha}} for generating correlation matrices from Cronbach's Alpha.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeScalesRegression.R
\name{print.makeScalesRegression}
\alias{print.makeScalesRegression}
\title{Print method for makeScalesRegression objects}
\usage{
\method{print}{makeScalesRegression}(x, ...)
}
\arguments{
\item{x}{An object of class "makeScalesRegression"}

\item{...}{Additional arguments (currently unused)}
}
\description{
Print method for makeScalesRegression objects
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeCorrLoadings.R
\name{makeCorrLoadings}
\alias{makeCorrLoadings}
\title{Generate Inter-Item Correlation Matrix from Factor Loadings}
\usage{
makeCorrLoadings(
  loadings,
  factorCor = NULL,
  uniquenesses = NULL,
  nearPD = FALSE,
  diagnostics = FALSE
)
}
\arguments{
\item{loadings}{Numeric matrix. A \eqn{k \times f} matrix of standardized
factor loadings \eqn{items \times factors}.
Row names and column names are used for diagnostics if present.}

\item{factorCor}{Optional \eqn{f \times f} matrix of factor
correlations (\eqn{\Phi}).
If NULL, assumes orthogonal factors.}

\item{uniquenesses}{Optional vector of length k. If NULL, calculated
as \eqn{1 - rowSums(loadings^2)}.}

\item{nearPD}{Logical.
If \code{TRUE}, attempts to coerce non–positive-definite matrices
using \code{\link[Matrix:nearPD]{Matrix::nearPD()}}.}

\item{diagnostics}{Logical.
If \code{TRUE}, returns diagnostics including McDonald's Omega and
item-level summaries.}
}
\value{
If diagnostics = FALSE, returns a correlation matrix (class: matrix).
If diagnostics = TRUE, returns a list with:
- R: correlation matrix
- Omega: per-factor Omega or adjusted Omega
- OmegaTotal: total Omega across all factors
- Diagnostics: dataframe of communalities, uniquenesses, and primary factor
}
\description{
Constructs an inter-item correlation matrix based on a user-supplied
matrix of standardised factor loadings and (optionally) a factor
correlation matrix.
The \code{makeCorrLoadings()} function does a surprisingly good job of
reproducing a target correlation matrix when all item-factor loadings
are present, as shown in the \emph{makeCorrLoadings() validation article}.
}
\examples{

# --------------------------------------------------------
# Example 1: Basic use without diagnostics
# --------------------------------------------------------

factorLoadings <- matrix(
  c(
    0.05, 0.20, 0.70,
    0.10, 0.05, 0.80,
    0.05, 0.15, 0.85,
    0.20, 0.85, 0.15,
    0.05, 0.85, 0.10,
    0.10, 0.90, 0.05,
    0.90, 0.15, 0.05,
    0.80, 0.10, 0.10
  ),
  nrow = 8, ncol = 3, byrow = TRUE
)

rownames(factorLoadings) <- paste0("Q", 1:8)
colnames(factorLoadings) <- c("Factor1", "Factor2", "Factor3")

factorCor <- matrix(
  c(
    1.0,  0.7, 0.6,
    0.7,  1.0, 0.4,
    0.6,  0.4, 1.0
  ),
  nrow = 3, byrow = TRUE
)

itemCor <- makeCorrLoadings(factorLoadings, factorCor)
round(itemCor, 3)

# --------------------------------------------------------
# Example 2: Diagnostics with factor correlations (Adjusted Omega)
# --------------------------------------------------------

result_adj <- makeCorrLoadings(
  loadings = factorLoadings,
  factorCor = factorCor,
  diagnostics = TRUE
)

# View outputs
round(result_adj$R, 3) # correlation matrix
round(result_adj$Omega, 3) # adjusted Omega
round(result_adj$OmegaTotal, 3) # total Omega
print(result_adj$Diagnostics) # communality and uniqueness per item

# --------------------------------------------------------
# Example 3: Diagnostics assuming orthogonal factors (Per-Factor Omega)
# --------------------------------------------------------

result_orth <- makeCorrLoadings(
  loadings = factorLoadings,
  diagnostics = TRUE
)

round(result_orth$Omega, 3) # per-factor Omega
round(result_orth$OmegaTotal, 3) # total Omega
print(result_orth$Diagnostics)

}
\seealso{
\itemize{
\item makeCorrLoadings() validation article:
\url{https://winzarh.github.io/LikertMakeR/articles/makeCorrLoadings_validate.html}
\item Package website: \url{https://winzarh.github.io/LikertMakeR/}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeScales.R
\name{makeScales}
\alias{makeScales}
\title{Synthesise rating-scale data with given first and second moments and a
predefined correlation matrix}
\usage{
makeScales(n, means, sds, lowerbound = 1, upperbound = 5, items = 1, cormatrix)
}
\arguments{
\item{n}{(positive, int) sample-size - number of observations}

\item{means}{(real) target means: a vector of length k
of mean values for each scale item}

\item{sds}{(positive, real) target standard deviations: a vector of length k
of standard deviation values for each scale item}

\item{lowerbound}{(positive, int) a vector of length k
(same as rows & columns of correlation matrix) of values for lower bound
of each scale item (e.g. '1' for a 1-5 rating scale). Default = 1.}

\item{upperbound}{(positive, int) a vector of length k
(same as rows & columns of correlation matrix) of values for upper bound
of each scale item (e.g. '5' for a 1-5 rating scale). Default = 5.}

\item{items}{(positive, int) a vector of length k of number of items in each scale. Default = 1.}

\item{cormatrix}{(real, matrix) the target correlation matrix:
a square symmetric positive-semi-definite matrix of values ranging
between -1 and +1, and '1' in the diagonal.}
}
\value{
a dataframe of rating-scale values
}
\description{
Generates a dataframe of random discrete
values so the data replicate a rating scale,
and are correlated close to a predefined correlation matrix.
}
\details{
\code{makeScales()} is wrapper function for:
\itemize{
\item \code{\link[=lfast]{lfast()}}, generates a dataframe that best fits the desired
moments, and
\item \code{\link[=lcor]{lcor()}}, which rearranges values in each column of the dataframe
so they closely match the desired correlation matrix.
}
}
\examples{

## Example 1: four correlated items (questions)

### define parameters

n <- 16
dfMeans <- c(2.5, 3.0, 3.0, 3.5)
dfSds <- c(1.0, 1.0, 1.5, 0.75)
lowerbound <- rep(1, 4)
upperbound <- rep(5, 4)

corMat <- matrix(
  c(
    1.00, 0.30, 0.40, 0.60,
    0.30, 1.00, 0.50, 0.70,
    0.40, 0.50, 1.00, 0.80,
    0.60, 0.70, 0.80, 1.00
  ),
  nrow = 4, ncol = 4
)

scale_names <- c("Q1", "Q2", "Q3", "Q4")
rownames(corMat) <- scale_names
colnames(corMat) <- scale_names

### apply function

df1 <- makeScales(
  n = n, means = dfMeans, sds = dfSds,
  lowerbound = lowerbound, upperbound = upperbound, cormatrix = corMat
)

### test function

str(df1)

#### means
apply(df1, 2, mean) |> round(3)

#### standard deviations
apply(df1, 2, sd) |> round(3)

#### correlations
cor(df1) |> round(3)


## Example 2: five correlated Likert scales

### a study on employee engagement and organizational climate:
# Job Satisfaction (JS)
# Organizational Commitment (OC)
# Perceived Supervisor Support (PSS)
# Work Engagement (WE)
# Turnover Intention (TI) (reverse-related to others).

### define parameters

n <- 128
dfMeans <- c(3.8, 3.6, 3.7, 3.9, 2.2)
dfSds <- c(0.7, 0.8, 0.7, 0.6, 0.9)
lowerbound <- rep(1, 5)
upperbound <- rep(5, 5)
items <- c(4, 4, 3, 3, 3)

corMat <- matrix(
  c(
    1.00, 0.72, 0.58, 0.65, -0.55,
    0.72, 1.00, 0.54, 0.60, -0.60,
    0.58, 0.54, 1.00, 0.57, -0.45,
    0.65, 0.60, 0.57, 1.00, -0.50,
    -0.55, -0.60, -0.45, -0.50, 1.00
  ),
  nrow = 5, ncol = 5
)

scale_names <- c("JS", "OC", "PSS", "WE", "TI")
rownames(corMat) <- scale_names
colnames(corMat) <- scale_names

### apply function

df2 <- makeScales(
  n = n, means = dfMeans, sds = dfSds,
  lowerbound = lowerbound, upperbound = upperbound,
  items = items, cormatrix = corMat
)

### test function

str(df2)

#### means
apply(df2, 2, mean) |> round(3)

#### standard deviations
apply(df2, 2, sd) |> round(3)

#### correlations
cor(df2) |> round(3)

}
\seealso{
\code{\link{lfast}}, \code{\link{lcor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeCorrAlpha.R
\name{makeCorrAlpha}
\alias{makeCorrAlpha}
\title{Correlation matrix from Cronbach's Alpha}
\usage{
makeCorrAlpha(
  items,
  alpha,
  variance = 0.1,
  alpha_noise = 0,
  diagnostics = FALSE
)
}
\arguments{
\item{items}{Integer. Number of items (>= 2).}

\item{alpha}{Numeric. Target Cronbach's alpha (0 < alpha < 1).}

\item{variance}{Numeric. Controls heterogeneity of item loadings in the
underlying one-factor model.

Larger values produce greater dispersion among item loadings, which
results in a wider spread of inter-item correlations while preserving
the requested Cronbach's alpha.

Typical guidance:
\itemize{
\item \code{0.05} — near-parallel items (very similar correlations)
\item \code{0.10} — modest heterogeneity (default)
\item \code{0.15} — strong heterogeneity
\item \code{0.20} — very strong heterogeneity
\item \code{> 0.25} — extreme dispersion; internal shrinkage may occur
}

For most applied psychometric scales (k < 20), values between
\code{0.05} and \code{0.15} produce realistic correlation structures.
Values above \code{0.30} are automatically reduced to \code{0.30} to
satisfy algorithm constraints.}

\item{alpha_noise}{Numeric. Controls random variation in the target
Cronbach's alpha before the correlation matrix is constructed.

When \code{alpha_noise = 0} (default), the requested alpha is
reproduced deterministically (subject to numerical tolerance).

When \code{alpha_noise > 0}, a small amount of random variation is
added to the requested alpha prior to constructing the matrix. This
can be useful in teaching or simulation settings where slightly
different reliability values are desired across repeated runs.

Internally, noise is added on the Fisher \emph{z}-transformed scale
to ensure the resulting alpha remains within valid bounds (0, 1).

Typical guidance:
\itemize{
\item \code{0.00} — deterministic alpha (default)
\item \code{0.02} — very small variation
\item \code{0.05} — moderate variation
\item \code{0.10} — substantial variation (caution)
}

Larger values increase the spread of achieved alpha across runs.}

\item{diagnostics}{Logical. If \code{TRUE}, returns a list containing
the matrix and diagnostic information.}
}
\value{
If \code{diagnostics = FALSE}, a positive-definite correlation matrix.
If \code{diagnostics = TRUE}, a list containing:
\itemize{
\item \code{R}: the correlation matrix
\item \code{diagnostics}: list including achieved alpha,
minimum eigenvalue, and internal variance used
}
}
\description{
Generate a Positive-Definite
Correlation Matrix for a target \emph{Cronbach's alpha}.
}
\details{
Constructs a correlation matrix with a specified number of items and
target \emph{Cronbach's alpha} using a constructive one-factor model.

Such a correlation matrix can be applied to the \code{\link[=makeScales]{makeScales()}}
function to generate synthetic data with the predefined alpha.

The algorithm directly builds a positive-definite correlation matrix
by solving for item loadings that reproduce the desired average
inter-item correlation implied by \emph{alpha}.
Unlike earlier versions of this function, this method guarantees positive
definiteness by construction, without \emph{post-hoc} repair.

The function computes the average inter-item correlation implied by the
requested alpha and solves for a one-factor loading structure that
reproduces this value. A small adaptive reduction in dispersion may be
applied when necessary to ensure a valid positive-definite solution.

The constructive generator assumes a single common factor structure,
consistent with typical psychometric scale construction.
}
\examples{

# Example 1
# define parameters
items <- 4
alpha <- 0.85

# apply function
set.seed(42)
cor_matrix <- makeCorrAlpha(
  items = items,
  alpha = alpha
)

# test function output
print(cor_matrix) |> round(3)
alpha(cor_matrix)
eigenvalues(cor_matrix, 1)

# Example 2
# higher alpha, more items, more variability
cor_matrix2 <- makeCorrAlpha(
  items = 8,
  alpha = 0.95,
  variance = 0.10
)

# test output
cor_matrix2 |> round(2)
alpha(cor_matrix2) |> round(3)
eigenvalues(cor_matrix2, 1) |> round(3)

# Example 3
# large random variation around alpha
cor_matrix3 <- makeCorrAlpha(
  items = 6,
  alpha = 0.85,
  alpha_noise = 0.10
)

# test output
cor_matrix3 |> round(2)
alpha(cor_matrix3) |> round(3)
eigenvalues(cor_matrix3, 1) |> round(3)

# Example 4
# with diagnostics
cor_matrix4 <- makeCorrAlpha(
  items = 4,
  alpha = 0.80,
  diagnostics = TRUE
)

# test output
cor_matrix4

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{ordinal_diagnostics}
\alias{ordinal_diagnostics}
\title{Extract ordinal diagnostics from a reliability() result}
\usage{
ordinal_diagnostics(x)
}
\arguments{
\item{x}{An object returned by \code{\link[=reliability]{reliability()}}.}
}
\value{
A data.frame describing observed response categories and sparsity checks,
or NULL if no diagnostics are available.
}
\description{
Extract ordinal diagnostics from a reliability() result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{print.likert_reliability}
\alias{print.likert_reliability}
\title{Print method for reliability objects}
\usage{
\method{print}{likert_reliability}(x, ...)
}
\arguments{
\item{x}{An object returned by \code{\link[=reliability]{reliability()}}.}

\item{...}{Unused.}
}
\description{
Print method for reliability objects
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{reliability}
\alias{reliability}
\title{Estimate scale reliability for Likert and rating-scale data}
\usage{
reliability(
  data,
  include = "none",
  ci = FALSE,
  ci_level = 0.95,
  n_boot = 1000,
  na_method = c("pairwise", "listwise"),
  min_count = 2,
  digits = 3,
  verbose = TRUE
)
}
\arguments{
\item{data}{A data frame or matrix containing item responses.
Each column represents one item; rows represent respondents.}

\item{include}{Character vector specifying which additional estimates
to compute. Possible values are:
\itemize{
\item \code{"none"} (default): Pearson-based alpha and omega only.
\item \code{"lambda6"}: Include Guttman’s lambda-6 (requires package \pkg{psych}).
\item \code{"polychoric"}: Include ordinal (polychoric-based) alpha and omega.
}
Multiple options may be supplied.}

\item{ci}{Logical; if \code{TRUE}, confidence intervals are computed using
nonparametric bootstrap. Default is \code{FALSE}.}

\item{ci_level}{Confidence level for bootstrap intervals.
Default is \code{0.95}.}

\item{n_boot}{Number of bootstrap resamples used when \code{ci = TRUE}.
Default is \code{1000}.}

\item{na_method}{Method for handling missing values. Either
\code{"pairwise"} (default) or \code{"listwise"}.}

\item{min_count}{Minimum observed frequency per response category required
to attempt polychoric correlations. Ordinal reliability estimates are
skipped if this condition is violated. Default is \code{2}.}

\item{digits}{Number of decimal places used when printing estimates.
Default is \code{3}.}

\item{verbose}{Logical; if \code{TRUE}, warnings and progress indicators
are displayed. Default is \code{TRUE}.}
}
\value{
A tibble with one row per reliability coefficient and columns:
\itemize{
\item \code{coef_name}: Name of the reliability coefficient.
\item \code{estimate}: Point estimate.
\item \code{ci_lower}, \code{ci_upper}: Confidence interval bounds
(only present when \code{ci = TRUE}).
\item \code{notes}: Methodological notes describing how the estimate
was obtained.
}

The returned object has class \code{"likert_reliability"} and includes
additional attributes containing diagnostics and bootstrap information.
}
\description{
Computes internal consistency reliability estimates for a single-factor
scale, including Cronbach’s alpha, McDonald’s omega (total), and optional
ordinal (polychoric-based) variants. Confidence intervals may be obtained
via nonparametric bootstrap.
}
\details{
The function is designed for Likert-type and rating-scale data and
prioritises transparent diagnostics when ordinal reliability estimates
are not feasible due to sparse response categories.

Cronbach’s alpha and McDonald’s omega are computed from Pearson correlations.
When \code{include = "polychoric"}, ordinal reliability estimates are computed
using polychoric correlations and correspond to Zumbo’s alpha and ordinal omega.

Ordinal reliability estimates are skipped if response categories are sparse
or if polychoric estimation fails. Diagnostics explaining these decisions
are stored in the returned object and may be inspected using
\code{\link{ordinal_diagnostics}}.

This function assumes a single common factor and is not intended for
multidimensional or structural equation modelling contexts.
}
\examples{

## create dataset
my_cor <- LikertMakeR::makeCorrAlpha(
  items = 4,
  alpha = 0.80
)

my_data <- LikertMakeR::makeScales(
  n = 64,
  means = c(2.75, 3.00, 3.25, 3.50),
  sds = c(1.25, 1.50, 1.30, 1.25),
  lowerbound = rep(1, 4),
  upperbound = rep(5, 4),
  cormatrix = my_cor
)

## run function
reliability(my_data)

reliability(
  my_data,
  include = c("lambda6", "polychoric")
)

\donttest{
## slower (not run on CRAN checks)
reliability(
  my_data,
  include = "polychoric",
  ci = TRUE,
  n_boot = 200
)
}

}
\seealso{
\code{\link{ordinal_diagnostics}}

\code{\link{alpha_sensitivity}}, \code{\link{alpha}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alpha.R
\name{alpha}
\alias{alpha}
\title{Calculate Cronbach's Alpha from a correlation matrix or dataframe}
\usage{
alpha(cormatrix = NULL, data = NULL)
}
\arguments{
\item{cormatrix}{(real) a square symmetrical matrix with values
ranging from -1 to +1 and '1' in the diagonal}

\item{data}{(real) a dataframe or matrix}
}
\value{
a single value
}
\description{
\code{alpha()} calculates Cronbach's Alpha from a given
correlation matrix or a given dataframe.
}
\examples{

## Sample data frame
df <- data.frame(
  V1  =  c(4, 2, 4, 3, 2, 2, 2, 1),
  V2  =  c(4, 1, 3, 4, 4, 3, 2, 3),
  V3  =  c(4, 1, 3, 5, 4, 1, 4, 2),
  V4  =  c(4, 3, 4, 5, 3, 3, 3, 3)
)

## example correlation matrix
corMat <- matrix(
  c(
    1.00, 0.35, 0.45, 0.70,
    0.35, 1.00, 0.60, 0.55,
    0.45, 0.60, 1.00, 0.65,
    0.70, 0.55, 0.65, 1.00
  ),
  nrow = 4, ncol = 4
)

## apply function examples

alpha(cormatrix = corMat)

alpha(, df)

alpha(corMat, df)

}
\seealso{
\code{\link{alpha_sensitivity}}, \code{\link{reliability}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lfast.R
\name{lfast}
\alias{lfast}
\title{Synthesise rating-scale data with predefined mean and standard deviation}
\usage{
lfast(n, mean, sd, lowerbound, upperbound, items = 1, precision = 0)
}
\arguments{
\item{n}{(positive, int) number of observations to generate}

\item{mean}{(real) target mean, between upper and lower bounds}

\item{sd}{(positive, real) target standard deviation}

\item{lowerbound}{(int) lower bound (e.g. '1' for a 1-5 rating scale)}

\item{upperbound}{(int) upper bound (e.g. '5' for a 1-5 rating scale)}

\item{items}{(positive, int) number of items in the rating scale.
Default = 1}

\item{precision}{(positive, real) can relax the level of accuracy required.
(e.g. '1' generally generates a vector with moments correct within '0.025',
'2' generally within '0.05') Default = 0}
}
\value{
a vector approximating user-specified conditions.
}
\description{
\code{lfast()} applies a simple Evolutionary Algorithm to
find a vector that best fits the desired moments.

\code{lfast()} generates random discrete values from a
scaled Beta distribution so the data replicate an ordinal rating scale -
for example, a Likert scale made from multiple items (questions) or 0-10
likelihood-of-purchase scale. Data generated are generally consistent with
real data, as shown in the \emph{lfast() validation article}.
}
\examples{

## six-item 1-7 rating scale
x <- lfast(
  n = 256,
  mean = 4.0,
  sd = 1.25,
  lowerbound = 1,
  upperbound = 7,
  items = 6
)

## five-item -3 to +3 rating scale
x <- lfast(
  n = 64,
  mean = 0.025,
  sd = 1.25,
  lowerbound = -3,
  upperbound = 3,
  items = 5
)

## four-item 1-5 rating scale with medium variation
x <- lfast(
  n = 128,
  mean = 3.0,
  sd = 1.00,
  lowerbound = 1,
  upperbound = 5,
  items = 4,
  precision = 5
)

## eleven-point 'likelihood of purchase' scale
x <- lfast(256, 3, 3.0, 0, 10)

}
\seealso{
\itemize{
\item lfast validation article:
\url{https://winzarh.github.io/LikertMakeR/articles/lfast_validation.html}
\item Package website:
\url{https://winzarh.github.io/LikertMakeR/}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lexact.R
\name{lexact}
\alias{lexact}
\title{Deprecated. Use lfast() instead}
\usage{
lexact(n, mean, sd, lowerbound, upperbound, items = 1)
}
\arguments{
\item{n}{(positive, int) number of observations to generate}

\item{mean}{(real) target mean}

\item{sd}{(real) target standard deviation}

\item{lowerbound}{(positive, int) lower bound}

\item{upperbound}{(positive, int) upper bound}

\item{items}{(positive, int) number of items in the rating scale.}
}
\value{
a vector of simulated data approximating user-specified conditions.
}
\description{
\code{lexact} is DEPRECATED.
Replaced in LikertMakeR Version 0.4.0 by new version of \code{lfast}.

\code{lexact} remains as a legacy for earlier package users.
It is now just a wrapper for \code{lfast}

Previously, \code{lexact} used a Differential Evolution (DE) algorithm to
find an optimum solution with desired mean and standard deviation,
but we found that the updated \code{lfast} function is much faster and just
as accurate.

Also the package is much less bulky.
}
\examples{

x <- lexact(
  n = 256,
  mean = 4.0,
  sd = 1.0,
  lowerbound = 1,
  upperbound = 7,
  items = 6
)

x <- lexact(256, 2, 1.8, 0, 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alpha_sensitivity.R
\name{alpha_sensitivity}
\alias{alpha_sensitivity}
\title{Sensitivity of Cronbach's alpha to scale design parameters}
\usage{
alpha_sensitivity(
  data = NULL,
  k = NULL,
  r_bar = NULL,
  vary = c("r_bar", "k"),
  k_range = NULL,
  r_bar_range = NULL,
  plot = TRUE,
  digits = 3
)
}
\arguments{
\item{data}{A data frame or matrix of item responses. If provided,
\eqn{k} and \eqn{\bar{r}} are computed from the data.}

\item{k}{Number of items. Required if \code{data} is not supplied.}

\item{r_bar}{Average inter-item correlation. Required if \code{data}
is not supplied.}

\item{vary}{Character string indicating which parameter to vary:
\code{"r_bar"} (default) or \code{"k"}.}

\item{k_range}{Numeric vector of item counts to evaluate when
\code{vary = "k"}. Default is \code{2:20}.}

\item{r_bar_range}{Numeric vector of average inter-item correlations
to evaluate when \code{vary = "r_bar"}. Default is
\code{seq(0.05, 0.9, by = 0.05)}.}

\item{plot}{Logical; if \code{TRUE}, a base R plot is produced.
Default is \code{TRUE}.}

\item{digits}{Number of decimal places for rounding output.
Default is \code{3}.}
}
\value{
A data frame with columns:
\itemize{
\item \code{k}: number of items
\item \code{r_bar}: average inter-item correlation
\item \code{alpha}: Cronbach's alpha
}

The returned object includes an attribute \code{"baseline"} containing
the reference \eqn{k} and \eqn{\bar{r}} values.
}
\description{
Computes how Cronbach's alpha changes as a function of either
the average inter-item correlation (\eqn{\bar{r}}) or the number
of items (\eqn{k}), holding the other parameter constant.
}
\details{
The function supports two modes:
\itemize{
\item Empirical: derive \eqn{k} and \eqn{\bar{r}} from a dataset
\item Theoretical: specify \eqn{k} and \eqn{\bar{r}} directly
}
}
\examples{
# Theoretical example

\dontrun{
alpha_sensitivity(k = 6) # produces plot
}

alpha_sensitivity(k = 6, r_bar = 0.4, plot = FALSE)

# Vary number of items
alpha_sensitivity(k = 6, r_bar = 0.4, vary = "k", plot = FALSE)

# Empirical example
df <- data.frame(
  V1 = c(1, 2, 3, 4, 5),
  V2 = c(3, 2, 4, 2, 5),
  V3 = c(2, 1, 5, 4, 3)
)

\dontrun{
alpha_sensitivity(data = df) # produces plot
}

alpha_sensitivity(df, vary = "r_bar", plot = FALSE)

alpha_sensitivity(df, vary = "k", plot = FALSE)

}
\seealso{
\code{\link{alpha}}, \code{\link{reliability}}
}
